// 
// sussen-agent - Main.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Security.Cryptography;
using System.Text;
using Sussen.Agent.Webservice;
using Sussen.Oval;

#if MONO_BUILD
using Mono.Unix;
#endif

namespace Sussen.Agent
{
	public class MainApp
	{
		static bool agentSendResults = false;
	
		static void PrintUsage ()
		{
			StringBuilder usage = new StringBuilder ();
			usage.AppendFormat ("{0} {1}\n", Defines.PACKAGE, Defines.VERSION);
			usage.AppendLine ("Copyright (c) 2004-2006 MMG Security, Inc.");
			usage.AppendLine ();
			usage.AppendFormat ("{0} [OPTIONS] <definitions file>\n\n", Defines.PACKAGE);
			usage.AppendLine (I18N.UsageOptions);
			usage.AppendLine (I18N.UsageAnalyze);
			usage.AppendLine (I18N.UsageCollect);
			usage.AppendLine (I18N.UsageDebug);			
			usage.AppendLine (I18N.UsageHelp);
			usage.AppendLine (I18N.UsageOutput);
			usage.AppendLine (I18N.UsageOutputType);
			usage.AppendLine (I18N.UsageRegisterAgent);
			usage.AppendLine (I18N.UsageScan);								
			usage.AppendLine (I18N.UsageVersion);
			usage.AppendLine (I18N.UsageVerifyMD5);
			usage.AppendLine (I18N.UsageWebCollect);
			usage.AppendLine (I18N.UsageWebScan);			

			Console.WriteLine (usage.ToString());
		}

		static void PrintVersion ()
		{
			StringBuilder version = new StringBuilder ();
			version.AppendFormat ("{0} {1}\n", Defines.PACKAGE, Defines.VERSION);
			version.AppendLine ("Copyright (c) 2004-2006 MMG Security, Inc.");
			version.AppendLine ("This is free software; see the source for copying conditions. There is NO");
			version.AppendLine ("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.");
		
			Console.WriteLine (version.ToString());		
		}

#if MONO_BUILD
		[DllImport("libc")]
		static extern int prctl(int option, byte [] arg2, ulong arg3 , ulong arg4, ulong arg5);
        
       	static void SetProcessName (string name)
    	{
    		try {
            	if(prctl(15 /* PR_SET_NAME */, Encoding.ASCII.GetBytes(name), 0, 0, 0) != 0)
	            	Log.Debug (I18N.ErrorSettingProcessName, Mono.Unix.Native.Stdlib.GetLastError());
	        } catch {}
       	}
#endif

		static void DoWebScan (bool debugMode, bool collectOnly)
		{
			string id = GetAgentID ();		
			string defXml = WebMethods.GetDefinitions ();

#if MONO_BUILD
			Config config = new Config ("unix");
#else
			Config config = new Config ("windows");
#endif
			
			config.XmlDirectory = ConfigurationManager.AppSettings.Get("sussenXmlDirectory");
			config.DownloadDirectory = ConfigurationManager.AppSettings.Get ("sussenDownloadDirectory");

			Repository.Initialize (config.XmlDirectory);
			
			config.DefinitionsXml = defXml;
			config.HostIdentification = id;
			config.ResultsFile = "results.xml";
			config.ResultsFileFormat = ResultOutputTypes.XML;

			if (debugMode)
				config.LoggingLevel = LogLevel.Debug;
			else
				config.LoggingLevel = LogLevel.Error;
			
			try {

				Interpreter i = new Interpreter (config);
				i.CollectData ();
				
				if (!collectOnly)
					i.AnalyzeData ();					
				
				WebMethods.SendSystemCharacteristics (i.SystemData);
			
				if (!collectOnly)
					WebMethods.SendResults (i.Report);
			
			} catch (Exception e) {
				Console.WriteLine (e.ToString());
				Environment.Exit (-1);
			}			
		}

		static string GetAgentID ()
		{
			Assembly asm = Assembly.GetExecutingAssembly ();
			string asm_dir = System.IO.Path.GetDirectoryName (asm.Location);
			string agentIDFile = Path.Combine (asm_dir, "sussen-agent.id");

			if (!File.Exists (agentIDFile))
				throw new ApplicationException (I18N.ErrorOpeningAgentId);

			StreamReader sr = new StreamReader (agentIDFile);
			string sid = sr.ReadLine ();			
			sr.Close ();
			
			return sid;
		}

		static ResultOutputTypes GetFileFormat (string fileFormat)
		{
			if (fileFormat == null)
				throw new ArgumentNullException ("output type");
			
			switch (fileFormat.ToLower()) {
			
			case "html":
				return ResultOutputTypes.HTML; 
				
			case "xml":
				return ResultOutputTypes.XML;
				
			default:
				throw new ArgumentOutOfRangeException ("Invalid output type");
			}
		}

		static string GetDefaultFilename (ResultOutputTypes format)
		{
			string retVal = null;
			
			switch (format) {
			
			case ResultOutputTypes.HTML:
				retVal = "results.html";
				break;
				
			case ResultOutputTypes.XML:
				retVal = "results.xml";
				break;
			}
			
			return retVal;
		}

		static Config GetConfig (string[] args)
		{
			string definitionFile = null;
			string dataFileName = null;
			string resultsFileName = null;
			string resultsFileFormat = null;
			string fileHash = null;		

			bool debugMode = false;
			bool doScan = false;
			bool collectOnly = false;
			bool analyzeOnly = false;
			bool verifyMD5 = false;

			// Parse command-line arguments 			
			for (int i = 0; i < args.Length; i++) {

				string arg = args[i];

				switch (arg) {
				
				case "-a":
				case "--analyze":
					++i;					
					dataFileName = i < args.Length ? args[i] : null;				
					analyzeOnly = true;
					break;

				case "-c":
				case "--collect":
					collectOnly = true;
					break;

				case "-d":
				case "--debug":
					debugMode = true;
					break;
				
				case "-h":
				case "--help":
					PrintUsage ();
					Environment.Exit (0);
					break;

				case "-o":
				case "--output":
					++i;
					resultsFileName = i < args.Length ? args[i] : null;
					break;
					
				case "-ot":
				case "--output-type":
					++i;
					resultsFileFormat = i < args.Length ? args[i] : null;
					break;
					
				case "-r":
				case "--register-agent":
                    WebMethods.RegisterAgent(ConfigurationManager.AppSettings.Get("webserviceWSDL"));
					return null;				

				case "-s":
				case "--scan":
					doScan = true;
					break;

				case "-sr":
				case "--send-results":
					agentSendResults = true;
					break;

				case "-v":
				case "--version":
					PrintVersion ();
					Environment.Exit (0);
					break;

				case "-vm":
				case "--verify-md5":
					++i;
					fileHash = i < args.Length ? args[i] : null;
					verifyMD5 = true;
					break;

				case "-wc":
				case "--web-collect":
					DoWebScan (debugMode, true);
					Environment.Exit (0);
					break;

				case "-ws":
				case "--web-scan":
					DoWebScan (debugMode, false);
					Environment.Exit (0);
					break;
					
				default:
					definitionFile = args[i];
					break;
				}
			}

			if (definitionFile == string.Empty || definitionFile == null) {				
				Console.WriteLine (I18N.ErrorNoDefinitionsFileSpecified);
				PrintUsage ();
				
				Environment.Exit (-1);
			}

#if MONO_BUILD
			Config config = new Config ("unix");
#else
			Config config = new Config ("windows");
#endif

			try {
            	config.XmlDirectory = ConfigurationManager.AppSettings.Get("sussenXmlDirectory");
	            config.DownloadDirectory = ConfigurationManager.AppSettings.Get("sussenDownloadDirectory");
	        } catch {	        	
#if MONO_BUILD
				// Mono 1.1.13.x doesn't like the above two lines. Use old method if it fails.
				config.XmlDirectory = ConfigurationSettings.AppSettings.Get ("sussenXmlDirectory");
				config.DownloadDirectory = ConfigurationSettings.AppSettings.Get ("sussenDownloadDirectory");
#endif	        	
	        }

			if (!Directory.Exists (config.XmlDirectory))
				config.XmlDirectory = Path.GetDirectoryName(Assembly.GetExecutingAssembly().CodeBase);

			Repository.Initialize (config.XmlDirectory);
			config.DefinitionsFile = Repository.GetResource (definitionFile);

			if (agentSendResults) {
				config.ResultsFile = "results.xml";
				config.ResultsFileFormat = ResultOutputTypes.XML;			
			} else {
				config.ResultsFile = "results.html";
				config.ResultsFileFormat = ResultOutputTypes.HTML;
			}

			if (dataFileName != null)
				config.SystemCharacteristicsFile = dataFileName;

			if (analyzeOnly) {			
				config.AnalyzeOnly = true;
				config.CollectOnly = false;
			}
			
			if (collectOnly) {
				config.AnalyzeOnly = false;
				config.CollectOnly = true;			
			}

			if (!doScan && !analyzeOnly && !collectOnly) {
				Console.WriteLine (I18N.ErrorNoOperationSpecified);
				Environment.Exit (0);
			}

			if (verifyMD5) {
				config.DefinitionsFileHash = fileHash;
				config.VerifyMD5 = true;
			}

			if (debugMode)
				config.LoggingLevel = LogLevel.Debug;
			else
				config.LoggingLevel = LogLevel.Error;

			if (resultsFileName != null)
				config.ResultsFile = resultsFileName;

			if (resultsFileFormat != null) {
				config.ResultsFileFormat = GetFileFormat (resultsFileFormat);				
				if (resultsFileName == null)
					config.ResultsFile = GetDefaultFilename (config.ResultsFileFormat);
			}
			
			return config;
		}

		static void Main (string[] args) 
		{
#if MONO_BUILD
			SetProcessName ("sussen-agent");
			Catalog.Init("sussen", Defines.LOCALE_DIR);
#endif

			if (args.Length == 0) {
				PrintUsage ();
				Environment.Exit (0);
			} 
			
			Config config = null;

			try {
			
				config = GetConfig (args);
				if (config == null)
					return;
				
			} catch (Exception e) {
			
				Console.WriteLine (e.ToString());				
				Environment.Exit (-1);
				
			}

			if (!File.Exists (config.DefinitionsFile)) {
				Console.WriteLine (I18N.ErrorDefinitionFileDoesNotExist, config.DefinitionsFile);
				Environment.Exit (-1);
			}

			if (config.AnalyzeOnly && !File.Exists (config.SystemCharacteristicsFile)) {
				Console.WriteLine (I18N.ErrorDataFileDoesNotExist, config.SystemCharacteristicsFile);
				Environment.Exit (-1);			
			}

			try {

				Interpreter i = new Interpreter (config);
				i.CollectData ();
				
				if (!config.CollectOnly)
					i.AnalyzeData ();
			
				if (agentSendResults)
					WebMethods.SendResults (i.Report);
			
			} catch (Exception e) {
				Log.Error (e.Message);
				Log.Debug (e);
				Environment.Exit (-1);
			}
			
		}
	}
}
