// 
// sussen-applet - AppletDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Net;
using System.Security.Cryptography;
using System.Threading;
using Gtk;
using Sussen.Oval;

namespace Sussen.Applet
{
	public enum AppletDialogType { Scan, Analyze };

	public class AppletDialog : Gtk.Dialog
	{
		protected Gtk.Label label3;
		protected Gtk.Entry ovalDefinitionsEntry;
		protected Gtk.Entry ovalResultsEntry;
		protected Gtk.Label label4;
		protected Gtk.Entry systemDataFileEntry;
		protected Gtk.CheckButton saveResultsCheckButton;
		protected Gtk.ProgressBar progressBar;
		protected Gtk.Label progressLabel;
		protected Gtk.Image computerImage;
		protected Gtk.VBox vbox3;
		protected Gtk.HBox hbox6;
		protected Gtk.Button executeButton;
		protected Gtk.Button cancelButton;
		
		AppletDialogType dialogType;
		
		Gtk.ComboBox ovalDefinitionsCombo;
		Hashtable ovalDefinitionsHash;		
	
		bool scanDone;
		bool userCancelled;
		
		uint scanTimer;
		string xmlDirectory;
		string downloadDirectory;
		string scanResultsFilename;
		
		ThreadNotify notify;
		string progressString;		
				
		public AppletDialog(AppletDialogType dialogType)
		{
			Stetic.Gui.Build(this, typeof(Sussen.Applet.AppletDialog));

			ovalDefinitionsHash = new Hashtable ();
			scanDone = false;
			this.dialogType = dialogType;
			
			LoadPreference (Preferences.OVAL_XML_DIRECTORY);
			LoadPreference (Preferences.OVAL_DOWNLOAD_DIRECTORY);			

			ovalResultsEntry.Sensitive = false;
			saveResultsCheckButton.Toggled += new EventHandler (OnSaveResultsToggled);

			ovalDefinitionsCombo = ComboBox.NewText ();			
			string xmldir = (string) Preferences.Get (Preferences.OVAL_XML_DIRECTORY);			
			DirectoryInfo dir = new DirectoryInfo (xmldir);
			if (dir.Exists) {
				foreach (FileInfo f in dir.GetFiles("*.xml")) {
					ovalDefinitionsCombo.AppendText (f.Name);
					ovalDefinitionsHash.Add (f.Name, f.FullName);
				}
			}

			if (ovalDefinitionsHash.Count > 0) {
				label3.Hide ();
				ovalDefinitionsEntry.Hide ();
			}

			ovalDefinitionsCombo.AppendText ("Specify a file or url...");
			ovalDefinitionsCombo.Active = 0;
			ovalDefinitionsCombo.Changed += new EventHandler (OnComboChanged);
			ovalDefinitionsCombo.Show ();
			hbox6.PackStart (ovalDefinitionsCombo, true, true, 0);

			if (dialogType == AppletDialogType.Scan) {			
				computerImage.Pixbuf = Gdk.Pixbuf.LoadFromResource ("computer.png");
				label4.HideAll ();
				systemDataFileEntry.HideAll ();				
			} else {
				computerImage.Pixbuf = Gdk.Pixbuf.LoadFromResource ("edit-find.png");
			}
			
			this.Icon = Gdk.Pixbuf.LoadFromResource ("sussen-icon.png");
			while (!scanDone)
				this.Run ();
				
			this.Destroy ();
		}

		void LoadPreference (String key)
		{
			object val = Preferences.Get (key);

			if (val == null)
				return;

			switch (key) {
			case Preferences.OVAL_XML_DIRECTORY:
				xmlDirectory = (string) val;
				break;

			case Preferences.OVAL_DOWNLOAD_DIRECTORY:
				downloadDirectory = (string) val;
				break;			
			}
		}

		void OnComboChanged (object o, EventArgs args)
		{
			ComboBox combo = o as ComboBox;
			if (o == null)
				return;
				
			TreeIter iter;
			if (combo.GetActiveIter (out iter)) {
				string selection = (string) combo.Model.GetValue (iter, 0);
				
				if (selection == "Specify a file or url...") {
					label3.Show ();
					ovalDefinitionsEntry.Show ();				
				} else {					
					label3.Hide ();
					ovalDefinitionsEntry.Hide ();
				}
			}
		}

		void OnSaveResultsToggled (object o, EventArgs args)
		{
			if (saveResultsCheckButton.Active)
				ovalResultsEntry.Sensitive = true;
			else
				ovalResultsEntry.Sensitive = false;
		}

		protected virtual void OnCancelClicked(object sender, System.EventArgs e)
		{
	 		if (!scanDone && notify != null)
	 			notify.WakeupMain ();
	 		
	 		scanDone = true;
	 		userCancelled = true;		
		}

		void StartScan (Interpreter interpreter)
		{
			try {
		
				progressString = "Collecting data from the system...";
				interpreter.CollectData ();
				
				progressString = "Analyzing system data...";
				interpreter.AnalyzeData ();
			
			} catch (Exception e) {
			
				ShowErrorDialog ("Scan error", e.ToString());
				
			} finally {		
			
				notify.WakeupMain ();
			}
		}
		
		void OnScanCompleted ()
		{
			GLib.Source.Remove (scanTimer);
			notify.Close ();
			
			if (userCancelled) {
				userCancelled = false;
				return;
			}
			
			scanDone = true;
			executeButton.Sensitive = true;

			if (scanResultsFilename != null) {
				UriBuilder ub = new UriBuilder (scanResultsFilename);
				
				try {
					Gnome.Url.Show (ub.ToString());
				} catch (Exception e) {
					ShowErrorDialog ("Error displaying results", e.Message); 
				}
				
			} else
				ShowErrorDialog ("Error displaying results", "No scan results found");
				
			this.HideAll ();
		}
		
		bool progress_timeout ()
		{
			if (scanDone)
				return false;
		
			progressBar.Pulse ();
			progressLabel.Text = progressString;

			return true;
		}
		
		void ShowErrorDialog (string title, string msg)
		{
			HIGMessageDialog dialog = new HIGMessageDialog (
							null,
							0,
							Gtk.MessageType.Error,
							Gtk.ButtonsType.Ok,
							title,
							msg);
						
			dialog.Run ();
			dialog.Destroy ();
		}
			
		static string GetTempFileName ()
		{
			string tmpFile = System.IO.Path.GetTempFileName ();
			File.Delete (tmpFile);
			
			return tmpFile;
		}

		protected virtual void OnExecuteClicked(object sender, System.EventArgs e)
		{
			string defFileName = null;		
		
			TreeIter iter;
			if (ovalDefinitionsCombo.GetActiveIter (out iter)) {
				string selection = (string) ovalDefinitionsCombo.Model.GetValue (iter, 0);
				
				if (selection == "Specify a file or url...")
					defFileName = ovalDefinitionsEntry.Text;
				else
					defFileName = (string) ovalDefinitionsHash [selection]; 
			}
	
			Config config = new Config ("unix");
			config.ResultsFileFormat = ResultOutputTypes.HTML;
			config.XmlDirectory = xmlDirectory;
			config.DownloadDirectory = downloadDirectory;

			Repository.Initialize (config.XmlDirectory);
			config.DefinitionsFile = Repository.GetResource (defFileName);
	
			if (!File.Exists (config.DefinitionsFile)) {
				ShowErrorDialog ("File does not exist",
					String.Format ("ERROR: OVAL definitions file '{0}' does not exist.", config.DefinitionsFile));
				
				return;
			}

			if (dialogType == AppletDialogType.Analyze) {

				config.SystemCharacteristicsFile = systemDataFileEntry.Text;
				config.AnalyzeOnly = true;
			
				if (!File.Exists (config.SystemCharacteristicsFile)) {
					ShowErrorDialog ("File does not exist",
						String.Format ("ERROR: System data file '{0}' does not exist.", config.SystemCharacteristicsFile));

					return;				
				}
				
			} else {
								
				config.SystemCharacteristicsFile = GetTempFileName (); 

			}
			
			if (saveResultsCheckButton.Active) {
			
				if (ovalResultsEntry.Text == "") {

					ShowErrorDialog ("Invalid results file",
						String.Format ("ERROR: results file '{0}' is not valid.", ovalResultsEntry.Text));
						
					return;
				}
				
				string path = System.IO.Path.GetDirectoryName (ovalResultsEntry.Text);
				if (path == "") {
					string desktopDir = Environment.GetFolderPath(Environment.SpecialFolder.Desktop);
					config.ResultsFile = System.IO.Path.Combine (desktopDir, ovalResultsEntry.Text); 
				} else {
					config.ResultsFile = ovalResultsEntry.Text;
				}

			} else {			

				config.ResultsFile = GetTempFileName () + ".html";

			}

			scanResultsFilename = config.ResultsFile;

			Interpreter ovalInterpreter = null;
			
			try {

				ovalInterpreter = new Interpreter (config);
			
			} catch (Exception ex) {
			
				ShowErrorDialog ("Error", ex.Message);
				return;
			}

			executeButton.Sensitive = false;			
		
			scanTimer = GLib.Timeout.Add (100, new GLib.TimeoutHandler (progress_timeout));
		
			notify = new ThreadNotify (new ReadyEvent (OnScanCompleted));
			
			ThreadStart starter = new ThreadStart (delegate() {
				StartScan (ovalInterpreter);			
			});
			new Thread (starter).Start ();		
		}
	}
}
