// 
// sussen-editor - GtkEditorWindow.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 .
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;
using Sussen.Oval;
using Gtk;
using Gnome;

namespace Sussen.Editor.Gui
{
	public class GtkEditorWindow : Gtk.Window
	{
		Gnome.Program program;
		DefinitionsDocument doc;
		
		GtkFileView fileView;
		
		DefinitionView definitionView;
		GtkDefinitionOutputView defOutputView;
		
		TestView testView;
		GtkTestOutputView testOutputView;
		
		bool documentChanged;
		int newCount;
		
		protected Gtk.VPaned vpaned1;
		protected Gtk.HPaned hpaned1;
		protected Gtk.HBox hbox2;
		protected Gtk.VBox vbox4;
		protected Gtk.Notebook outputNotebook;
		protected Gtk.HBox hbox1;
		protected Gtk.HBox hbox3;

		public GtkEditorWindow() : base("")
		{
			program = new Gnome.Program (Defines.PACKAGE, Defines.VERSION, Modules.UI, new string[0]);
			
			Stetic.Gui.Build(this, typeof(Sussen.Editor.Gui.GtkEditorWindow));

			fileView = new GtkFileView ();
			fileView.Sensitive = false;
			fileView.DefinitionCreated += OnDefinitionCreated;
			fileView.ItemSelected += OnItemSelected;
			fileView.TestCreated += OnTestCreated;
			fileView.ShowAll ();
			hbox2.PackStart (fileView, true, true, 0);

			testOutputView = new GtkTestOutputView ();
			testOutputView.Sensitive = false;
			testOutputView.ShowAll ();
			hbox1.PackStart (testOutputView, true, true, 0);		

			defOutputView = new GtkDefinitionOutputView ();
			defOutputView.Sensitive = false;
			defOutputView.ShowAll ();
			hbox3.PackStart (defOutputView, true, true, 0);
						
			this.Icon = new Gdk.Pixbuf (null, "sussen-icon.png");
			this.Title = Defines.PACKAGE;

			// Restore window positions
			LoadPreference (Preferences.MAIN_WINDOW_WIDTH);
			LoadPreference (Preferences.MAIN_WINDOW_X);
			LoadPreference (Preferences.MAIN_WINDOW_MAXIMIZED);
			LoadPreference (Preferences.MAIN_WINDOW_HPANED);
			LoadPreference (Preferences.MAIN_WINDOW_VPANED);

			// Watch for any changes
			Preferences.SettingChanged += OnPreferencesChanged;
		}

		public void Run ()
		{
			program.Run ();
		}

		#region internal_methods

		void Close ()
		{
			int x, y, width, height;
			this.GetPosition (out x, out y);
			this.GetSize (out width, out height);

			bool maximized = ((this.GdkWindow.State & Gdk.WindowState.Maximized) > 0);
			Preferences.Set (Preferences.MAIN_WINDOW_MAXIMIZED, maximized);

			if (!maximized) {
				Preferences.Set (Preferences.MAIN_WINDOW_X, x);
				Preferences.Set (Preferences.MAIN_WINDOW_Y, y);
				Preferences.Set (Preferences.MAIN_WINDOW_WIDTH, width);
				Preferences.Set (Preferences.MAIN_WINDOW_HEIGHT, height);
			}

			Preferences.Set (Preferences.MAIN_WINDOW_HPANED, hpaned1.Position);
			Preferences.Set (Preferences.MAIN_WINDOW_VPANED, vpaned1.Position);
			
			program.Quit ();
		}

		DefinitionsDocument LoadDefinitions (string fileName)
		{
			if (fileName == null || fileName == string.Empty)
				return null;

			if (!File.Exists (fileName))
				return null;

			try {				
			
				DefinitionsDocument doc = new DefinitionsDocument ();
				doc.LoadAll (fileName);
				
				return doc;				
				
			} catch (Exception e) {
				Log.Debug (e);
				
				MessageDialog md = new MessageDialog (this, DialogFlags.DestroyWithParent, 
					MessageType.Error, ButtonsType.Close, e.Message);
						
				md.Run ();
				md.Destroy ();				
			}
			
			return null;
		}

		void LoadPreference (String key)
		{
			object val = Preferences.Get (key);

			if (val == null) {

				if (key == Preferences.MAIN_WINDOW_HPANED)
					hpaned1.Position = 250;
				else if (key == Preferences.MAIN_WINDOW_VPANED)
					vpaned1.Position = 400;

				return;
			}
			
			Log.Debug ("Setting {0} to {1}", key, val);

			switch (key) {
			case Preferences.MAIN_WINDOW_MAXIMIZED:
				if ((bool) val)
					this.Maximize ();
				else
					this.Unmaximize ();
				break;

			case Preferences.MAIN_WINDOW_X:
			case Preferences.MAIN_WINDOW_Y:
				this.Move((int) Preferences.Get(Preferences.MAIN_WINDOW_X), (int) Preferences.Get(Preferences.MAIN_WINDOW_Y));
				break;
			
			case Preferences.MAIN_WINDOW_WIDTH:
			case Preferences.MAIN_WINDOW_HEIGHT:
				this.SetDefaultSize((int) Preferences.Get(Preferences.MAIN_WINDOW_WIDTH),
						(int) Preferences.Get(Preferences.MAIN_WINDOW_HEIGHT));

				this.ReshowWithInitialSize();
				break;

			case Preferences.MAIN_WINDOW_HPANED:
				hpaned1.Position = (int) Preferences.Get (Preferences.MAIN_WINDOW_HPANED);
				break;
				
			case Preferences.MAIN_WINDOW_VPANED:
				vpaned1.Position = (int) Preferences.Get (Preferences.MAIN_WINDOW_VPANED);
				break;
			}
		}

		void RunDefinition (string definitionID)
		{
			if (definitionID == null)
				throw new ArgumentNullException("definitionID");
				
			Match m = Regex.Match (definitionID, Vocabulary.DEFINITION_ID_PATTERN);
			if (!m.Success)
				throw new ArgumentException("Invalid format for definition ID", "definitionID");

			outputNotebook.CurrentPage = 1;							
			defOutputView.Clear ();
			
			DefinitionElement de = doc.Definitions [definitionID];
			CriteriaElement ce = de.Criteria[0];
			List<TestResult> testResults = new List<TestResult> (); 
			
			foreach (string s in ce.Criterions.Keys) {
				TestElement te = doc.Tests [s];
				ObjectElement oe = doc.Objects [te.ObjectRef];
				
				Test test = Interpreter.CreateTest (te.Name, "5.2");
				CollectedItem[] res = test.Run (te, oe);
			
				TestAnalyzer testAnalyzer = new TestAnalyzer (); 
				TestResult testResult = testAnalyzer.Run (te, res, doc);
				testResults.Add (testResult);
				
				string resText = Vocabulary.GetTestResult (testResult);
				defOutputView.AddItem (s, resText);
			}
			
			DefinitionAnalyzer da = new DefinitionAnalyzer ();			
			TestResult defResult = da.Run (ce, testResults);
			
			string defText = Vocabulary.GetTestResult (defResult);
			defOutputView.Result = defText.ToUpper();
		}

		void RunTest (string testID)
		{
			if (testID == null)
				throw new ArgumentNullException("testID");
				
			Match m = Regex.Match (testID, Vocabulary.TEST_ID_PATTERN);
			if (!m.Success)
				throw new ArgumentException("Invalid format for test ID", "id");
		
			outputNotebook.CurrentPage = 0;
			testOutputView.Clear ();
		
			TestElement te = doc.Tests [testID];
			ObjectElement oe = doc.Objects [te.ObjectRef];
			
			Test test = Interpreter.CreateTest (te.Name, "5.2");
			CollectedItem[] res = test.Run (te, oe);
			
			TestAnalyzer testAnalyzer = new TestAnalyzer (); 
			TestResult testResult = testAnalyzer.Run (te, res, doc);			
			string resText = Vocabulary.GetTestResult (testResult);
			testOutputView.Result = resText.ToUpper();
			
			foreach (CollectedItem c in res) {
				foreach (EntityObject e in c.Items) {					
					testOutputView.AddItem (e.Name, e.Value, Vocabulary.GetStatusFlag (e.Status));
				}
			}
		}
			
		void ShowDefinition (string name)
		{
			if (testView != null) {
				testView.Destroy ();
				testView = null;
			}
			
			if (definitionView == null) {
				definitionView = new DefinitionView ();				
				vbox4.PackStart (definitionView, true, true, 5);
			}
				
			if (doc.Definitions.ContainsKey (name)) {
				definitionView.CurrentDefinition = doc.Definitions [name];
				
				List<TestElement> list = new List<TestElement> ();
				foreach (string s in doc.Tests.Keys) {
					TestElement te = doc.Tests[s];
					list.Add (te);
				}
					
				definitionView.CurrentTests = list.ToArray(); 
			}
		}

		void ShowTest (string name)
		{
			if (definitionView != null) {
				definitionView.Destroy ();
				definitionView = null;
			}
			
			if (testView == null) {
				testView = new TestView ();
				testView.TestEdited += (OnTestEdited);
				vbox4.PackStart (testView, true, true, 5);
			}
			
			if (doc.Tests.ContainsKey (name)) {
				TestElement te = doc.Tests [name];
				testView.ShowTest (te, doc.Objects [te.ObjectRef], doc.States [te.StateRef]);
			}
		}

		void SaveDocument ()
		{
			FileChooserDialog fcd = new FileChooserDialog (
				"Save definitions as...",
				Gtk.Stock.Save, 
				this, 
				FileChooserAction.Save);

			fcd.AddButton (Gtk.Stock.Cancel, ResponseType.Cancel);
			fcd.AddButton (Gtk.Stock.Save, ResponseType.Ok);

			FileFilter filter = new FileFilter ();
			filter.Name = "Extensible Markup Language (XML)";
			filter.AddPattern (".xml");
			fcd.AddFilter (filter);

			ResponseType response = (ResponseType) fcd.Run();
			if (response == ResponseType.Ok) {
				doc.Save (fcd.Filename);
				
				documentChanged = false;
				fileView.SetFileDirty (documentChanged);
				fileView.Filename = fcd.Filename;				
			}
		
			fcd.Destroy();				
		}

		void ToggleWidgets (bool state)
		{
			fileView.Sensitive = state;
			testOutputView.Sensitive = state;
			defOutputView.Sensitive = state;		
		}

		#endregion

		protected virtual void OnDefinitionCreated (object sender, DefinitionCreatedEventArgs e)
		{
			newCount++;
					
			string defId = string.Format ("oval:org.example:def:{0}", newCount);
			DefinitionElement de = new DefinitionElement (defId, "1", "vulnerability");
			de.AddCriteria (new CriteriaElement ());
			doc.Definitions.Add (defId, de);

			documentChanged = true;
			
			Log.Debug ("Created definition: {0}", defId);			
			ShowDefinition (defId);
			
			e.Name = defId;
		}

		protected virtual void OnNewClicked(object sender, System.EventArgs e)
		{
			doc = new DefinitionsDocument ();
			fileView.SetOpenFile ("untitled", doc);
			documentChanged = true;

			ToggleWidgets (true);
			
			Log.Debug ("Created new definitions document");
		}

		protected virtual void OnOpenClicked(object sender, System.EventArgs e)
		{
			FileChooserDialog fcd = new FileChooserDialog (
				"Select a definitions file to open",
				Gtk.Stock.Open, 
				this, 
				FileChooserAction.Open);

			fcd.AddButton (Gtk.Stock.Cancel, ResponseType.Cancel);
			fcd.AddButton (Gtk.Stock.Open, ResponseType.Ok);
			fcd.SelectMultiple = false;

			ResponseType response = (ResponseType) fcd.Run();
			if (response == ResponseType.Ok) {
				Log.Debug ("Opening file: {0}", fcd.Filename);				
				doc = LoadDefinitions (fcd.Filename);				
				if (doc != null) {
					fileView.SetOpenFile (fcd.Filename, doc);
					Log.Debug ("Definitions count  : {0}", doc.Definitions.Count);
					Log.Debug ("Tests count        : {0}", doc.Tests.Count);
					ToggleWidgets (true);
				}
			}
		
			fcd.Destroy();		
		}

		void OnPreferencesChanged (object sender, GConf.NotifyEventArgs args)
		{
			LoadPreference (args.Key);
		}

		protected virtual void OnSaveClicked(object sender, System.EventArgs e)
		{
			if (doc == null)
				return;
		
			if (documentChanged)
				doc.Update ();

			if (doc.Filename == "untitled") {
				SaveDocument ();
				return;
			}
			
			doc.Save (doc.Filename);

			documentChanged = false;			
			fileView.SetFileDirty (documentChanged);
		}

		protected virtual void OnSaveAsClicked(object sender, System.EventArgs e)
		{
			if (documentChanged)
				doc.Update ();
			
			SaveDocument ();
		}

		protected void OnTestCreated (object sender, TestCreatedEventArgs e)
		{
			newCount++;
			
			string objName = e.Type.Replace ("_test", "_object");
			string objId = string.Format ("oval:org.example:obj:{0}", newCount);			
			ObjectElement oe = new ObjectElement (objName, objId, "1", "");
			
			string steName = e.Type.Replace ("_test", "_state");
			string steId = string.Format ("oval:org.example:ste:{0}", newCount);
			StateElement se = new StateElement (steName, steId, "1", "");
			
			string testId = string.Format ("oval:org.example:tst:{0}", newCount);
			TestElement te = new TestElement (e.Type, testId, objId, steId);
			te.Comment = "";
			
			doc.Tests.Add (testId, te);
			doc.Objects.Add (objId, oe);
			doc.States.Add (steId, se);
			
			documentChanged = true;
			
			Log.Debug ("Created test: {0}", testId);			
			ShowTest (testId);
						
			e.Name = testId;
		}

		protected void OnTestEdited (object sender, TestEditedEventArgs e)
		{
			if (!documentChanged) {
				documentChanged = true;
				fileView.SetFileDirty (true);
				
				Log.Debug ("Test {0} edited", e.ID);
			}
		}

		protected virtual void OnQuitActivate(object sender, System.EventArgs e)
		{
			Close ();
		}

		protected virtual void OnAboutActivate(object sender, System.EventArgs e)
		{
			string[] author = { "Loren Bandiera" };
			string[] docs = { "" };
			string translators = "";
			string desc = "OVAL definition editor";
			string copy = "Copyright \xa9 2004-2006 MMG Security, Inc.";
			
			Gnome.About ab = new Gnome.About (
				Defines.PACKAGE,
				Defines.VERSION,
				copy,
				desc,
				author,
				docs, 
				translators, 
				new Gdk.Pixbuf (null, "sussen.png"));

			ab.Icon = new Gdk.Pixbuf (null, "sussen-icon.png");
			ab.Show ();		
		}

		protected virtual void OnExecuteClicked(object sender, System.EventArgs e)
		{
			string name = fileView.Selected;
			if (name == null)
				return;
						
			Log.Debug ("Executing item: {0}", name);

			try {

				Match m = Regex.Match (name, Vocabulary.TEST_ID_PATTERN);
				if (m.Success)			
					RunTest (name);
				else
					RunDefinition (name);
					
			} catch (Exception ex) {

				Log.Debug (ex);
				
				MessageDialog md = new MessageDialog (this, DialogFlags.DestroyWithParent, 
					MessageType.Error, ButtonsType.Close, ex.Message);
						
				md.Run ();
				md.Destroy ();				
			}
		}
		
		protected void OnItemSelected (object o, ItemSelectedEventArgs args)
		{
			Log.Debug ("Item selected: {0}", args.Name);
			
			if (!args.IsDefinition)			
				ShowTest (args.Name);
			else
				ShowDefinition (args.Name);
		}

		protected virtual void OnWindowDelete(object o, Gtk.DeleteEventArgs args)
		{
			Close ();
			args.RetVal = true;
		}

	}
}
