// 
// EnvironmentVariableTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;
using Sussen.Oval;

namespace Sussen.Oval.Independent
{
	public class EnvironmentVariableTest : Test
	{	
		public EnvironmentVariableTest () : base ()
		{
			Operations[] ops = { 
				Operations.Equals, 
				Operations.NotEqual, 
				Operations.PatternMatch
			};
		
			this.objectData.Add ("name", ops);
			
			// FIXME: Need to handle any value type (bool,string,etc.)
			this.stateData.Add ("name", ops);			
			this.stateData.Add ("value", ops);
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			if (!objectElement.Entities.ContainsKey ("name"))
				throw new TestException (testElement.ID, "Missing name object from environment variable test");

			EntityObject varName = objectElement.Entities["name"];

			if (varName.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for name object in environment variable test");

			if (varName.VariableReference != null) {
			
				Variable var = Global.Variables [varName.VariableReference];
				foreach (string v in var.ValueArray) {
					string[] names = GetVariableNames (v, varName.Operation);
					if (names.Length == 0) {
						items.Add (CreateNotFound (v));
						itemCount++;
					}
					
					foreach (string n in names) {
						items.Add (CollectData (n));
						itemCount++;
					}				
				}
			
			} else {
			
				string[] names = GetVariableNames (varName.Value, varName.Operation);
				if (names.Length == 0)
					items.Add (CreateNotFound (varName.Value));
				
				foreach (string n in names) {
					items.Add (CollectData (n));
					itemCount++;
				}
			}

			return items.ToArray ();
		}

		string[] GetVariableNames (string name, Operations operation)
		{		
			List<string> items = new List<string> ();

			switch (operation) {
			
			case Operations.Equals:
				items.Add (name);
				break;

			case Operations.NotEqual:
			case Operations.PatternMatch:				
				return FindVariables (name, operation);
				
			default:
				throw new TestException ("", "Invalid operation for name object in environmentvariable test");
			}
			
			return items.ToArray ();
		}

		string[] FindVariables (string varName, Operations operation)
		{
			List<string> items = new List<string> ();

			try {
				
				IDictionary vars = Environment.GetEnvironmentVariables();
				foreach (DictionaryEntry de in vars) {
					string name = (string) de.Key;
					if (CompareItems (varName, name, operation))
						items.Add (name);
				}
			
			} catch (Exception e) {			
				Log.Debug (e);
			}
			
			return items.ToArray ();
		}

		CollectedItem CreateNotFound (string varName)
		{
			CollectedItem item = new CollectedItem ("environmentvariable_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.DoesNotExist);
			item.AddItem (new EntityObject ("name", varName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("value", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			
			return item;
		}

		CollectedItem CollectData (string varName)
		{
			CollectedItem item = new CollectedItem ("environmentvariable_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.Complete);
			
			try {

				string evValue = Environment.GetEnvironmentVariable (varName);
		
				item.AddItem (new EntityObject ("name", varName, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new EntityObject ("value", evValue, OvalDataTypes.StringType, StatusFlags.Exists, false));

			} catch (Exception e) {
				
				item.Flag = Flags.Error;
				item.Message = e.Message;
				item.MessageLevel = MessageLevels.Error;
				
				item.AddItem (new EntityObject ("name", varName, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new EntityObject ("value", "", OvalDataTypes.StringType, StatusFlags.Error, false));
			}			

			return item;
		}
	}
}
