// 
// FileCrawler.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;

namespace Sussen.Oval.Independent
{
	public enum FileRecurseDirection { None, Up, Down };

	public class FileBehaviors
	{
		int maxDepth;
		FileRecurseDirection recurseDirection;
	
		public FileBehaviors (Dictionary<string,string> behaviors)
		{
			if (behaviors == null)
				throw new ArgumentNullException ("behaviors");
				
			if (behaviors.Count == 0)
				SetDefaults ();
			else
				SetValues (behaviors);
		}

		void SetDefaults ()
		{
			maxDepth = 1;
			recurseDirection = FileRecurseDirection.None;
		}
		
		void SetValues (Dictionary<string,string> behaviors)
		{
			string md = behaviors ["max_depth"];
			if (md == null) {
				maxDepth = 1;
			} else {
				try { maxDepth = int.Parse (md); } 
				catch { maxDepth = 1; }
			}
						
			string rd = behaviors ["recurse_direction"];
			if (rd == null) {
				recurseDirection = FileRecurseDirection.None;
			} else {
				switch (rd.ToLower()) {
				case "none":
					recurseDirection = FileRecurseDirection.None;
					break;
					
				case "up":
					recurseDirection = FileRecurseDirection.Up;
					break;
					
				case "down":
					recurseDirection = FileRecurseDirection.Down;
					break;
					
				default:
					recurseDirection = FileRecurseDirection.None;
					break;
				}
			}			
		}
		
		public int MaxDepth
		{
			get { return maxDepth; }
		}
		
		public FileRecurseDirection RecurseDirection
		{
			get { return recurseDirection; } 
		}
	}

	public class FileCrawler
	{
		EntityObject path;
		EntityObject file;
		FileBehaviors behaviors;
	
		List<string> results;
		int currentDepth;
	
		public FileCrawler (EntityObject filePath, EntityObject fileName, FileBehaviors fileBehaviors)
		{
			path = filePath;
			file = fileName;
			behaviors = fileBehaviors;
			
			currentDepth = 1;
			results = new List<string> ();
		}
		
		public void Run ()
		{	
			string[] dirs = GetDirectories (path);
			
			foreach (string d in dirs) {		
				currentDepth = 1;				
				string[] files = GetFiles (file, d, behaviors);				
				foreach (string f in files)
					results.Add (f);
			}
		}
		
		static string[] GetDirectories (EntityObject filePath)
		{
			List<string> items = new List<string> ();

			if (filePath.VariableReference != null) {
			
				Variable var = Global.Variables [filePath.VariableReference];				
				foreach (string v in var.ValueArray) {
					switch (filePath.Operation) {
					
					case Operations.Equals:
						items.Add (v);
						break;
						
					case Operations.NotEqual:
					case Operations.PatternMatch:
						string[] dirs = FindDirectories (v, filePath.Operation);
						foreach (string d in dirs)
							items.Add (d);
						break;
						
					default:
						throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (filePath.Operation));
					}
				}
			
			} else {
			
				switch (filePath.Operation) {
				
				case Operations.Equals:
					items.Add (filePath.Value);
					break;
					
				case Operations.NotEqual:
				case Operations.PatternMatch:
					string[] dirs = FindDirectories (filePath.Value, filePath.Operation);
					foreach (string d in dirs)
						items.Add (d);
					break;
					
				default:
					throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (filePath.Operation));
				}

			}
			
			return items.ToArray ();
		}		
		
		static bool CompareItems (string fileA, string fileB, Operations fileOperation)
		{
			bool retVal = false;
			
			switch (fileOperation) {

			case Operations.Equals:
				if (fileA == fileB)
					retVal = true;
				break;
						
			case Operations.NotEqual:
				if (fileA != fileB)
					retVal = true;
				break;
				
			case Operations.PatternMatch:
				// FIXME: OVAL uses POSIX format for regex; not sure .NET supports that
				Regex r = new Regex (fileA, RegexOptions.None);
				Match m = r.Match (fileB);
				if (m.Success)
					retVal = true;			
				break;
				
			default:
				throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (fileOperation));
			}
			
			return retVal;
		}

		static string[] FindDirectories (string dirPath, Operations dirOperation)
		{
			List<string> items = new List<string> ();
			
			string[] dirs = Directory.GetDirectories ("/");
			foreach (string d in dirs) {
				if (CompareItems (dirPath, d, dirOperation))
					items.Add (d);
					
				// FIXME: not sure if I'm supposed to search all directories recursively
				string[] subDirs = Directory.GetDirectories (d);
				foreach (string d2 in subDirs) {
					string[] tmp = FindDirectories (d2, dirOperation);
					foreach (string t in tmp)
						items.Add (t);
				}
			}
			
			return items.ToArray ();
		}
				
		string[] FindFiles (EntityObject fileName, string filePath, FileBehaviors fileBehaviors)
		{
			List<string> items = new List<string> ();
			
			string[] files = Directory.GetFiles (filePath);
			foreach (string f in files) {
				string tmp = Path.Combine (filePath, fileName.Value);			
				if (CompareItems (tmp, f, fileName.Operation))
					items.Add (Path.Combine (filePath, f));
			}
			
			if (currentDepth < fileBehaviors.MaxDepth || fileBehaviors.MaxDepth == -1) {				
					
				currentDepth++;
				string nextDir = filePath;
				bool moreToGo = true;
			
				if (fileBehaviors.RecurseDirection ==  FileRecurseDirection.Up) {
					DirectoryInfo parent = Directory.GetParent (filePath);
					if (parent != null)
						nextDir = parent.FullName;
					else
						moreToGo = false;
				}

				if (moreToGo) {											
					string[] dirs = Directory.GetDirectories (nextDir);
					foreach (string d in dirs) {
						string[] subFiles = FindFiles (fileName, d, fileBehaviors);
						foreach (string a in subFiles)
							items.Add (a);
					}
				}
			}
			
			return items.ToArray ();
		}
		
		string[] GetFiles (EntityObject fileName, string filePath, FileBehaviors fileBehaviors)
		{
			return FindFiles (fileName, filePath, fileBehaviors);
		}
		
		public string[] FoundFiles
		{
			get { return results.ToArray (); }
		}
	}
}