// 
// FileHashTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Xml;
using Sussen.Oval;

namespace Sussen.Oval.Independent
{
	public class FileHashTest : Test
	{
		public FileHashTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
		
			this.objectData.Add ("path", ops);
			this.objectData.Add ("filename", ops);
			
			this.stateData.Add ("path", ops);
			this.stateData.Add ("filename", ops);
			this.stateData.Add ("md5", ops);
			this.stateData.Add ("sha1", ops);
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			if (!objectElement.Entities.ContainsKey ("path"))
				throw new TestException (testElement.ID, "Invalid filehash test; missing path");
				
			if (!objectElement.Entities.ContainsKey ("filename"))
				throw new TestException (testElement.ID, "Invalid filehash test; missing filename");
				
			EntityObject path = objectElement.Entities["path"];
			EntityObject file = objectElement.Entities["filename"];

			if (path.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for path in filehash test");

			if (file.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for filename in filehash test");

			CheckEntityOperation (path);
			CheckEntityOperation (file);

			// If the nillable attribute is set to true, then the object being specified is the higher 
			// level path.  In this case, the filename element should not be collected or used in analysis.
			if (file.Nil) {
				// FIXME: does that mean md5sum a whole directory? error in schema docs?
		
			} else {
				FileBehaviors fb = new FileBehaviors (objectElement.Behaviors);
				FileCrawler fc = new FileCrawler (path, file, fb);
				fc.Run ();

				if (fc.FoundFiles.Length == 0)
					items.Add (CreateNotFound (path.Value, file.Value));

				foreach (string f in fc.FoundFiles) {
					items.Add (CollectData (f));
					itemCount++;
				}				
			}
			
			return items.ToArray ();
		}

		void CheckEntityOperation (EntityObject obj)
		{
			switch (obj.Operation) {
			case Operations.Equals:
			case Operations.NotEqual:
			case Operations.PatternMatch:
				break;
				
			default:
				throw new TestException ("Invalid operation for " + obj.Name + " object in filehash test");
			}		
		}

		CollectedItem CreateNotFound (string filePath, string fileName)
		{
			CollectedItem item = new CollectedItem ("filehash_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.DoesNotExist);
			item.AddItem (new EntityObject ("path", filePath, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("filename", fileName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("md5", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			
			return item;
		}

		static byte[] ReadFileBytes (string fileName)
		{
			List<byte> fileBytes = new List<byte> ();
		
			try {
								
				FileStream fs = File.OpenRead (fileName);
					
				byte[] buf = new byte[4096];
				int ret = 0;
				
				do {
				
					ret = fs.Read (buf, 0, buf.Length);
					for (int i = 0; i < ret; i++)
						fileBytes.Add (buf[i]);
				
				} while (ret != 0);
				
				fs.Close ();				
			
			} catch (Exception e) {
		
				Log.Debug (e);				
			}
			
			return fileBytes.ToArray();
		}

		string GetFileHash (byte[] fileBytes, HashAlgorithm algorithm)
		{
			string fileHashString = null;
			
			try {
			
				byte[] fileHash = algorithm.ComputeHash (fileBytes);

				fileHashString = BitConverter.ToString(fileHash).Replace("-", "");		
				fileHashString = fileHashString.ToLower ();
				
			} catch (Exception e) {
				Log.Debug (e);
			}

			return fileHashString;
		}

		CollectedItem CollectData (string fullName)
		{
			CollectedItem item = new CollectedItem ("filehash_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.Complete); 

			string filePath, fileName;			
			FileInfo tmpFi = new FileInfo (fullName);
			filePath = tmpFi.Directory.FullName;
			fileName = tmpFi.Name;

			try {

				byte[] fileBytes = ReadFileBytes (fullName);
				string md5Hash = GetFileHash (fileBytes, new MD5CryptoServiceProvider());
				string sha1Hash = GetFileHash (fileBytes, new SHA1Managed());
			
				item.AddItem (new EntityObject ("path", filePath, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new EntityObject ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new EntityObject ("md5", md5Hash, OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("sha1", sha1Hash, OvalDataTypes.StringType, StatusFlags.Exists, false));

			} catch (Exception e) {

				item.Flag = Flags.Error;
				item.Message = e.Message;
				item.MessageLevel = MessageLevels.Error;
				
				item.AddItem (new EntityObject ("path", filePath, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new EntityObject ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new EntityObject ("md5", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("sha1", "", OvalDataTypes.StringType, StatusFlags.Error, false));

			}			
			
			return item;
		}
	}
}
