// 
// TextFileContentTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml;

namespace Sussen.Oval.Independent
{
	public class TextFileContentTest : Test
	{				
		public TextFileContentTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };		
			Operations[] lineOps = { Operations.Equals,	Operations.NotEqual }; 
		
			this.objectData.Add ("path", ops);
			this.objectData.Add ("filename", ops);
			this.objectData.Add ("line", lineOps);
			
			// FIXME: subexpression can be of any type (bool, string, etc.)
			this.stateData.Add ("path", ops);
			this.stateData.Add ("filename", ops);
			this.stateData.Add ("line", lineOps);
			this.stateData.Add ("subexpression", ops);
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			if (!objectElement.Entities.ContainsKey ("path"))
				throw new TestException (testElement.ID, "Invalid textfilecontent test; missing path");
				
			if (!objectElement.Entities.ContainsKey ("filename"))
				throw new TestException (testElement.ID, "Invalid textfilecontent test; missing filename");				

			if (!objectElement.Entities.ContainsKey ("line"))
				throw new TestException (testElement.ID, "Invalid textfilecontent test; missing line");
				
			EntityObject path = objectElement.Entities["path"];
			EntityObject file = objectElement.Entities["filename"];
			EntityObject line = objectElement.Entities["line"];

			if (path.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for path in textfilecontent test");

			if (file.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for filename in textfilecontent test");

			if (line.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for line in textfilecontent test");

			CheckEntityOperation (path);
			CheckEntityOperation (file);

			if (!(line.Operation == Operations.NotEqual || line.Operation == Operations.PatternMatch))
				throw new TestException (testElement.ID, "Invalid operation for line in textfilecontent test");

			FileBehaviors fb = new FileBehaviors (objectElement.Behaviors);
			FileCrawler fc = new FileCrawler (path, file, fb);
			fc.Run ();
			
			if (fc.FoundFiles.Length == 0)
				items.Add (CreateNotFound (path.Value, file.Value, line.Value));
			
			foreach (string f in fc.FoundFiles) {
				items.Add (CollectData (f, line));
				itemCount++;
			}
			
			return items.ToArray ();
		}

		void CheckEntityOperation (EntityObject obj)
		{
			switch (obj.Operation) {
			case Operations.Equals:
			case Operations.NotEqual:
			case Operations.PatternMatch:
				break;
				
			default:
				throw new TestException ("Invalid operation for " + obj.Name + " object in textfilecontent test");
			}		
		}

		CollectedItem CreateNotFound (string path, string file, string line)
		{
			CollectedItem item = new CollectedItem ("textfilecontent_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.DoesNotExist);
			item.AddItem (new EntityObject ("path", path, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("filename", file, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("line", line, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("subexpression", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			
			return item;
		}

		CollectedItem CollectData (string fullName, EntityObject line)
		{
			CollectedItem item = new CollectedItem ("textfilecontent_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.Complete);

			string filePath, fileName;			
			FileInfo tmpFi = new FileInfo (fullName);
			filePath = tmpFi.Directory.FullName;
			fileName = tmpFi.Name;
			
			try {

				StreamReader sr = new StreamReader (fullName);
				string txt = null;

				while ((txt = sr.ReadLine()) != null) {
					if (CompareItems (line.Value, txt, line.Operation)) {
						item.AddItem (new EntityObject ("path", filePath, OvalDataTypes.StringType, StatusFlags.Exists, true));
						item.AddItem (new EntityObject ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Exists, true));
						item.AddItem (new EntityObject ("line", txt, OvalDataTypes.StringType, StatusFlags.Exists, true));
						item.AddItem (new EntityObject ("subexpression", txt, OvalDataTypes.StringType, StatusFlags.Exists, false));
					}
				}

				sr.Close ();

			} catch (Exception e) {

				item.Flag = Flags.Error;
				item.Message = e.Message;
				item.MessageLevel = MessageLevels.Error; 

				item.AddItem (new EntityObject ("path", filePath, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new EntityObject ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new EntityObject ("line", "", OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new EntityObject ("subexpression", "", OvalDataTypes.StringType, StatusFlags.Error, false));
			}
			
			return item;
		}
	}
}
