// 
// XmlFileContentTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml;
using System.Xml.XPath;

namespace Sussen.Oval.Independent
{
	public class XmlFileContentTest : Test
	{	
		public XmlFileContentTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
			Operations[] xops = { Operations.Equals };
		
			this.objectData.Add ("path", ops);
			this.objectData.Add ("filename", ops);
			this.objectData.Add ("xpath", xops);
			
			this.stateData.Add ("path", ops);
			this.stateData.Add ("filename", ops);
			this.stateData.Add ("xpath", xops);
			this.stateData.Add ("value_of", ops);
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			if (!objectElement.Entities.ContainsKey ("path"))
				throw new TestException (testElement.ID, "Invalid xmlfilecontent test; missing path");
				
			if (!objectElement.Entities.ContainsKey ("filename"))
				throw new TestException (testElement.ID, "Invalid xmlfilecontent test; missing filename");				

			if (!objectElement.Entities.ContainsKey ("xpath"))
				throw new TestException (testElement.ID, "Invalid xmlfilecontent test; missing xpath");			
			
			EntityObject path = objectElement.Entities["path"];
			EntityObject file = objectElement.Entities["filename"];
			EntityObject xpath = objectElement.Entities["xpath"];

			if (path.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for path in xmlfilecontent test");

			if (file.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for filename in xmlfilecontent test");

			if (xpath.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for xpath in xmlfilecontent test");

			CheckEntityOperation (path);
			CheckEntityOperation (file);
			
			if (xpath.Operation != Operations.Equals)
				throw new TestException (testElement.ID, "Invalid operation for xpath in xmlfilecontent test");

			FileBehaviors fb = new FileBehaviors (objectElement.Behaviors);
			FileCrawler fc = new FileCrawler (path, file, fb);
			fc.Run ();
			
			if (fc.FoundFiles.Length == 0)
				items.Add (CreateNotFound (path.Value, file.Value, xpath.Value));
			
			foreach (string f in fc.FoundFiles) {
				items.Add (CollectData (f, xpath.Value));
				itemCount++;
			}
			
			return items.ToArray ();
		}

		void CheckEntityOperation (EntityObject obj)
		{
			switch (obj.Operation) {
			case Operations.Equals:
			case Operations.NotEqual:
			case Operations.PatternMatch:
				break;
				
			default:
				throw new TestException ("Invalid operation for " + obj.Name + " object in textfilecontent test");
			}		
		}

		CollectedItem CreateNotFound (string filePath, string fileName, string xpath)
		{
			CollectedItem item = new CollectedItem ("xmlfilecontent_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.DoesNotExist);
			item.AddItem (new EntityObject ("path", filePath, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("filename", fileName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("xpath", xpath, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("value_of", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			
			return item;
		}

		CollectedItem CollectData (string fullName, string fileXpath)
		{
			CollectedItem item = new CollectedItem ("xmlfilecontent_item", itemCount, Vocabulary.INDEPENDENT_NAMESPACE, Flags.Complete);

			string filePath, fileName;			
			FileInfo tmpFi = new FileInfo (fullName);
			filePath = tmpFi.Directory.FullName;
			fileName = tmpFi.Name;
			
			try {

				XPathNavigator nav;
				XPathDocument doc;
				XPathNodeIterator NodeIter;

				doc = new XPathDocument (fullName);
				nav = doc.CreateNavigator();

				NodeIter = nav.Select(fileXpath);
				while (NodeIter.MoveNext()) {
					
					item.AddItem (new EntityObject ("path", filePath, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new EntityObject ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new EntityObject ("xpath", fileXpath, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new EntityObject ("value_of", NodeIter.Current.Value, OvalDataTypes.StringType, StatusFlags.Exists, false));
				}

			} catch (Exception e) {

				item.Flag = Flags.Error;
				item.Message = e.Message;
				item.MessageLevel = MessageLevels.Error;
				
				item.AddItem (new EntityObject ("path", filePath, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new EntityObject ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new EntityObject ("xpath", fileXpath, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new EntityObject ("value_of", "", OvalDataTypes.StringType, StatusFlags.Error, false));
			}
			
			return item;
		}	
	}
}
