//
// Interpreter.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Xml;
using System.Xml.Schema;

using Sussen.Oval.Apache;
using Sussen.Oval.Independent;
using Sussen.Oval.Unix;
using Sussen.Oval.Windows;

namespace Sussen.Oval
{
    public class Interpreter
    {
        Config configuration;
        OvalDocuments docs;

        DataCollector dc;
        DataAnalyzer analyzer;

        public Interpreter()
        {
        }

        public Interpreter(Config config)
        {
            if (config == null)
                throw new InterpreterException("Invalid configuration");

            if ((config.DefinitionsFile == "" || config.DefinitionsFile == null) && config.DefinitionsXml == null)
                throw new InterpreterException("No definitions file specified");

			Log.Initialize(config.LoggingLevel);

            configuration = config;
            docs = new OvalDocuments();

            LoadDefinitions();
        }

        void CollectSystemData()
        {
            DateTime start;
            TimeSpan ts;

            try
            {

                docs.SystemCharacteristics.Create();

                start = DateTime.Now;
                dc = new DataCollector(docs);
                dc.CreateTest = new PlatformCreateTestHandler(CreateTest);
                
                Log.Info("Examining system information");

#if MONO_BUILD
                docs.SystemCharacteristics.WriteSystemInfo(new UnixSystemInfo());
#else
                docs.SystemCharacteristics.WriteSystemInfo(new WindowsSystemInfo());
#endif

				if (configuration.HostIdentification != null)
					docs.SystemCharacteristics.AddHostIdentification (configuration.HostIdentification);

                Log.Info("Collecting data from the system");
                dc.Run();

                ts = DateTime.Now - start;
                Log.Debug("Time taken to collect data: " + ts.ToString());

                Log.Info("Saving data to {0}", configuration.SystemCharacteristicsFile);
                docs.SystemCharacteristics.Save(configuration.SystemCharacteristicsFile);

            }
            catch (UnauthorizedAccessException uae)
            {
                Log.Error(uae);
                throw new InterpreterException(uae.Message);
            }
            catch (DirectoryNotFoundException dnfe)
            {
                Log.Error(dnfe);
                throw new InterpreterException(dnfe.Message);
            }
        }

        void LoadDefinitions()
        {
        	if (configuration.DefinitionsXml == null) {

	            if (configuration.VerifyMD5)
	                if (!Document.CheckFileHash(configuration.DefinitionsFile, configuration.DefinitionsFileHash))
	                    throw new InterpreterException(String.Format("file '{0}' signature is not valid.", configuration.DefinitionsFile));

	            docs.Definitions.Load(configuration.DefinitionsFile);
	            if (!docs.Definitions.CheckSchemaVersion())
	                throw new InterpreterException("Unsupported schema_version");

				string schemaDirectory = Path.Combine(configuration.XmlDirectory, "schema");
	            if (configuration.VerifyXML)
	                if (!docs.Definitions.Validate(configuration.DefinitionsFile, schemaDirectory))
	                    throw new InterpreterException(String.Format("file '{0}' cannot be validated", configuration.DefinitionsFile));
	            
	            Global.CurrentSchemaVersion = docs.Definitions.SchemaVersion;
	            docs.Definitions.CollectVariables (CreateTest);
	            
			} else {
			
				docs.Definitions.LoadXml (configuration.DefinitionsXml);
	            if (!docs.Definitions.CheckSchemaVersion())
	                throw new InterpreterException("Unsupported schema_version");
	            
	            Global.CurrentSchemaVersion = docs.Definitions.SchemaVersion;
	            docs.Definitions.CollectVariables (CreateTest);				
			}
        }

        void LoadSystemData()
        {
        	string schemaDirectory = Path.Combine(configuration.XmlDirectory, "schema");
            if (configuration.VerifyXML)
                if (!docs.SystemCharacteristics.Validate(configuration.SystemCharacteristicsFile, schemaDirectory))
                    throw new InterpreterException(String.Format("ERROR: file '{0}' cannot be validated", configuration.SystemCharacteristicsFile));

            try
            {

                Log.Info("Loading data document '{0}'", configuration.SystemCharacteristicsFile);
                docs.SystemCharacteristics.Load(configuration.SystemCharacteristicsFile);

            }
            catch (XmlException xe)
            {
                Log.Error(xe);
                throw new InterpreterException("Unable to load data document. " + xe.Message);
            }
        }

        public void CollectData()
        {
            if (!configuration.AnalyzeOnly)
                CollectSystemData();
            else
                LoadSystemData();
        }

        public void AnalyzeData()
        {
            DateTime start;
            TimeSpan ts;

            try
            {

                start = DateTime.Now;
                analyzer = new DataAnalyzer(docs);
                Log.Info("Analyzing system data");
                analyzer.Run();

                ts = DateTime.Now - start;
                Log.Debug("Time taken to analyze data: {0}", ts.ToString());

                Log.Info("Saving results to '{0}'", configuration.ResultsFile);
                docs.Results.Save(configuration);

            }
            catch (UnauthorizedAccessException uae)
            {
                Log.Error(uae);
                string msg = String.Format("Unable to save results. {0}", uae.Message);
                throw new InterpreterException(msg);
            }
            catch (DirectoryNotFoundException dnfe)
            {
                Log.Error(dnfe);
                string msg = String.Format("Unable to save results. {0}", dnfe.Message);
                throw new InterpreterException(msg);
            }
        }

        public string AnalyzeDefinition(string xml)
        {
            OvalDocuments od = new OvalDocuments();
            od.Definitions.LoadXml(xml);

            od.SystemCharacteristics.Create();

            dc = new DataCollector(od);
            dc.CreateTest = new PlatformCreateTestHandler(CreateTest);

            Log.Info("Collecting data from the system");

#if MONO_BUILD
            od.SystemCharacteristics.WriteSystemInfo(new UnixSystemInfo());
#else
            od.SystemCharacteristics.WriteSystemInfo(new WindowsSystemInfo());
#endif
            dc.Run();

            Log.Info("Analyzing system data");
            analyzer = new DataAnalyzer(od);
            analyzer.Run();

            return od.Results.ToString();
        }

        public static Test CreateTest(string testName, string testVersion)
        {
            switch (testName)
            {
#if MONO_BUILD
                case "dpkginfo_test":
                    return new DpkgInfoTest();

                case "environmentvariable_test":
                    return new EnvironmentVariableTest();

                case "family_test":
                    return new FamilyTest();

                case "file_test":
                    return new FileTest();

				case "filehash_test":
					return new FileHashTest ();

                case "filemd5_test":
                    return new FileMD5Test();

				case "httpd_test":
					return new HttpdTest ();

                case "inetd_test":
                    return new InetdTest();

                case "interface_test":
                    return new InterfaceTest();

                case "inetlisteningservers_test":
                    return new InetListeningServersTest();

                case "password_test":
                    return new PasswordTest();

                case "process_test":
                    return new ProcessTest();

                case "rpminfo_test":
                    return new RpmInfoTest();

                case "runlevel_test":
                    return new RunLevelTest();

                case "sccs_test":
                    return new SccsTest();

                case "shadow_test":
                    return new ShadowTest();

                case "textfilecontent_test":
                    return new TextFileContentTest();

                case "uname_test":
                    return new UnameTest();

//				case "unknown_test":
//					return new UnknownTest ();

				case "variable_test":
					return new VariableTest ();

                case "version_test":
                    return new VersionTest();

                case "xmlfilecontent_test":
                    return new XmlFileContentTest();
                    
#else

                case "accesstoken_test":
                    return new AccessTokenTest();

                case "activedirectory_test":
                    return new ActiveDirectoryTest();

                case "auditeventpolicy_test":
                    return new AuditEventPolicyTest();

                case "environmentvariable_test":
                    return new EnvironmentVariableTest();

                case "family_test":
                    return new FamilyTest();

                case "file_test":
                	if (testVersion == "5.0")
                    	return new FileTest();
                   	else
                   		return new FileTest51();

				case "filehash_test":
					return new FileHashTet ();

                case "filemd5_test":
                    return new FileMD5Test();

                case "fileauditedpermissions_test":
                    return new FileAuditedPermissionsTest();

                case "fileeffectiverights_test":
                    return new FileEffectiveRightsTest();

                case "group_test":
                    return new GroupTest();

				case "httpd_test":
					return new HttpdTest ();

                case "interface_test":
                    return new InterfaceTest();

                case "lockoutpolicy_test":
                    return new LockoutPolicyTest();

                case "metabase_test":
                    return new MetabaseTest();

                case "passwordpolicy_test":
                    return new PasswordPolicyTest();

                case "port_test":
                    return new PortTest();

                case "process_test":
                    return new ProcessTest();

                case "registry_test":
                    return new RegistryTest();

                case "regkeyauditedpermissions_test":
                    return new RegKeyAuditedPermissionsTest();

                case "regkeyeffectiverights_test":
                    return new RegKeyEffectiveRightsTest();

                case "sid_test":
                    return new SidTest();

                case "textfilecontent_test":
                    return new TextFileContentTest();

//                case "unknown_test":
//                    return new UnknownTest ();

                case "user_test":
                    return new UserTest();

                case "variable_test":
                	return new VariableTest();

                case "version_test":
                    return new VersionTest();

                case "volume_test":
                    return new VolumeTest();

                case "wmi_test":
                    return new WMITest();

                case "xmlfilecontent_test":
                    return new XmlFileContentTest();

#endif
                default:
                	throw new ArgumentOutOfRangeException ("Unsupported test type: " + testName);
            }
        }

		public static string GetSystemInfo ()
		{
#if MONO_BUILD
			UnixSystemInfo platformSystemInfo = new UnixSystemInfo();
#else
			WindowsSystemInfo platformSystemInfo = new WindowsSystemInfo();
#endif

			XmlDocument doc = new XmlDocument ();
			XmlElement system_info = doc.CreateElement ("system_info", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			doc.AppendChild (system_info);
			
			XmlElement os_name = doc.CreateElement ("os_name", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			os_name.InnerText = platformSystemInfo.OsName;
			system_info.AppendChild (os_name);
			
			XmlElement os_version = doc.CreateElement ("os_version", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			os_version.InnerText = platformSystemInfo.OsVersion;
			system_info.AppendChild (os_version);

			XmlElement arch = doc.CreateElement ("architecture", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			arch.InnerText = platformSystemInfo.OsArchitecture;
			system_info.AppendChild (arch);
				
			XmlElement primary_host_name = doc.CreateElement ("primary_host_name", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			primary_host_name.InnerText = platformSystemInfo.PrimaryHostName;
			system_info.AppendChild (primary_host_name);
			
			XmlElement interfaces = doc.CreateElement ("interfaces", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");

			foreach (SystemInterfaceData ifd in platformSystemInfo.Interfaces) {
			
				XmlElement xe = doc.CreateElement ("interface", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
				
				XmlElement name = doc.CreateElement ("interface_name", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
				name.InnerText = ifd.Name;
				xe.AppendChild (name);

				XmlElement ip = doc.CreateElement ("ip_address", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
				ip.InnerText = ifd.IpAddress;
				xe.AppendChild (ip);
				
				XmlElement mac = doc.CreateElement ("mac_address", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
				mac.InnerText = ifd.MacAddress;
				xe.AppendChild (mac);
				
				interfaces.AppendChild (xe);
			}

			system_info.AppendChild (interfaces);
		
			return doc.InnerXml;
		}

        public Config CurrentConfiguration
        {
            get { return configuration; }
            set { configuration = value; }
        }

        public string Report
        {
            get { return docs.Results.ToString(); }
        }
        
		public string SystemData
		{
			get { return docs.SystemCharacteristics.ToString(); }
		}        
    }

    public class InterpreterException : Exception
    {
        public InterpreterException() : base()
        {
        }

        public InterpreterException(string message) : base(message)
        {
        }
    }
}

#if MONO_BUILD
namespace Sussen.Oval.Windows 
{
}
#else
namespace Sussen.Oval.Unix
{
}
#endif
