// 
// DpkgInfoTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	internal struct DpkgEntry
	{
		public string Name;
		public string Arch;
		public string Epoch;
		public string Release;
		public string Version;
		public string Evr;
	}	

	internal class DpkgParser
	{
		List<DpkgEntry> packages;
		
		const string statusFile = "/var/lib/dpkg/status";
		const string hasEpochPattern = @"[0-9]:[0-9\.]+-[A-Za-z0-9_\-\.]*";
		
		char[] versionReleaseSep = {'-'};
		char[] epochVersionSep = {':'};
	
		public DpkgParser ()
		{
			Init (statusFile);
		}
		
		public DpkgParser (string statusFileName)
		{
			Init (statusFileName);
		}

		void Init (string statusFileName)
		{
			packages = new List<DpkgEntry> ();
		
			if (!File.Exists (statusFileName))
				return;

			try {
			
				StreamReader sr = new StreamReader (statusFileName);
				string line = null;
			
				while ((line = sr.ReadLine()) != null) {
					if (line.StartsWith ("Package:"))
						packages.Add (ReadEntry (line, sr));
				}

				sr.Close ();
				
			} catch (Exception e) {
				Log.Debug (e);
			}		
		}

		DpkgEntry ReadEntry (string packageString, StreamReader sr)
		{
			char[] delim = {':'};
			string[] pairs = packageString.Split (delim, 2);		

			DpkgEntry de = new DpkgEntry ();
			de.Name  = pairs[1].Trim();
			de.Epoch = "";
			de.Release = "";
			de.Version = "";
			
			string line = null;
			while ((line = sr.ReadLine()) != null) {
				if (line.Equals ("")) {
					break;
				} else if (line.StartsWith ("Architecture")) {
					pairs = line.Split (delim, 2);
					de.Arch = pairs[1].Trim();
				} else if (line.StartsWith ("Version")) {
					string tmp = line.Remove (0, 8);
					tmp = tmp.Trim ();

					Match m = Regex.Match (tmp, hasEpochPattern);
					if (m.Success) {
						string[] eparts = tmp.Split (epochVersionSep);

						string tmpVersion = eparts[1];
						string[] vparts = tmpVersion.Split (versionReleaseSep);

						de.Epoch = eparts[0];
						de.Version = vparts[0];

						if (vparts.Length == 2)
							de.Release = vparts[1];
						
					} else {
						string[] vparts = tmp.Split (versionReleaseSep);
						de.Version = vparts[0];

						if (vparts.Length == 2)
							de.Release = vparts[1];
					}

					de.Evr = tmp;
				}
			}	
			
			return de;
		}

		public DpkgEntry[] Packages
		{
			get { return packages.ToArray (); }
		}
	}

	internal static class Dpkg
	{
		public static DpkgEntry[] Packages; 
	}

	public class DpkgInfoTest : Test
	{		
		public DpkgInfoTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
			Operations[] nops = { Operations.Equals, Operations.NotEqual, Operations.GreaterThan, Operations.GreaterThanOrEqual, Operations.LessThan, Operations.LessThanOrEqual, Operations.PatternMatch };
		
			this.objectData.Add ("name", ops);
			
			this.stateData.Add ("name", ops);
			this.stateData.Add ("arch", ops);
			this.stateData.Add ("epoch", nops);
			this.stateData.Add ("release", nops);
			this.stateData.Add ("version", nops);
			this.stateData.Add ("evr", nops);
		}

		public DpkgInfoTest (string statusFile) : base ()
		{
			DpkgParser dp = new DpkgParser (statusFile);
			Dpkg.Packages = dp.Packages;
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
					
			if (!objectElement.Entities.ContainsKey ("name"))
				throw new TestException (testElement.ID, "Invalid dpkginfo test");
			
			EntityObject packageName = objectElement.Entities["name"];

			if (packageName.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for name object in dpkginfo test");

			if (Dpkg.Packages == null) {
				DpkgParser dp = new DpkgParser ();
				Dpkg.Packages = dp.Packages;
			}

			if (packageName.VariableReference != null) {
			
				Variable var = Global.Variables [packageName.VariableReference];
				foreach (string v in var.ValueArray) {
					string[] names = GetPackageNames (v, packageName.Operation);
					if (names.Length == 0) {
						items.Add (CreateNotFound (v));
						itemCount++;
					}
					
					foreach (string n in names) {
						items.Add (CollectData (n));
						itemCount++;
					}				
				}
			
			} else {
			
				string[] names = GetPackageNames (packageName.Value, packageName.Operation);
				if (names.Length == 0)
					items.Add (CreateNotFound (packageName.Value));
					
				foreach (string n in names) {
					items.Add (CollectData (n));
					itemCount++;
				}
			}
			
			return items.ToArray ();
		}

		string[] GetPackageNames (string pkgName, Operations operation)
		{		
			List<string> items = new List<string> ();

			switch (operation) {
			
			case Operations.Equals:
				items.Add (pkgName);
				break;

			case Operations.NotEqual:
			case Operations.PatternMatch:				
				return FindPackages (pkgName, operation);
				
			default:
				throw new TestException ("", "Invalid operation for name object in dpkginfo test");
			}
			
			return items.ToArray ();
		}

		string[] FindPackages (string packageName, Operations operation)
		{
			List<string> items = new List<string> ();

			foreach (DpkgEntry de in Dpkg.Packages) 
				if (CompareItems (packageName, de.Name, operation))
					items.Add (de.Name);
			
			return items.ToArray ();
		}

		CollectedItem CreateNotFound (string packageName)
		{
			CollectedItem item = new CollectedItem ("dpkginfo_item", itemCount, Vocabulary.LINUX_NAMESPACE, Flags.DoesNotExist);
			item.AddItem (new EntityObject ("name", packageName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("arch", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("epoch", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("release", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("version", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("evr", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			
			return item;
		}

		CollectedItem CollectData (string packageName)
		{
			CollectedItem item = new CollectedItem ("dpkginfo_item", itemCount, Vocabulary.LINUX_NAMESPACE, Flags.Complete);
			bool foundMatch = false;
		
			foreach (DpkgEntry de in Dpkg.Packages) {
				if (de.Name == packageName) {					
					item.AddItem (new EntityObject ("name", packageName, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new EntityObject ("arch", de.Arch, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("epoch", de.Epoch, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("release", de.Release, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("version", de.Version, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("evr", de.Evr, OvalDataTypes.StringType, StatusFlags.Exists, false));
			
					foundMatch = true;
				}
			}
					
			if (!foundMatch)
				return CreateNotFound (packageName);
			
			return item;
		}		
	}
}
