// 
// FileCrawler.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;

namespace Sussen.Oval.Unix
{
	public class FileCrawler
	{
		EntityObject path;
		EntityObject file;
		FileBehaviors behaviors;
	
		List<string> results;
		int currentDepth;
	
		public FileCrawler (EntityObject filePath, EntityObject fileName, FileBehaviors fileBehaviors)
		{
			path = filePath;
			file = fileName;
			behaviors = fileBehaviors;
			
			currentDepth = 0;
			results = new List<string> ();
		}
		
		public void Run ()
		{	
			string[] dirs = GetDirectories (path); 
			foreach (string d in dirs) {
				if (d == null)
					continue;
					
				currentDepth = 0;
				string[] files = GetFiles (file, d, behaviors);
				foreach (string f in files)
					results.Add (f);
			}
		}
		
		static string[] GetDirectories (EntityObject filePath)
		{
			List<string> items = new List<string> ();

			if (filePath.VariableReference != null) {
			
				Variable var = Global.Variables [filePath.VariableReference];
				foreach (string v in var.ValueArray) {
					switch (filePath.Operation) {
					
					case Operations.Equals:
						items.Add (v);
						break;
						
					case Operations.NotEqual:
					case Operations.PatternMatch:
						string[] dirs = FindDirectories (v, filePath.Operation);
						foreach (string d in dirs)
							items.Add (d);
						break;
						
					default:
						throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (filePath.Operation));
					}
				}
			
			} else {
			
				switch (filePath.Operation) {
				
				case Operations.Equals:
					items.Add (filePath.Value);
					break;
					
				case Operations.NotEqual:
				case Operations.PatternMatch:
					string[] dirs = FindDirectories (filePath.Value, filePath.Operation);
					foreach (string d in dirs)
						items.Add (d);
					break;
					
				default:
					throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (filePath.Operation));
				}

			}
			
			return items.ToArray ();
		}		
		
		static string[] GetFilenames (string filePath, EntityObject fileName, Operations fileOperation)
		{
			List<string> items = new List<string> ();

			if (fileName.VariableReference != null) {
			
				Variable var = Global.Variables [fileName.VariableReference];
				foreach (string v in var.ValueArray) {				
					switch (fileOperation) {
					
					case Operations.Equals:
						items.Add (Path.Combine (filePath, v));
						break;
						
					case Operations.NotEqual:
					case Operations.PatternMatch:
						string[] files = Directory.GetFiles (filePath);
						foreach (string f in files)
							if (CompareItems (v, f, fileName.Operation))
								items.Add (Path.Combine (filePath, f));			
						break;
						
					default:
						throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (fileOperation));
					}				
				}
				
			} else {
			
				switch (fileOperation) {
				
				case Operations.Equals:
					items.Add (Path.Combine (filePath, fileName.Value));
					break;
					
				case Operations.NotEqual:
				case Operations.PatternMatch:
					string[] files = Directory.GetFiles (filePath);
					foreach (string f in files)
						if (CompareItems (fileName.Value, f, fileName.Operation))
							items.Add (Path.Combine (filePath, f));			
					break;
					
				default:
					throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (fileOperation));
				}
			}
						
			return items.ToArray ();
		}
		
		static bool CompareItems (string fileA, string fileB, Operations fileOperation)
		{
			bool retVal = false;
			
			switch (fileOperation) {

			case Operations.Equals:
				if (fileA == fileB)
					retVal = true;
				break;
						
			case Operations.NotEqual:
				if (fileA != fileB)
					retVal = true;
				break;
				
			case Operations.PatternMatch:
				// FIXME: OVAL uses POSIX format for regex; not sure .NET supports that
				Regex r = new Regex (fileA, RegexOptions.None);
				Match m = r.Match (fileB);
				if (m.Success)
					retVal = true;			
				break;
				
			default:
				throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (fileOperation));
			}
			
			return retVal;
		}

		static string[] FindDirectories (string dirPath, Operations dirOperation)
		{
			List<string> items = new List<string> ();
			
			string[] dirs = Directory.GetDirectories ("/");
			foreach (string d in dirs) {
				if (CompareItems (dirPath, d, dirOperation))
					items.Add (d);
					
				// FIXME: not sure if I'm supposed to search all directories recursively
				string[] subDirs = Directory.GetDirectories (d);
				foreach (string d2 in subDirs) {
					string[] tmp = FindDirectories (d2, dirOperation);
					foreach (string t in tmp)
						items.Add (t);
				}
			}
			
			return items.ToArray ();
		}
				
		string[] FindFiles (EntityObject fileName, string filePath, FileBehaviors fileBehaviors)
		{
			List<string> items = new List<string> ();
			
			string[] files = Directory.GetFiles (filePath);
			foreach (string f in files) {
				string tmp = Path.Combine (filePath, fileName.Value);
				if (CompareItems (tmp, f, fileName.Operation))
					items.Add (Path.Combine (filePath, f));
			}
			
			if (fileBehaviors.Recurse == FileRecurse.FileAndDirectories) {
				if (currentDepth < fileBehaviors.MaxDepth || fileBehaviors.MaxDepth == -1) {				
					
					currentDepth++;
					string nextDir = filePath;
					bool moreToGo = true;
				
					if (fileBehaviors.RecurseDirection ==  FileRecurseDirection.Up) {
						DirectoryInfo parent = Directory.GetParent (filePath);
						if (parent != null)
							nextDir = parent.FullName;
						else
							moreToGo = false;
					}

					if (moreToGo) {											
						string[] dirs = Directory.GetDirectories (nextDir);
						foreach (string d in dirs) {
							string[] subFiles = FindFiles (fileName, d, fileBehaviors);
							foreach (string a in subFiles)
								items.Add (a);
						}
					}
				}
			}
			
			return items.ToArray ();
		}
		
		string[] GetFiles (EntityObject fileName, string filePath, FileBehaviors fileBehaviors)
		{
			switch (fileBehaviors.Recurse) {
			
			case FileRecurse.None:
				return GetFilenames (filePath, fileName, fileName.Operation);
				
			case FileRecurse.Directories:	
			case FileRecurse.Files:
			case FileRecurse.FileAndDirectories:
				return FindFiles (fileName, filePath, fileBehaviors);
			}
			
			return null;
		}
		
		public string[] FoundFiles
		{
			get { return results.ToArray (); }
		}
	}
}