// 
// FileTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml;
using Mono.Unix.Native;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public enum FileRecurse { None, Files, Directories, FileAndDirectories };
	public enum FileRecurseDirection { None, Up, Down };
	public enum FileRecurseFilesystem { All, Local, Defined };

	public class FileBehaviors
	{
		int maxDepth;
		FileRecurse recurse;
		FileRecurseDirection recurseDirection;
		FileRecurseFilesystem recurseFilesystem; 
	
		public FileBehaviors (Dictionary<string,string> behaviors)
		{
			if (behaviors == null)
				throw new ArgumentNullException ("behaviors");
				
			if (behaviors.Count == 0)
				SetDefaults ();
			else
				SetValues (behaviors);
		}

		void SetDefaults ()
		{
			maxDepth = -1;
			recurse = FileRecurse.None;
			recurseDirection = FileRecurseDirection.None;
			recurseFilesystem = FileRecurseFilesystem.All;		
		}
		
		void SetValues (Dictionary<string,string> behaviors)
		{
			string md = behaviors ["max_depth"];
			if (md == null) {
				maxDepth = -1;
			} else {
				try { maxDepth = int.Parse (md); } 
				catch { maxDepth = -1; }
			}
			
			string r = behaviors ["recurse"];
			if (r == null) {
				recurse = FileRecurse.None;
			} else {
				switch (r.ToLower()) {
				case "none":
					recurse = FileRecurse.None;
					break;
					
				case "files":
					recurse = FileRecurse.Files;
					break;
					
				case "directories":
					recurse = FileRecurse.Directories;
					break;
					
				case "files and directories":
					recurse = FileRecurse.FileAndDirectories;
					break;
					
				default:
					recurse = FileRecurse.None;
					break;
				}
			}
			
			string rd = behaviors ["recurse_direction"];
			if (rd == null) {
				recurseDirection = FileRecurseDirection.None;
			} else {
				switch (rd.ToLower()) {
				case "none":
					recurseDirection = FileRecurseDirection.None;
					break;
					
				case "up":
					recurseDirection = FileRecurseDirection.Up;
					break;
					
				case "down":
					recurseDirection = FileRecurseDirection.Down;
					break;
					
				default:
					recurseDirection = FileRecurseDirection.None;
					break;
				}
			}
			
			string rf = behaviors ["recurse_file_system"];
			if (rf == null) {
				recurseFilesystem = FileRecurseFilesystem.All;
			} else {
				switch (rf.ToLower()) {
				case "all":
					recurseFilesystem = FileRecurseFilesystem.All;
					break;
					
				case "local":
					recurseFilesystem = FileRecurseFilesystem.Local;
					break;
					
				case "defined":
					recurseFilesystem = FileRecurseFilesystem.Defined;
					break;
					
				default:
					recurseFilesystem = FileRecurseFilesystem.All;
					break;
				}
			}			
		}
		
		public int MaxDepth
		{
			get { return maxDepth; }
		}
		
		public FileRecurse Recurse
		{
			get { return recurse; } 
		}
		
		public FileRecurseDirection RecurseDirection
		{
			get { return recurseDirection; } 
		}
		
		public FileRecurseFilesystem RecurseFilesystem
		{
			get { return recurseFilesystem; } 
		}		
	}

	public class FileTest : Test
	{		
		public FileTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
			Operations[] nops = { Operations.Equals, Operations.NotEqual, Operations.GreaterThan, Operations.GreaterThanOrEqual, Operations.LessThan, Operations.LessThanOrEqual };
			Operations[] bops = { Operations.Equals, Operations.NotEqual };
		
			this.objectData.Add ("path", ops);
			this.objectData.Add ("filename", ops);
			
			this.stateData.Add ("type", ops);
			this.stateData.Add ("user_id", ops);
			this.stateData.Add ("group_id", ops);
			this.stateData.Add ("a_time", ops);
			this.stateData.Add ("c_time", ops);
			this.stateData.Add ("m_time", ops);
			this.stateData.Add ("size", nops);
			
			this.stateData.Add ("suid", bops);
			this.stateData.Add ("sgid", bops);
			this.stateData.Add ("sticky", bops);
			this.stateData.Add ("uread", bops);
			this.stateData.Add ("uwrite", bops);
			this.stateData.Add ("uexec", bops);
			this.stateData.Add ("gread", bops);
			this.stateData.Add ("gwrite", bops);
			this.stateData.Add ("gexec", bops);
			this.stateData.Add ("oread", bops);
			this.stateData.Add ("owrite", bops);
			this.stateData.Add ("oexec", bops);
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			if (!testElement.XmlNs.EndsWith ("unix"))
				throw new NotSupportedException ();
			
			if (!objectElement.Entities.ContainsKey ("path"))
				throw new TestException (testElement.ID, "Invalid file test; missing path");
				
			if (!objectElement.Entities.ContainsKey ("filename"))
				throw new TestException (testElement.ID, "Invalid file test; missing filename");

			EntityObject path = objectElement.Entities["path"];
			EntityObject file = objectElement.Entities["filename"];

			if (path.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for path in file test");

			if (file.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for filename in file test");

			CheckEntityOperation (path);
			CheckEntityOperation (file);

			if (file.Nil || file.Value == "") {

				if (path.VariableReference != null)
                {
                    Variable var = Global.Variables[path.VariableReference];
                    foreach (string v in var.ValueArray)
                        items.Add(CheckPath(v));
                }
                else
                {
                    items.Add(CheckPath(path.Value));
                }
                				
			} else {

				FileBehaviors fb = new FileBehaviors (objectElement.Behaviors);
				FileCrawler fc = new FileCrawler (path, file, fb);
				fc.Run ();

				if (fc.FoundFiles.Length == 0)
					items.Add (CreateNotFound (path.Value, file.Value));

				foreach (string f in fc.FoundFiles) {
					items.Add (CollectData (f));
					itemCount++;
				}
			}

			return items.ToArray ();					
		}

		void CheckEntityOperation (EntityObject obj)
		{
			switch (obj.Operation) {
			case Operations.Equals:
			case Operations.NotEqual:
			case Operations.PatternMatch:
				break;
				
			default:
				throw new TestException ("Invalid operation for " + obj.Name + " object in file test");
			}		
		}

		CollectedItem CheckPath (string pathName)
		{
			CollectedItem item = new CollectedItem("file_item", itemCount, Vocabulary.WINDOWS_NAMESPACE, Flags.Complete);

			if (pathName == "")
			{
				item.Flag = Flags.DoesNotExist;
				item.AddItem(new EntityObject("path", pathName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));

				return item;
			}
		
			DirectoryInfo di = new DirectoryInfo (pathName);
							
			if (di.Exists) {
				item.AddItem (new EntityObject ("path", pathName, OvalDataTypes.StringType, StatusFlags.Exists, true));
			} else {
				item.Flag = Flags.DoesNotExist;
				item.AddItem (new EntityObject ("path", pathName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			}
				
			return item;
		}

		string GetFileType (Mono.Unix.UnixFileInfo fi)
		{
			string fileType = null;
			
			if (fi.IsFifo) {					
				fileType = "fifo";
			} else if (fi.IsCharacterDevice) {
				fileType = "character";
			} else if (fi.IsDirectory) {
				fileType = "directory";
			} else if (fi.IsBlockDevice) {
				fileType = "block special";
			} else if (fi.IsRegularFile) {		
				fileType = "regular";
			} else if (fi.IsSymbolicLink) {
				fileType = "symbolic link";
			} else if (fi.IsSocket) {
				fileType = "socket";
			}
			
			return fileType;
		}

		string GetFilePermission (FilePermissions filePermissions, FilePermissions permission)
		{
			string retVal = null;
			
			if ((filePermissions & permission) == permission)
				retVal = "1";
			else
				retVal = "0";
				
			return retVal;
		}

		CollectedItem CreateNotFound (string filePath, string fileName)
		{
			CollectedItem item = new CollectedItem ("file_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.DoesNotExist);
			item.AddItem (new EntityObject ("path", filePath, OvalDataTypes.StringType, StatusFlags.DoesNotExist , true));
			item.AddItem (new EntityObject ("filename", fileName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				
			item.AddItem (new EntityObject ("type", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("user_id", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("group_id", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("a_time", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("c_time", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("m_time", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("size", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
				
			item.AddItem (new EntityObject ("suid", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("sgid", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("sticky", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("uread", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("uwrite", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("uexec", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("gread", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("gwrite", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("gexec", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("oread", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("owrite", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("oexec", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			
			return item;
		}

		CollectedItem CollectData (string fullName)
		{	
			CollectedItem item = new CollectedItem ("file_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.Complete); 
			
			string filePath, fileName;			
			filePath = Path.GetDirectoryName (fullName);
			fileName = Path.GetFileName (fullName);
			
			if (!File.Exists (fullName)) 
				return CreateNotFound (filePath, fileName);
			
			try {
				
				Mono.Unix.UnixFileInfo fi = new Mono.Unix.UnixFileInfo (fullName);

				item.AddItem (new EntityObject ("path", filePath, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new EntityObject ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new EntityObject ("type", GetFileType (fi), OvalDataTypes.StringType, StatusFlags.Exists, false));

				Stat fileStat = fi.ToStat (); 
				item.AddItem (new EntityObject ("user_id", fi.OwnerUserId.ToString(), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("group_id", fi.OwnerGroupId.ToString(), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("a_time", fileStat.st_atime.ToString(), OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("c_time", fileStat.st_ctime.ToString(), OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("m_time", fileStat.st_mtime.ToString(), OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("size", fi.Length.ToString(), OvalDataTypes.IntegerType, StatusFlags.Exists, false));

				// Get permissions
				FilePermissions fp = fi.Protection;

				// suid, sgid, sticky
				item.AddItem (new EntityObject ("suid", GetFilePermission (fp, FilePermissions.S_ISUID), OvalDataTypes.BooleanType, StatusFlags.Exists, false));			
				item.AddItem (new EntityObject ("sgid", GetFilePermission (fp, FilePermissions.S_ISGID), OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("sticky", GetFilePermission (fp, FilePermissions.S_ISVTX), OvalDataTypes.BooleanType, StatusFlags.Exists, false));
								
				// User perms
				item.AddItem (new EntityObject ("uread", GetFilePermission (fp, FilePermissions.S_IRUSR), OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("uwrite", GetFilePermission (fp, FilePermissions.S_IWUSR), OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("uexec", GetFilePermission (fp, FilePermissions.S_IXUSR), OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				
				// group perms
				item.AddItem (new EntityObject ("gread", GetFilePermission (fp, FilePermissions.S_IRGRP), OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("gwrite", GetFilePermission (fp, FilePermissions.S_IWGRP), OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("gexec", GetFilePermission (fp, FilePermissions.S_IXGRP), OvalDataTypes.BooleanType, StatusFlags.Exists, false));
						
				// other perms
				item.AddItem (new EntityObject ("oread", GetFilePermission (fp, FilePermissions.S_IROTH), OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("owrite", GetFilePermission (fp, FilePermissions.S_IWOTH), OvalDataTypes.BooleanType, StatusFlags.Exists, false));		
				item.AddItem (new EntityObject ("oexec", GetFilePermission (fp, FilePermissions.S_IXOTH), OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				
				return item;
				
			} catch (Exception e) {

				Log.Debug (e);

				item.Flag = Flags.Error;
				item.Message = e.Message;
				item.MessageLevel = MessageLevels.Error;
				
				item.AddItem (new EntityObject ("path", filePath, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new EntityObject ("filename", fileName, OvalDataTypes.StringType, StatusFlags.Error, true));
				
				item.AddItem (new EntityObject ("type", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("user_id", "", OvalDataTypes.IntegerType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("group_id", "", OvalDataTypes.IntegerType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("a_time", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("c_time", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("m_time", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("size", "", OvalDataTypes.IntegerType, StatusFlags.Error, false));
				
				item.AddItem (new EntityObject ("suid", "", OvalDataTypes.BooleanType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("sgid", "", OvalDataTypes.BooleanType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("sticky", "", OvalDataTypes.BooleanType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("uread", "", OvalDataTypes.BooleanType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("uwrite", "", OvalDataTypes.BooleanType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("uexec", "", OvalDataTypes.BooleanType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("gread", "", OvalDataTypes.BooleanType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("gwrite", "", OvalDataTypes.BooleanType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("gexec", "", OvalDataTypes.BooleanType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("oread", "", OvalDataTypes.BooleanType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("owrite", "", OvalDataTypes.BooleanType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("oexec", "", OvalDataTypes.BooleanType, StatusFlags.Error, false));
				
				return item;
			}			
		}
	}
}
