// 
// InetListeningServersTest.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	internal struct NetstatItem
	{
		public string Protocol;
		public string LocalAddress;
		public string LocalPort;
		public string LocalFullAddress;
		public string Program;
		public string ForeignAddress;
		public string ForeignPort;
		public string ForeignFullAddress;
		public string PID;
		public string UID;
	}

	internal class NetstatParser
	{
		List<NetstatItem> netList;

		public NetstatParser ()
		{
			netList = new List<NetstatItem> ();

			string [] paths = new string[] { "/bin/netstat", "/usr/bin/netstat" };
			string line = null;
		
			ProcessStartInfo startInfo = new ProcessStartInfo ();
			startInfo.Arguments = "-p -l -t -u --numeric-ports -e";
		
			startInfo.RedirectStandardOutput = true;
			startInfo.RedirectStandardError = true;
			startInfo.UseShellExecute = false;
		
			foreach (string p in paths) {
				if (!File.Exists (p))
					continue;

				try  {			
					startInfo.FileName = p;
					Process proc = Process.Start (startInfo);

					StreamReader stdout = proc.StandardOutput;
					string output = stdout.ReadToEnd ();

					if (!proc.HasExited) {
						proc.Kill ();
						proc.Close ();
					}

					StringReader sr = new StringReader(output);

					// eat the the header
					sr.ReadLine();
					sr.ReadLine();

					while ((line = sr.ReadLine()) != null) {
						ParseLine (line);
					}

					sr.Close ();

				} catch (Exception e) {
					Log.Debug (e);
				}
			}
		}

		void ParseLine (string line)
		{
			// Proto Recv-Q Send-Q Local Address           Foreign Address         State User       Inode      PID/Program name
			// tcp        0      0 localhost:832           *:*                     LISTEN root       14845      -
			// tcp        0      0 *:389                   *:*                     LISTEN root       11631      -
			// tcp        0      0 *:5900                  *:*                     LISTEN lorenb     16279      10786/vino-server

			Regex r = new Regex (@"(\S+)(\s+)(\S+)");
			string[] split = r.Split (line);
			string[] tmp = null;

			if (split.Length < 17)
				return;

			NetstatItem ni = new NetstatItem ();
			
			tmp = SplitString (split[7], ":");
			ni.LocalAddress = tmp[0];
			ni.LocalPort = tmp[1];

			tmp = SplitString (split[9], ":");
			ni.ForeignAddress = tmp[0];
			ni.ForeignPort = tmp[1];

			ni.UID = split[13];
			ni.Protocol = split[1];
		
			string tmp2 = split[16].Trim();
			if (tmp2 == "" || tmp2 == "-") {
				ni.PID = "-";
				ni.Program = "-";
			} else {
				tmp = SplitString (split[16].Trim(), "/");
				ni.PID = tmp[0];
				ni.Program = tmp[1];
			}

			netList.Add (ni);
		}

		static string[] SplitString (string stringValue, string stringDelimiter)
		{
			string [] split = null;
			split = stringValue.Split (stringDelimiter.ToCharArray());

			return split;
		}

		public NetstatItem[] NetstatInfo
		{
			get { return netList.ToArray (); }
		}
	}

	public class InetListeningServersTest : Test
	{
		NetstatParser np;
		
		public InetListeningServersTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
			Operations[] nops = { Operations.Equals, Operations.NotEqual, Operations.GreaterThan, Operations.GreaterThanOrEqual, Operations.LessThan, Operations.LessThanOrEqual, Operations.PatternMatch };
		
			this.objectData.Add ("protocol", ops);
			this.objectData.Add ("local_address", ops);
			this.objectData.Add ("local_port", ops);
			
			this.stateData.Add ("protocol", ops);
			this.stateData.Add ("local_address", ops);
			this.stateData.Add ("local_port", ops);
			this.stateData.Add ("program_name", ops);
			this.stateData.Add ("pid", nops);
			this.stateData.Add ("local_full_address", ops);
			this.stateData.Add ("foreign_full_address", ops);
			this.stateData.Add ("foreign_address", ops);
			this.stateData.Add ("foreign_port", ops);
			this.stateData.Add ("user_id", ops);	
		}		
		
		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			if (!objectElement.Entities.ContainsKey ("protocol"))
				throw new TestException (testElement.ID, "Invalid inetlisteningservers test; missing protocol");

			if (!objectElement.Entities.ContainsKey ("local_address"))
				throw new TestException (testElement.ID, "Invalid inetlisteningservers test; missing local_address");
				
			if (!objectElement.Entities.ContainsKey ("local_port"))
				throw new TestException (testElement.ID, "Invalid inetlisteningservers test; missing local_port");				
				
			EntityObject protocol = objectElement.Entities["protocol"];
			EntityObject localAddress = objectElement.Entities["local_address"];
			EntityObject localPort = objectElement.Entities["local_port"];

			if (protocol.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for protocol in inetlisteningservers test");

			if (localAddress.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for local_address in inetlisteningservers test");

			if (localPort.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for local_port in inetlisteningservers test");

			CheckEntityOperation (protocol);
			CheckEntityOperation (localAddress);
			CheckEntityOperation (localPort);

			np = new NetstatParser ();			
			NetstatItem[] matchingItems = GetMatchingItems (protocol, localAddress, localPort);
			if (matchingItems.Length == 0)
				items.Add (CreateNotFound (protocol.Value, localAddress.Value, localPort.Value));

			foreach (NetstatItem mi in matchingItems)
				items.Add (CollectData (mi));

			return items.ToArray ();
		}

		void CheckEntityOperation (EntityObject obj)
		{
			switch (obj.Operation) {
			case Operations.Equals:
			case Operations.NotEqual:
			case Operations.PatternMatch:
				break;
				
			default:
				throw new TestException ("Invalid operation for " + obj.Name + " object in inetlisteningservers test");
			}		
		}

		bool IsNetstatItemMatchForEntityObject (string ni, EntityObject eobject)
		{
			if (eobject.VariableReference != null) {

				Variable var = Global.Variables [eobject.VariableReference];
				foreach (string v in var.ValueArray)
					if (CompareItems (v, ni, eobject.Operation))
						return true;
			} else {
			
				if (CompareItems (eobject.Value, ni, eobject.Operation))
					return true;
			}
			
			return false;
		}

		NetstatItem[] GetMatchingItems (EntityObject protocol, EntityObject localAddress, EntityObject localPort)
		{
			List<NetstatItem> items = new List<NetstatItem> (); 
		
			foreach (NetstatItem ni in np.NetstatInfo) {
				
				bool foundMatch = false;
		
				if (IsNetstatItemMatchForEntityObject (ni.Protocol, protocol)) {
					items.Add (ni);
					foundMatch = true;
				}
				
				if (IsNetstatItemMatchForEntityObject (ni.LocalAddress, localAddress) && !foundMatch) {
					items.Add (ni);
					foundMatch = true;
				}
					
				if (IsNetstatItemMatchForEntityObject (ni.LocalPort, localPort) && !foundMatch)
					items.Add (ni);
			}
			
			return items.ToArray ();
		}

		CollectedItem CreateNotFound (string protocol, string localAddress, string localPort)
		{
			CollectedItem item = new CollectedItem ("inetlisteningservers_item", itemCount, Vocabulary.LINUX_NAMESPACE, Flags.DoesNotExist);	
			item.AddItem (new EntityObject ("protocol", protocol, OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("local_address", localAddress, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("local_port", localPort, OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("program_name", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("pid", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("local_full_address", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("foreign_full_address", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("foreign_address", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("foreign_port", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("user_id", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			
			return item;
		}


		CollectedItem CollectData (NetstatItem ni)
		{
			CollectedItem item = new CollectedItem ("inetlisteningservers_item", itemCount, Vocabulary.LINUX_NAMESPACE, Flags.Complete);	

			// FIXME: Collect items: local_full_address, foreign_full_address
			item.AddItem (new EntityObject ("protocol", ni.Protocol, OvalDataTypes.IntegerType, StatusFlags.Exists, true));
			item.AddItem (new EntityObject ("local_address", ni.LocalAddress, OvalDataTypes.StringType, StatusFlags.Exists, true));
			item.AddItem (new EntityObject ("local_port", ni.LocalPort, OvalDataTypes.IntegerType, StatusFlags.Exists, true));
			item.AddItem (new EntityObject ("program_name", ni.Program, OvalDataTypes.StringType, StatusFlags.Exists, false));
			item.AddItem (new EntityObject ("pid", ni.PID, OvalDataTypes.IntegerType, StatusFlags.Exists, false));
			item.AddItem (new EntityObject ("local_full_address", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new EntityObject ("foreign_full_address", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
			item.AddItem (new EntityObject ("foreign_address", ni.ForeignAddress, OvalDataTypes.StringType, StatusFlags.Exists, false));
			item.AddItem (new EntityObject ("foreign_port", ni.ForeignPort, OvalDataTypes.IntegerType, StatusFlags.Exists, false));
			item.AddItem (new EntityObject ("user_id", ni.UID, OvalDataTypes.StringType, StatusFlags.Exists, false));
					
			return item;
		}
	}
}
