// 
// InetdTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	internal struct InetdItem
	{
		public string ServiceName;
		public string EndPointType;
		public string Protocol;
		public string WaitStatus;
		public string UID;
		public string ServerProgram;
		public string ServerArgs;
	}

	internal class InetdParser
	{
		List<InetdItem> services;
		
		public InetdParser (string fileName)
		{
			services = new List<InetdItem> ();
			
			if (!File.Exists (fileName))
				return;
			
			try {
			
				string line = null;
				StreamReader sr = new StreamReader (fileName);
				while ((line = sr.ReadLine()) != null) {
					if (line.StartsWith ("#"))
						continue;

					char[] splitter  = {'\t',' '};
					string[] fields = line.Split (splitter, 7);
					
					InetdItem ii = new InetdItem ();
					ii.ServiceName = fields[0].Trim();
					ii.EndPointType = fields[1].Trim();
					ii.Protocol = fields[2].Trim();
					ii.WaitStatus = fields[3].Trim();
					ii.UID = fields[4].Trim();
					ii.ServerProgram = fields[5].Trim();
					
					if (fields.Length == 7)
						ii.ServerArgs = fields[6].Trim();
						
					services.Add (ii);
				}
				
				sr.Close ();
				
			} catch (Exception e) {
				Log.Debug (e);
			}
		}
		
		public InetdItem[] Services
		{
			get { return services.ToArray (); }  
		}
	}

	public class InetdTest : Test
	{
		InetdParser ip;
		
		public InetdTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
			Operations[] nops = { Operations.Equals, Operations.NotEqual };
		
			this.objectData.Add ("protocol", ops);
			this.objectData.Add ("service_name", ops);
			
			this.stateData.Add ("protocol", ops);
			this.stateData.Add ("service_name", ops);
			this.stateData.Add ("server_program", ops);
			this.stateData.Add ("endpoint_type", nops);
			this.stateData.Add ("exec_as_user", ops);
			this.stateData.Add ("server_arguments", ops);
			this.stateData.Add ("wait_status", nops);
		}		

		public InetdTest (string inetdFileName) : base ()
		{
			ip = new InetdParser (inetdFileName);
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();

			if (ip == null)
				ip = new InetdParser ("/etc/inetd.conf");

			if (!objectElement.Entities.ContainsKey ("protocol"))
				throw new TestException (testElement.ID, "Invalid inetd test; missing protocol");
				
			if (!objectElement.Entities.ContainsKey ("service_name"))
				throw new TestException (testElement.ID, "Invalid inetd test; missing service_name");	
			
			EntityObject protocol = objectElement.Entities["protocol"];
			EntityObject serviceName = objectElement.Entities["service_name"];

			if (protocol.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for protocol in inetd test");

			if (serviceName.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for service_name in inetd test");

			CheckEntityOperation (protocol);
			CheckEntityOperation (serviceName);

			InetdItem[] matchingItems = GetMatchingItems (protocol, serviceName);
			if (matchingItems.Length == 0)
				items.Add (CreateNotFound (protocol.Value, serviceName.Value));
			
			foreach (InetdItem idi in matchingItems)
				items.Add (CollectData (idi));

			return items.ToArray ();
		}

		void CheckEntityOperation (EntityObject obj)
		{
			switch (obj.Operation) {
			case Operations.Equals:
			case Operations.NotEqual:
			case Operations.PatternMatch:
				break;
				
			default:
				throw new TestException ("Invalid operation for " + obj.Name + " object in inetd test");
			}		
		}

		bool IsInetdItemItemMatchForEntityObject (string idi, EntityObject eobject)
		{
			if (eobject.VariableReference != null) {

				Variable var = Global.Variables [eobject.VariableReference];
				foreach (string v in var.ValueArray)
					if (CompareItems (v, idi, eobject.Operation))
						return true;
			} else {
			
				if (CompareItems (eobject.Value, idi, eobject.Operation))
					return true;
			}
			
			return false;
		}

		InetdItem[] GetMatchingItems (EntityObject protocol, EntityObject serviceName)
		{
			List<InetdItem> items = new List<InetdItem> (); 
		
			foreach (InetdItem idi in ip.Services) {
				
				bool foundMatch = false;
		
				if (IsInetdItemItemMatchForEntityObject (idi.Protocol, protocol)) {
					items.Add (idi);
					foundMatch = true;
				}
				
				if (IsInetdItemItemMatchForEntityObject (idi.ServiceName, serviceName) && !foundMatch)
					items.Add (idi);
			}
			
			return items.ToArray ();
		}

		CollectedItem CreateNotFound (string protocol, string serviceName)
		{
			CollectedItem item = new CollectedItem ("inetd_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.DoesNotExist);
			item.AddItem (new EntityObject ("protocol", protocol, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("service_name", serviceName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("server_program", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("endpoint_type", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("exec_as_user", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));			
			item.AddItem (new EntityObject ("server_arguments", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("wait_status", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
					
			return item;		
		}

		CollectedItem CollectData (InetdItem idi)
		{
			CollectedItem item = new CollectedItem ("inetd_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.Complete);
			item.AddItem (new EntityObject ("protocol", idi.Protocol, OvalDataTypes.StringType, StatusFlags.Exists, true));
			item.AddItem (new EntityObject ("service_name", idi.ServiceName, OvalDataTypes.StringType, StatusFlags.Exists, true));
			item.AddItem (new EntityObject ("server_program", idi.ServerProgram, OvalDataTypes.StringType, StatusFlags.Exists, false));
			item.AddItem (new EntityObject ("endpoint_type", idi.EndPointType, OvalDataTypes.StringType, StatusFlags.Exists, false));
			item.AddItem (new EntityObject ("exec_as_user", idi.UID , OvalDataTypes.StringType, StatusFlags.Exists, false));			
			item.AddItem (new EntityObject ("server_arguments", idi.ServerArgs , OvalDataTypes.StringType, StatusFlags.Exists, false));
			item.AddItem (new EntityObject ("wait_status", idi.WaitStatus, OvalDataTypes.StringType, StatusFlags.Exists, false));
					
			return item;		
		}
	}
}
