// 
// InterfaceTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public class InterfaceTest : Test
	{
		public InterfaceTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
		
			this.objectData.Add ("name", ops);
			
			this.stateData.Add ("name", ops);
			this.stateData.Add ("hardware_addr", ops);
			this.stateData.Add ("inet_addr", ops);
			this.stateData.Add ("broadcast_addr", ops);
			this.stateData.Add ("netmask", ops);
			this.stateData.Add ("flag", ops);
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			if (!objectElement.Entities.ContainsKey ("name"))
				throw new TestException (testElement.ID, "Invalid interface test");
			
			EntityObject interfaceName = objectElement.Entities["name"];

			if (interfaceName.DataType != OvalDataTypes.StringType)
				throw new TestException (objectElement.ID, "Invalid datatype for name object in interface test");

			if (interfaceName.VariableReference != null) {
			
				Variable var = Global.Variables [interfaceName.VariableReference];
				foreach (string v in var.ValueArray) {
					string[] names = GetInterfaceNames (v, interfaceName.Operation);
					foreach (string n in names) {
						items.Add (CollectData (n));
						itemCount++;
					}				
				}
			
			} else {
			
				string[] names = GetInterfaceNames (interfaceName.Value, interfaceName.Operation);
				foreach (string n in names) {
					items.Add (CollectData (n));
					itemCount++;
				}
			}
			
			return items.ToArray ();
		}

		string[] GetInterfaceNames (string name, Operations operation)
		{		
			List<string> items = new List<string> ();

			switch (operation) {
			
			case Operations.Equals:
				items.Add (name);
				break;

			case Operations.NotEqual:
			case Operations.PatternMatch:				
				return FindInterfaces (name, operation);
				
			default:
				throw new TestException ("", "Invalid operation for name object in interface test");
			}
			
			return items.ToArray ();
		}

		string[] FindInterfaces (string interfaceName, Operations operation)
		{
			List<string> items = new List<string> ();

			string lst;
			int rc = get_network_interface_names (out lst);

			if (rc == 0) {
				char[] splitter  = {','};
				string[] ifn;
				ifn = lst.Split (splitter);

				foreach (string n in ifn)
					if (CompareItems (interfaceName, n, operation))
						items.Add (n);
			}
			
			return items.ToArray ();
		}

		CollectedItem CollectData (string interfaceName)
		{
			CollectedItem item = new CollectedItem ("interface_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.Complete);
			
			try {

				string hw_addr, inet_addr, bc_addr, netmask;
				short flags;
				
				int rc = get_network_interface_info (interfaceName,	out hw_addr, out inet_addr, out bc_addr, out netmask, out flags);
				if (rc == 0) {

					item.AddItem (new EntityObject ("name", interfaceName, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new EntityObject ("hardware_addr", hw_addr, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("inet_addr", inet_addr, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("broadcast_addr", bc_addr, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("netmask", netmask, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("flag", flags.ToString(), OvalDataTypes.StringType, StatusFlags.Exists, false));
									
				} else {
				
					item.Flag = Flags.DoesNotExist;					
					item.AddItem (new EntityObject ("name", interfaceName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
					item.AddItem (new EntityObject ("hardware_addr", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
					item.AddItem (new EntityObject ("inet_addr", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
					item.AddItem (new EntityObject ("broadcast_addr", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
					item.AddItem (new EntityObject ("netmask", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
					item.AddItem (new EntityObject ("flag", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));					
				}
				
				return item;
				
			} catch (Exception e) {

				item.Flag = Flags.Error;
				item.Message = e.Message;
				item.MessageLevel = MessageLevels.Error;

				item.AddItem (new EntityObject ("name", interfaceName, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new EntityObject ("hardware_addr", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("inet_addr", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("broadcast_addr", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("netmask", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("flag", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				
				return item;
			}			
		}
					
		[DllImport("libsussensupport")]
		internal extern static int get_network_interface_info (string ifname,
			out string hw_addr, out string inet_addr, out string bc_addr,
			out string netmask, out short flags);
			
		[DllImport("libsussensupport")]
		internal extern static int get_network_interface_names (out string csvList);			
	}
}
