// 
// PasswordTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using Mono.Unix;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public class PasswordTest : Test
	{
		public PasswordTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
		
			this.objectData.Add ("username", ops);
			
			this.stateData.Add ("username", ops);
			this.stateData.Add ("password", ops);
			this.stateData.Add ("user_id", ops);
			this.stateData.Add ("group_id", ops);
			this.stateData.Add ("gcos", ops);
			this.stateData.Add ("home_dir", ops);
			this.stateData.Add ("login_shell", ops);
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			if (!objectElement.Entities.ContainsKey ("username"))
				throw new TestException (testElement.ID, "Invalid password test");
				
			EntityObject userName = objectElement.Entities["username"];

			if (userName.DataType != OvalDataTypes.StringType)
				throw new TestException (objectElement.ID, "Invalid datatype for name object in password test");				

			if (userName.VariableReference != null) {
			
				Variable var = Global.Variables [userName.VariableReference];
				foreach (string v in var.ValueArray) {
					string[] names = GetUserNames (v, userName.Operation);
					foreach (string n in names) {
						items.Add (CollectData (n));
						itemCount++;
					}				
				}
			
			} else {
			
				string[] names = GetUserNames (userName.Value, userName.Operation);
				foreach (string n in names) {
					items.Add (CollectData (n));
					itemCount++;
				}
			}
	
			return items.ToArray ();
		}

		string[] GetUserNames (string name, Operations operation)
		{		
			List<string> items = new List<string> ();

			switch (operation) {
			
			case Operations.Equals:
				items.Add (name);
				break;

			case Operations.NotEqual:
			case Operations.PatternMatch:				
				return FindUsers (name, operation);
				
			default:
				throw new TestException ("", "Invalid operation for username object in password test");
			}
			
			return items.ToArray ();
		}

		string[] FindUsers (string userName, Operations operation)
		{
			List<string> items = new List<string> ();

			UnixUserInfo[] userList = UnixUserInfo.GetLocalUsers ();
			foreach (UnixUserInfo u in userList)
				if (CompareItems (userName, u.UserName, operation))
					items.Add (u.UserName);
			
			return items.ToArray ();
		}

		CollectedItem CollectData (string userName)
		{
			CollectedItem item = new CollectedItem ("password_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.Complete);

			try {

				UnixUserInfo ui = new UnixUserInfo (userName);

				item.AddItem (new EntityObject ("username", userName, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new EntityObject ("password", ui.Password, OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("user_id", ui.UserId.ToString(), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("group_id", ui.GroupId.ToString(), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("gcos", ui.RealName, OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("home_dir", ui.HomeDirectory, OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("login_shell", ui.ShellProgram, OvalDataTypes.StringType, StatusFlags.Exists, false));
								
				return item;

			} catch {

				item.Flag = Flags.DoesNotExist;				
				item.AddItem (new EntityObject ("username", userName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new EntityObject ("password", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new EntityObject ("user_id", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
				item.AddItem (new EntityObject ("group_id", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
				item.AddItem (new EntityObject ("gcos", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new EntityObject ("home_dir", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new EntityObject ("login_shell", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				
				return item;
			}						
		}
	}
}
