// 
// ProcessTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	internal struct ProcessItem
	{
		public string Program;
		public string UID;
		public string PID;
		public string PPID;
		public string Start;
		public string TTY;
		public string Etime;
	}

	internal class PsParser
	{
		List<ProcessItem> processList;

		public PsParser ()
		{
			processList = new List<ProcessItem> ();

			string [] paths = new string[] { "/bin/ps", "/usr/bin/ps" };
			string line = null;
		
			ProcessStartInfo startInfo = new ProcessStartInfo ();
			startInfo.Arguments = "-ef";
		
			startInfo.RedirectStandardOutput = true;
			startInfo.RedirectStandardError = true;
			startInfo.UseShellExecute = false;
		
			foreach (string p in paths) {
				if (!File.Exists (p))
					continue;

				try  {
				
					startInfo.FileName = p;
					Process proc = Process.Start (startInfo);
					StreamReader stdout = proc.StandardOutput;

					// eat the ps header
					stdout.ReadLine();

					while ((line = stdout.ReadLine()) != null) {
						ParseLine (line);
					}

					if (!proc.HasExited) {
						proc.Kill ();
						proc.Close ();
					}

				} catch (Exception e) {
					Log.Debug (e);
				}
			}
		}

		void ParseLine (string line)
		{
			// UID        PID  PPID  C STIME TTY          TIME CMD
			// root      4785     1  0 Dec20 ?        00:00:00 /sbin/udevd --daemon

			Regex r = new Regex (@"(\S+)(\s+)(\S+)");
			string[] split = r.Split (line, 8);

			ProcessItem pi = new ProcessItem ();
			pi.UID = split[1];
			pi.PID = split[3];
			pi.PPID = split[5];
			pi.Start = split[9];
			pi.TTY = split[11];
			pi.Etime = split[13];
			pi.Program = split[15];
			
			processList.Add (pi);
		}

		public ProcessItem[] ProcessList
		{
			get { return processList.ToArray (); }
		}
	}

	public class ProcessTest : Test
	{
		PsParser ps;
		
		public ProcessTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
			Operations[] nops = { Operations.Equals, Operations.NotEqual, Operations.GreaterThan, Operations.GreaterThanOrEqual, Operations.LessThan, Operations.LessThanOrEqual };
		
			this.objectData.Add ("command", ops);
			
			this.stateData.Add ("command", ops);
			this.stateData.Add ("scheduling_class", ops);
			this.stateData.Add ("user_id", ops);
			this.stateData.Add ("pid", nops);
			this.stateData.Add ("ppid", nops);
			this.stateData.Add ("priority", ops);
			this.stateData.Add ("start_time", ops);
			this.stateData.Add ("tty", ops);
			this.stateData.Add ("exec_time", ops);
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			if (!objectElement.Entities.ContainsKey ("command"))
				throw new TestException (testElement.ID, "Invalid process test");
			
			EntityObject cmd = objectElement.Entities["command"];

			if (cmd.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid command datatype for process test");

			ps = new PsParser ();

			if (cmd.VariableReference != null) {
			
				Variable var = Global.Variables [cmd.VariableReference];
				foreach (string v in var.ValueArray) {
					string[] names = GetProcessNames (v, cmd.Operation);
					if (names.Length == 0) {
						items.Add (CreateNotFound (v));
						itemCount++;
					}						
					
					foreach (string n in names) {
						items.Add (CollectData (n));
						itemCount++;
					}				
				}
			
			} else {
			
				string[] names = GetProcessNames (cmd.Value, cmd.Operation);
				if (names.Length == 0)
					items.Add (CreateNotFound (cmd.Value));
					
				foreach (string n in names) {
					items.Add (CollectData (n));
					itemCount++;
				}
			}
			
			return items.ToArray ();
		}

		string[] GetProcessNames (string name, Operations operation)
		{		
			List<string> items = new List<string> ();

			switch (operation) {
			
			case Operations.Equals:
				items.Add (name);
				break;

			case Operations.NotEqual:
			case Operations.PatternMatch:				
				return FindProcesses (name, operation);
				
			default:
				throw new TestException ("", "Invalid operation for command object in process test");
			}
			
			return items.ToArray ();
		}

		string[] FindProcesses (string cmd, Operations operation)
		{
			List<string> items = new List<string> ();

			foreach (ProcessItem pi in ps.ProcessList) 
				if (CompareItems (cmd, pi.Program, operation))
					items.Add (pi.Program);
			
			return items.ToArray ();
		}

		CollectedItem CreateNotFound (string cmd)
		{
			CollectedItem item = new CollectedItem ("process_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.DoesNotExist);
			item.AddItem (new EntityObject ("command", cmd, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("scheduling_class", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("user_id", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("pid", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("ppid", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("priority", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("start_time", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("tty", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("exec_time", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			
			return item;
		}

		CollectedItem CollectData (string cmd)
		{
			CollectedItem item = new CollectedItem ("process_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.Complete);

			foreach (ProcessItem pi in ps.ProcessList) {
				if (cmd == pi.Program) {
					// FIXME: get scheduling_class and priority data items
					item.AddItem (new EntityObject ("command", pi.Program, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new EntityObject ("scheduling_class", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
					item.AddItem (new EntityObject ("user_id", pi.UID, OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("pid", pi.PID, OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("ppid", pi.PPID, OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("priority", "", OvalDataTypes.StringType, StatusFlags.NotCollected, false));
					item.AddItem (new EntityObject ("start_time", pi.Start, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("tty", pi.TTY, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("exec_time", pi.Etime, OvalDataTypes.StringType, StatusFlags.Exists, false));
				}
			}
			
			return item;
		}
	}
}
