// 
// RpmInfoTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using System.Xml;
using Mono.Unix;
using Mono.Unix.Native;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public class RpmInfoTest : Test
	{
		public RpmInfoTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
			Operations[] nops = { Operations.Equals, Operations.NotEqual, Operations.GreaterThan, Operations.GreaterThanOrEqual, Operations.LessThan, Operations.LessThanOrEqual, Operations.PatternMatch };
		
			this.objectData.Add ("name", ops);
			
			this.stateData.Add ("name", ops);
			this.stateData.Add ("arch", ops);
			this.stateData.Add ("epoch", nops);
			this.stateData.Add ("release", nops);
			this.stateData.Add ("version", nops);
			this.stateData.Add ("evr", nops);
			this.stateData.Add ("signature_keyid", ops);
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();

			if (!objectElement.Entities.ContainsKey ("name"))
				throw new TestException (testElement.ID, "Invalid rpminfo test");
				
			EntityObject rpmName = objectElement.Entities["name"];

			if (rpmName.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for name in rpminfo test");

			if (rpmName.VariableReference != null) {
			
				Variable var = Global.Variables [rpmName.VariableReference];
				foreach (string v in var.ValueArray) {
					string[] names = GetPackageNames (v, rpmName.Operation);
					foreach (string n in names) {
						items.Add (CollectData (n));
						itemCount++;
					}				
				}
			
			} else {
			
				string[] names = GetPackageNames (rpmName.Value, rpmName.Operation);
				foreach (string n in names) {
					items.Add (CollectData (n));
					itemCount++;
				}
			}
		
			return items.ToArray ();
		}

		string[] GetPackageNames (string pkgName, Operations operation)
		{		
			List<string> items = new List<string> ();

			switch (operation) {
			
			case Operations.Equals:
				items.Add (pkgName);
				break;

			case Operations.NotEqual:
			case Operations.PatternMatch:				
				return FindPackages (pkgName, operation);
				
			default:
				throw new TestException ("", "Invalid operation for name object in rpminfo test");
			}
			
			return items.ToArray ();
		}

		string[] FindPackages (string packageName, Operations operation)
		{
			List<string> items = new List<string>();

			// FIXME: Find better way to get list of package names.
			string tmp = Path.GetTempFileName ();
			rpm_dump_package_names (tmp);

			try {
							
				StreamReader sr = new StreamReader (tmp);
				string line = null;
			
				while ((line = sr.ReadLine()) != null) {
					if (CompareItems (packageName, line, operation))
						items.Add (line);
				}

				sr.Close ();
				
			} catch (Exception e) {
				Log.Debug (e);
			}

			File.Delete (tmp);

			return items.ToArray ();
		}

		string ParseKeyID (string keyString)
		{
			// Example format
			// "DSA/SHA1, Mon Mar  6 15:37:31 2006, Key ID b44269d04f2a6fd2"
		
			string retVal = null;

			if (keyString == null || keyString == string.Empty)
				return retVal;

			string name = "Key ID";
			string[] split = keyString.Split (new char[] {','});

			foreach (string s in split) {
				string t = s.Trim ();
				if (t.StartsWith (name)) {
					string k = s.Remove (1, name.Length);
					retVal = k.Trim();
				}
			}

			return retVal;
		}

		CollectedItem CollectData (string rpmName)
		{
			CollectedItem item = new CollectedItem ("rpminfo_item", itemCount, Vocabulary.LINUX_NAMESPACE, Flags.NotCollected);			
			_RpmInfo ri;
			
			int rc = rpm_get_info (rpmName, out ri);

			if (rc >= 0) {		

				string epoch = UnixMarshal.PtrToString (ri.rpm_epoch);
				string release = UnixMarshal.PtrToString (ri.rpm_release);
				string version = UnixMarshal.PtrToString (ri.rpm_version);
				string keystring = UnixMarshal.PtrToString (ri.rpm_keyid);

				item.Flag = Flags.Complete;
				item.AddItem (new EntityObject ("name", rpmName, OvalDataTypes.StringType, StatusFlags.Exists, true));
				item.AddItem (new EntityObject ("arch", UnixMarshal.PtrToString (ri.rpm_arch), OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("epoch", epoch, OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("release", release, OvalDataTypes.StringType, StatusFlags.Exists, false));
				item.AddItem (new EntityObject ("version", version, OvalDataTypes.StringType, StatusFlags.Exists, false));
				
				string evr = "";
				if (epoch == "-1")
					evr = String.Format ("{0}-{1}", version, release);
				else
					evr = String.Format ("{0}:{1}-{2}", epoch, version, release);
				
				item.AddItem (new EntityObject ("evr", evr, OvalDataTypes.StringType, StatusFlags.Exists, false));
				
				string keyid = ParseKeyID (keystring);
				if (keyid != null)
					item.AddItem (new EntityObject ("signature_keyid", keyid, OvalDataTypes.StringType, StatusFlags.Exists, false));
				else
					item.AddItem (new EntityObject ("signature_keyid", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
										
			} else {

				item.Flag = Flags.DoesNotExist;
				item.AddItem (new EntityObject ("name", rpmName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new EntityObject ("arch", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new EntityObject ("epoch", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new EntityObject ("release", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new EntityObject ("version", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));					
				item.AddItem (new EntityObject ("evr", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				item.AddItem (new EntityObject ("signature_keyid", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
				
			}
			
			return item;
		}
		
		struct _RpmInfo {
			public IntPtr rpm_name;
			public IntPtr rpm_epoch;
			public IntPtr rpm_version;
			public IntPtr rpm_release;
			public IntPtr rpm_arch;
			public IntPtr rpm_keyid;	
		}
		
		[DllImport("libsussensupport")]
		static extern int rpm_get_info (string packageName, out _RpmInfo ri);
		
		[DllImport("libsussensupport")]
		static extern int rpm_dump_package_names (string fileName);
	}
}
