// 
// RunLevelTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public class RunLevelTest : Test
	{
		public RunLevelTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
			Operations[] bops = { Operations.Equals, Operations.NotEqual };
		
			this.objectData.Add ("service_name", ops);
			
			this.stateData.Add ("service_name", ops);
			this.stateData.Add ("runlevel", ops);
			this.stateData.Add ("start", bops);
			this.stateData.Add ("kill", bops);
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();
			
			if (!objectElement.Entities.ContainsKey ("service_name"))
				throw new TestException (testElement.ID, "Invalid runlevel test; missing service_name");
				
			if (!objectElement.Entities.ContainsKey ("runlevel"))
				throw new TestException (testElement.ID, "Invalid runlevel test; missing runlevel");
			
			EntityObject serviceName = objectElement.Entities["service_name"];
			EntityObject runlevel = objectElement.Entities["runlevel"];

			if (serviceName.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid service_name datatype for runlevel test");
				
			if (runlevel.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid runlevel datatype for runlevel test");

			string[] snames = GetServiceNames (serviceName);
			string[] runlevels = GetRunLevels (runlevel);

			foreach (string s in snames) {
				foreach (string r in runlevels) {
					items.Add (CollectData (s, r));
				}
			}
					
			return items.ToArray ();
		}
		
		static string[] GetServiceNames (EntityObject serviceName)
		{
			List<string> items = new List<string> ();

			if (serviceName.VariableReference != null) {
			
				Variable var = Global.Variables [serviceName.VariableReference];
				foreach (string v in var.ValueArray) {				
					switch (serviceName.Operation) {
					
					case Operations.Equals:
						items.Add (v);
						break;

					case Operations.NotEqual:
					case Operations.PatternMatch:
						string[] dirs = FindServiceNames (v, serviceName.Operation);
						foreach (string d in dirs)
							items.Add (d);
						break;
						
					default:
						throw new TestException ("", Vocabulary.GetOperation (serviceName.Operation));
					}				
				}
			
			} else {
			
				switch (serviceName.Operation) {
				
				case Operations.Equals:
					items.Add (serviceName.Value);
					break;

				case Operations.NotEqual:
				case Operations.PatternMatch:
					string[] dirs = FindServiceNames (serviceName.Value, serviceName.Operation);
					foreach (string d in dirs)
						items.Add (d);
					break;
					
				default:
					throw new TestException ("", Vocabulary.GetOperation (serviceName.Operation));
				}
			
			}
			
			return items.ToArray ();
		}

		static string[] GetRunLevels (EntityObject runLevel)
		{
			List<string> items = new List<string> ();

			if (runLevel.VariableReference != null) {
			
				Variable var = Global.Variables [runLevel.VariableReference];
				foreach (string v in var.ValueArray) {
				
					switch (runLevel.Operation) {
					case Operations.Equals:
						items.Add (v);
						break;
						
					case Operations.NotEqual:
					case Operations.PatternMatch:
						string[] dirs = FindRunLevels (v, runLevel.Operation);
						foreach (string d in dirs)
							items.Add (d);
						break;
						
					default:
						throw new TestException ("", "Invalid operation for runlevel");
					}			
				}
				
			} else {
			
				switch (runLevel.Operation) {
				
				case Operations.Equals:
					items.Add (runLevel.Value);
					break;
					
				case Operations.NotEqual:
				case Operations.PatternMatch:
					string[] dirs = FindRunLevels (runLevel.Value, runLevel.Operation);
					foreach (string d in dirs)
						items.Add (d);
					break;
					
				default:
					throw new TestException ("", "Invalid operation for runlevel");
				}			
			
			}
			
			return items.ToArray ();
		}

		static string[] FindServiceNames (string serviceName, Operations serviceOperation)
		{
			List<string> items = new List<string> ();

			string filePath = "/etc/init.d";
			string[] files = Directory.GetFiles (filePath);
			foreach (string f in files)
				if (CompareItems (serviceName, f, serviceOperation))
					items.Add (f);
			
			return items.ToArray ();
		}

		static string[] FindRunLevels (string runLevel, Operations runLevelOperation)
		{
			List<string> items = new List<string> ();

			string filePath = "/etc";
			string[] dirs = Directory.GetDirectories (filePath);
			foreach (string d in dirs)
				if (CompareItems (runLevel, d, runLevelOperation))
					items.Add (d);
			
			return items.ToArray ();
		}
		
		CollectedItem CollectData (string serviceName, string runlevel)
		{
			CollectedItem item = new CollectedItem ("runlevel_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.NotCollected); 

			StringBuilder ip = new StringBuilder ();
			ip.AppendFormat ("/etc/init.d/{0}", serviceName);

			if (!File.Exists (ip.ToString())) {
				item.Flag = Flags.DoesNotExist;
				item.AddItem (new EntityObject ("service_name", serviceName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new EntityObject ("runlevel", runlevel, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new EntityObject ("start", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
				item.AddItem (new EntityObject ("kill", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
				
				return item;
			}

			StringBuilder rcp = new StringBuilder ();
			bool runlevelValid = false;

			DirectoryInfo dir = new DirectoryInfo (runlevel);
			if (dir.Exists)
				runlevelValid = true;				
			else
				rcp.AppendFormat ("/etc/rc{0}.d", runlevel);

			if (!runlevelValid) {
				dir = new DirectoryInfo (rcp.ToString());
				if (!dir.Exists) {
					item.Flag = Flags.DoesNotExist;
					item.AddItem (new EntityObject ("service_name", serviceName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
					item.AddItem (new EntityObject ("runlevel", runlevel, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
					item.AddItem (new EntityObject ("start", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
					item.AddItem (new EntityObject ("kill", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
					
					return item;
				}
			}

			bool allGood = false;

			foreach (FileInfo f in dir.GetFiles("*")) {
				string exp = "[SK][0-9][0-9]" + serviceName;
				Regex r = new Regex (exp);
				Match m = r.Match (f.Name);

				if (m.Success) {

					item.Flag = Flags.Complete;
					item.AddItem (new EntityObject ("service_name", serviceName, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new EntityObject ("runlevel", runlevel, OvalDataTypes.StringType, StatusFlags.Exists, true));
					
					if (f.Name.StartsWith ("S")) {
						item.AddItem (new EntityObject ("start", "1", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
						item.AddItem (new EntityObject ("kill", "0", OvalDataTypes.BooleanType, StatusFlags.Exists, false));					
					} else {
						item.AddItem (new EntityObject ("start", "0", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
						item.AddItem (new EntityObject ("kill", "1", OvalDataTypes.BooleanType, StatusFlags.Exists, false));					
					}

					allGood = true;
				}
			}

			if (!allGood) {
				item.Flag = Flags.DoesNotExist;
				item.AddItem (new EntityObject ("service_name", serviceName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new EntityObject ("runlevel", runlevel, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
				item.AddItem (new EntityObject ("start", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
				item.AddItem (new EntityObject ("kill", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			}			
			
			return item;
		}
	}
}
