// 
// ShadowTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;
using Mono.Unix;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	public class ShadowTest : Test
	{		
		public ShadowTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
		
			this.objectData.Add ("username", ops);
			
			this.stateData.Add ("username", ops);
			this.stateData.Add ("password", ops);
			this.stateData.Add ("chg_lst", ops);
			this.stateData.Add ("chg_allow", ops);
			this.stateData.Add ("chg_req", ops);
			this.stateData.Add ("exp_warn", ops);
			this.stateData.Add ("exp_inact", ops);
			this.stateData.Add ("exp_date", ops);
			this.stateData.Add ("flag", ops);		
		}

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();

			if (!objectElement.Entities.ContainsKey ("username"))
				throw new TestException (testElement.ID, "Invalid shadow test");
			
			EntityObject userName = objectElement.Entities["username"];

			if (userName.DataType != OvalDataTypes.StringType)
				throw new TestException (objectElement.ID, "Invalid datatype for name object in shadow test");				

			if (userName.VariableReference != null) {
			
				Variable var = Global.Variables [userName.VariableReference];
				foreach (string v in var.ValueArray) {
					string[] names = GetUserNames (v, userName.Operation);
					foreach (string n in names) {
						items.Add (CollectData (n));
						itemCount++;
					}				
				}
			
			} else {
			
				string[] names = GetUserNames (userName.Value, userName.Operation);
				foreach (string n in names) {
					items.Add (CollectData (n));
					itemCount++;
				}
			}
	
			return items.ToArray ();
		}

		string[] GetUserNames (string name, Operations operation)
		{		
			List<string> items = new List<string> ();

			switch (operation) {
			
			case Operations.Equals:
				items.Add (name);
				break;

			case Operations.NotEqual:
			case Operations.PatternMatch:				
				return FindUsers (name, operation);
				
			default:
				throw new TestException ("", "Invalid operation for username object in shadow test");
			}
			
			return items.ToArray ();
		}

		string[] FindUsers (string userName, Operations operation)
		{
			List<string> items = new List<string> ();

			UnixUserInfo[] userList = UnixUserInfo.GetLocalUsers ();
			foreach (UnixUserInfo u in userList)
				if (CompareItems (userName, u.UserName, operation))
					items.Add (u.UserName);
			
			return items.ToArray ();
		}

		CollectedItem CollectData (string userName)
		{
			CollectedItem item = new CollectedItem ("shadow_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.Complete);
			
			try {

				string account, password;
				long lstchg, min, max, warn, inact, expire, flag;
				
				int rc = shadow_helper_getspnam (userName, out account, out password, out lstchg, out min, out max,
					out warn, out inact, out expire, out flag);

				if (rc == 0) {

					item.AddItem (new EntityObject ("username", userName, OvalDataTypes.StringType, StatusFlags.Exists, true));
					item.AddItem (new EntityObject ("password", password, OvalDataTypes.StringType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("chg_lst", lstchg.ToString(), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("chg_allow", min.ToString (), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("chg_req", max.ToString (), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("exp_warn", warn.ToString (), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("exp_inact", inact.ToString (), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("exp_date", expire.ToString (), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
					item.AddItem (new EntityObject ("flag", flag.ToString (), OvalDataTypes.IntegerType, StatusFlags.Exists, false));
										
				} else {

					item.Flag = Flags.DoesNotExist;
					
					item.AddItem (new EntityObject ("username", userName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
					item.AddItem (new EntityObject ("password", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
					item.AddItem (new EntityObject ("chg_lst", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
					item.AddItem (new EntityObject ("chg_allow", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
					item.AddItem (new EntityObject ("chg_req", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
					item.AddItem (new EntityObject ("exp_warn", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
					item.AddItem (new EntityObject ("exp_inact", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
					item.AddItem (new EntityObject ("exp_date", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
					item.AddItem (new EntityObject ("flag", "", OvalDataTypes.IntegerType, StatusFlags.DoesNotExist, false));
									
				}

				return item;

			} catch (Exception e) {

				item.Flag = Flags.Error;
				item.Message = e.Message;
				item.MessageLevel = MessageLevels.Error;
				
				item.AddItem (new EntityObject ("username", userName, OvalDataTypes.StringType, StatusFlags.Error, true));
				item.AddItem (new EntityObject ("password", "", OvalDataTypes.StringType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("chg_lst", "", OvalDataTypes.IntegerType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("chg_allow", "", OvalDataTypes.IntegerType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("chg_req", "", OvalDataTypes.IntegerType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("exp_warn", "", OvalDataTypes.IntegerType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("exp_inact", "", OvalDataTypes.IntegerType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("exp_date", "", OvalDataTypes.IntegerType, StatusFlags.Error, false));
				item.AddItem (new EntityObject ("flag", "", OvalDataTypes.IntegerType, StatusFlags.Error, false));
			
				return item;
			}			
		}
	
		[DllImport("libsussensupport")]
		internal extern static int shadow_helper_getspnam (string username,
			out string account, out string password, out long lastchange,
			out long min, out long max, out long warn, out long inact,
			out long expire, out long flag);
	}
}
