// 
// XinetdTest.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using Sussen.Oval;

namespace Sussen.Oval.Unix
{
	internal class XinetdRecord
	{
		string serviceName;
		Dictionary<string,string> serviceValues;	

		public XinetdRecord ()
		{
			serviceValues = new Dictionary<string,string>();
		}

		public void AddValue (string argName, string argValue)
		{
			serviceValues.Add (argName, argValue);
		}

		public string LookupValue (string argName)
		{
			if (serviceValues.ContainsKey (argName))
				return serviceValues[argName];
			else
				return null;
		}

		public string ServiceName 
		{
			get { return serviceName; }
			set { serviceName = value; }
		}
	}

	public class XinetdTest : Test
	{
		List<XinetdRecord> xinetdRecords;
		const string xinetdConf = "/etc/xinetd.conf";
		const string xinetdDir = "/etc/xinetd";
		
		public XinetdTest () : base ()
		{
			Operations[] ops = { Operations.Equals, Operations.NotEqual, Operations.PatternMatch };
			Operations[] nops = { Operations.Equals, Operations.NotEqual };
		
			this.objectData.Add ("protocol", ops);
			this.objectData.Add ("service_name", ops);
			
			this.stateData.Add ("protocol", ops);
			this.stateData.Add ("service_name", ops);
			this.stateData.Add ("flags", ops);
			this.stateData.Add ("no_access", ops);
			this.stateData.Add ("only_from", ops);
			this.stateData.Add ("port", ops);
			this.stateData.Add ("server", ops);
			this.stateData.Add ("server_arguments", ops);
			this.stateData.Add ("socket_type", ops);
			this.stateData.Add ("type", ops);
			this.stateData.Add ("user", ops);
			this.stateData.Add ("disabled", nops);
		}		

		public override CollectedItem[] Run (TestElement testElement, ObjectElement objectElement)
		{
			List<CollectedItem> items = new List<CollectedItem> ();

			xinetdRecords = new List<XinetdRecord>();
			if (File.Exists (xinetdConf)) {
				XinetdRecord[] recs = ParseXinetdConf (xinetdConf);
				xinetdRecords.AddRange (recs);
			} else if (Directory.Exists (xinetdDir)) {
				string[] xinetdFiles = Directory.GetFiles (xinetdDir);
				foreach (string f in xinetdFiles) {
					XinetdRecord xr = ParseServiceFile (f);
					xinetdRecords.Add (xr);
				}
			}

			if (!objectElement.Entities.ContainsKey ("protocol"))
				throw new TestException (testElement.ID, "Invalid xinetd test; missing protocol");
				
			if (!objectElement.Entities.ContainsKey ("service_name"))
				throw new TestException (testElement.ID, "Invalid xinetd test; missing service_name");	
			
			EntityObject protocol = objectElement.Entities["protocol"];
			EntityObject serviceName = objectElement.Entities["service_name"];

			if (protocol.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for protocol in inetd test");

			if (serviceName.DataType != OvalDataTypes.StringType)
				throw new TestException (testElement.ID, "Invalid datatype for service_name in inetd test");

			CheckEntityOperation (protocol);
			CheckEntityOperation (serviceName);

			XinetdRecord[] matchingItems = GetMatchingItems (protocol, serviceName);
			if (matchingItems.Length == 0)
				items.Add (CreateNotFound (protocol.Value, serviceName.Value));
			
			foreach (XinetdRecord xr in matchingItems)
				items.Add (CollectData (xr));

			return items.ToArray ();
		}

		static XinetdRecord ParseServiceFile (string fileName)
		{
			string line = null;
			XinetdRecord xr = new XinetdRecord ();

	                StreamReader sr = new StreamReader (fileName);
	                while ((line = sr.ReadLine()) != null) {
	                        if (line.StartsWith ("#") || line.StartsWith ("{") || line.StartsWith ("}"))
	                                continue;

				if (line.StartsWith ("service")) {
					char[] splitter  = {' '};
		                        string[] fields = line.Split (splitter, 2);
					xr.ServiceName = fields[1];
				} else {
					char[] splitter  = {'='};
		                        string[] fields = line.Split (splitter, 2);
					xr.AddValue (fields[0].Trim(), fields[1].Trim());
				}
			}

			return xr;	
		}

		static XinetdRecord[] ParseXinetdConf (string fileName)
		{
			string line = null;
			List<XinetdRecord> xinetdRecords = new List<XinetdRecord>();

			StreamReader sr = new StreamReader (fileName);
			while ((line = sr.ReadLine()) != null) {
				if (line.StartsWith ("#"))
					continue;

				if (line.StartsWith ("service")) {
					XinetdRecord xr = new XinetdRecord ();

					char[] splitter  = {' '};
					string[] fields = line.Split (splitter, 2);
					xr.ServiceName = fields[1];

					while ((line = sr.ReadLine()) != "}") {
						if (line.StartsWith ("#") || line.StartsWith ("{"))
							continue;

						char[] splitter2  = {'='};
						string[] fields2 = line.Split (splitter2, 2);
						xr.AddValue (fields2[0].Trim(), fields2[1].Trim());
					}

					xinetdRecords.Add (xr);
				}
			}

			return xinetdRecords.ToArray();
		}

		void CheckEntityOperation (EntityObject obj)
		{
			switch (obj.Operation) {
			case Operations.Equals:
			case Operations.NotEqual:
			case Operations.PatternMatch:
				break;
				
			default:
				throw new TestException ("Invalid operation for " + obj.Name + " object in xinetd test");
			}		
		}

		bool IsInetdItemItemMatchForEntityObject (string idi, EntityObject eobject)
		{
			if (eobject.VariableReference != null) {

				Variable var = Global.Variables [eobject.VariableReference];
				foreach (string v in var.ValueArray)
					if (CompareItems (v, idi, eobject.Operation))
						return true;
			} else {
			
				if (CompareItems (eobject.Value, idi, eobject.Operation))
					return true;
			}
			
			return false;
		}

		XinetdRecord[] GetMatchingItems (EntityObject protocol, EntityObject serviceName)
		{
			List<XinetdRecord> items = new List<XinetdRecord> (); 
		
			foreach (XinetdRecord xr in xinetdRecords) {
				
				bool foundMatch = false;
		
				if (IsInetdItemItemMatchForEntityObject (xr.LookupValue("protocol") , protocol)) {
					items.Add (xr);
					foundMatch = true;
				}
				
				if (IsInetdItemItemMatchForEntityObject (xr.ServiceName, serviceName) && !foundMatch)
					items.Add (xr);
			}
			
			return items.ToArray ();
		}

		CollectedItem CreateNotFound (string protocol, string serviceName)
		{
			CollectedItem item = new CollectedItem ("xinetd_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.DoesNotExist);
			item.AddItem (new EntityObject ("protocol", protocol, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("service_name", serviceName, OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("flags", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("no_access", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("only_from", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("port", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));			
			item.AddItem (new EntityObject ("server", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("server_arguments", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("socket_type", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("type", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("user", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			item.AddItem (new EntityObject ("disabled", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
				
			return item;		
		}

		CollectedItem CollectData (XinetdRecord xr)
		{
			CollectedItem item = new CollectedItem ("xinetd_item", itemCount, Vocabulary.UNIX_NAMESPACE, Flags.Complete);
			
			item.AddItem (new EntityObject ("protocol", xr.LookupValue("protocol"), OvalDataTypes.StringType, StatusFlags.DoesNotExist, true));
			item.AddItem (new EntityObject ("service_name", xr.ServiceName, OvalDataTypes.StringType, StatusFlags.Exists, true));

			string xrValue = xr.LookupValue("flags");
			if (xrValue == null)	
				item.AddItem (new EntityObject ("flags", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			else
				item.AddItem (new EntityObject ("flags", xrValue, OvalDataTypes.StringType, StatusFlags.Exists, false));

			xrValue = xr.LookupValue("no_access");
			if (xrValue == null)				
				item.AddItem (new EntityObject ("no_access", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			else
				item.AddItem (new EntityObject ("no_access", xrValue, OvalDataTypes.StringType, StatusFlags.Exists, false));

			xrValue = xr.LookupValue("only_from");
			if (xrValue == null)				
				item.AddItem (new EntityObject ("only_from", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			else
				item.AddItem (new EntityObject ("only_from", xrValue, OvalDataTypes.StringType, StatusFlags.Exists, false));

			xrValue = xr.LookupValue("port");
			if (xrValue == null)				
				item.AddItem (new EntityObject ("port", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			else
				item.AddItem (new EntityObject ("port", xrValue, OvalDataTypes.StringType, StatusFlags.Exists, false));
			
			xrValue = xr.LookupValue("server");
			if (xrValue == null)
				item.AddItem (new EntityObject ("server", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			else
				item.AddItem (new EntityObject ("server", xrValue, OvalDataTypes.StringType, StatusFlags.Exists, false));

			xrValue = xr.LookupValue("server_arguments");
			if (xrValue == null)				
				item.AddItem (new EntityObject ("server_arguments", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			else
				item.AddItem (new EntityObject ("server_arguments", xrValue, OvalDataTypes.StringType, StatusFlags.Exists, false));
				
			xrValue = xr.LookupValue("socket_type");
			if (xrValue == null)				
				item.AddItem (new EntityObject ("socket_type", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			else
				item.AddItem (new EntityObject ("socket_type", xrValue, OvalDataTypes.StringType, StatusFlags.Exists, false));

			xrValue = xr.LookupValue("type");
			if (xrValue == null)				
				item.AddItem (new EntityObject ("type", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			else
				item.AddItem (new EntityObject ("type", xrValue, OvalDataTypes.StringType, StatusFlags.Exists, false));

			xrValue = xr.LookupValue("user");
			if (xrValue == null)				
				item.AddItem (new EntityObject ("user", "", OvalDataTypes.StringType, StatusFlags.DoesNotExist, false));
			else
				item.AddItem (new EntityObject ("user", xrValue, OvalDataTypes.StringType, StatusFlags.Exists, false));

			xrValue = xr.LookupValue("user");
			if (xrValue == null)				
				item.AddItem (new EntityObject ("disabled", "", OvalDataTypes.BooleanType, StatusFlags.DoesNotExist, false));
			else {
				if (xrValue.ToLower() == "yes")
					item.AddItem (new EntityObject ("disabled", "1", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
				else
					item.AddItem (new EntityObject ("disabled", "0", OvalDataTypes.BooleanType, StatusFlags.Exists, false));
			}
					
			return item;		
		}
	}
}
