// 
// DataCollector.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.Xml;

namespace Sussen.Oval
{
	public delegate Test PlatformCreateTestHandler (string testName, string testVersion);

	public class DataCollector
	{
		OvalDocuments docs;
		List<string> tdItems;
		int tdGUID = 1;

		public PlatformCreateTestHandler CreateTest;		

		public DataCollector (OvalDocuments ovalDocuments)
		{
			if (ovalDocuments == null)
				throw new ArgumentNullException ("ovalDocuments");
				
			docs = ovalDocuments;
			tdItems = new List<string> ();
		}

		public void Run ()
		{
			if (CreateTest == null)
				throw new ArgumentNullException ("CreateTest");
	
			foreach (KeyValuePair<string, TestElement> kvp in docs.Definitions.Tests) {
			
				TestElement te = kvp.Value;
				if (te.ObjectRef == null || !docs.Definitions.Objects.ContainsKey (te.ObjectRef))
                    continue;
                    
				ObjectElement oe = docs.Definitions.Objects [te.ObjectRef];
				if (oe.Set != null) {
					ProcessSet (te, oe);
					continue;
				}
				
				try {
					Test test = CreateTest (te.Name, docs.Definitions.SchemaVersion);
					RunTest (test, te, oe);
				} catch (Exception e) {			
					docs.SystemCharacteristics.WrongPlatform.Add (te.ID, te.ID); 
					docs.SystemCharacteristics.WriteCollectedObject(oe.ID, oe.Version, Flags.NotApplicable);
					Log.Debug (e);
				}
			}
		}	

		void ProcessSet (TestElement testElement, ObjectElement objectElement)
		{
			Log.Debug ("Processing set for object: {0}", objectElement.ID);

            bool intersection = false;
            bool union = false;

            if (objectElement.Set.Operator.ToLower() == "intersection")
                intersection = true;
            else if (objectElement.Set.Operator.ToLower() == "union")
                union = true;

            if (objectElement.Set.NestedSets.Count > 0) {
                foreach (SetElement set in objectElement.Set.NestedSets) {
                    if (set.Operator.ToLower() == "complement") {
                        List<CollectedItem> si = new List<CollectedItem>();
                        foreach (string s in set.ObjectReferences) {
                            ObjectElement oe = docs.Definitions.Objects[s];
                            if (oe == null) {
                                Log.Debug("Unable to find object for reference: {0}", s);
                                continue;
                            }

                            try {
                                Log.Debug("Running test {0} with object {1}", testElement.ID, oe.ID);
                                Test test = CreateTest(testElement.Name, docs.Definitions.SchemaVersion);                                
                                CollectedItem[] res = test.Run(testElement, oe);
                                docs.SystemCharacteristics.WriteCollectedObject(oe.ID, oe.Version, Flags.Complete);

                                foreach (CollectedItem c in res) {                                    
                                    si.Add(c);
                                    if (c.ID == 0) {
                                        c.ID = Global.CollectedObjectCount++;
                                        c.Write(docs.SystemCharacteristics);
                                    }
                                    docs.SystemCharacteristics.AddToCollectedObject(oe.ID, c.ID);                                    
                                }
                                
                            } catch (TestException e) {
                                Log.Debug(e);
                                if (!docs.SystemCharacteristics.WrongPlatform.ContainsKey (testElement.ID)) {
                                	docs.SystemCharacteristics.WrongPlatform.Add(testElement.ID, testElement.ID);
	                                docs.SystemCharacteristics.WriteCollectedObject(oe.ID, oe.Version, Flags.NotApplicable);
	                            }
                            } catch (ArgumentOutOfRangeException ex) {
                            	Log.Debug (ex);
                            	if (!docs.SystemCharacteristics.WrongPlatform.ContainsKey (testElement.ID)) {
                                	docs.SystemCharacteristics.WrongPlatform.Add(testElement.ID, testElement.ID);
	                                docs.SystemCharacteristics.WriteCollectedObject(oe.ID, oe.Version, Flags.NotApplicable);
	                            }
                            }

                            Log.Debug("si.Count in complement: {0}", si.Count);

                            foreach (string k in oe.Entities.Keys) {
                                EntityObject eo = oe.Entities[k];
                                if (eo.VariableReference != null) {
                                    Variable var = Global.Variables[eo.VariableReference];
                                    foreach (string vs in var.ValueArray)
                                        docs.SystemCharacteristics.AddToCollectedObject(objectElement.ID, var.ID, vs);
                                }
                            }
                        }

                        set.Results = si.ToArray();
                    } 
                    else 
                    {
                        foreach (string s in set.ObjectReferences) {
                            ObjectElement oe = docs.Definitions.Objects[s];
                            if (oe == null) {
                                Log.Debug("Unable to find object for reference: {0}", s);
                                continue;
                            }

                            try {
                                Test test = CreateTest(testElement.Name, docs.Definitions.SchemaVersion);
                                RunTest(test, testElement, oe);
                            } catch (Exception e) {
                                Log.Debug(e);
                                docs.SystemCharacteristics.WrongPlatform.Add(testElement.ID, testElement.ID);
                                docs.SystemCharacteristics.WriteCollectedObject(oe.ID, oe.Version, Flags.NotApplicable);
                            }
                        }
                    }
                }

                List<CollectedItem> setItems = new List<CollectedItem>();

                if (intersection)
                {
                    if (objectElement.Set.NestedSets.Count == 2)
                    {
                        Log.Debug("Intersection set A results length: {0}", objectElement.Set.NestedSets[0].Results.Length);
                        Log.Debug("Intersection set B results length: {0}", objectElement.Set.NestedSets[1].Results.Length);

                        foreach (CollectedItem aci in objectElement.Set.NestedSets[0].Results)
                            foreach (CollectedItem bci in objectElement.Set.NestedSets[1].Results)
                                if (AreCollectedItemsEqual(aci, bci))
                                    setItems.Add(aci);
                    }
                }
                else if (union)
                {
                    if (objectElement.Set.NestedSets.Count == 2)
                    {
                        foreach (CollectedItem aci in objectElement.Set.NestedSets[0].Results)
                            foreach (CollectedItem bci in objectElement.Set.NestedSets[1].Results)
                                if (!AreCollectedItemsEqual(aci, bci))
                                    setItems.Add(aci);

                        foreach (CollectedItem bci in objectElement.Set.NestedSets[1].Results)
                            setItems.Add(bci);
                    }
                }

                Log.Debug("Set items count: {0}", setItems.Count);

                foreach (CollectedItem ci in setItems)
                {
                    ci.ID = Global.CollectedObjectCount++;

                    if (!this.tdItems.Contains(objectElement.ID))
                    {
                        tdItems.Add(objectElement.ID);

                        docs.SystemCharacteristics.WriteCollectedObject(objectElement.ID, objectElement.Version, ci);
                        ci.Write(docs.SystemCharacteristics);

                    }
                    else
                    {
                        docs.SystemCharacteristics.AddToCollectedObject(objectElement.ID, ci.ID);
                        ci.Write(docs.SystemCharacteristics);
                    }
                }

            } else {

				foreach (string s in objectElement.Set.ObjectReferences) {

					ObjectElement oe = docs.Definitions.Objects [s];
					
					if (oe == null) {
						Log.Debug ("Unable to find object for reference: {0}", s);
						continue;
					}

					try {
						Test test = CreateTest (testElement.Name, docs.Definitions.SchemaVersion);
						RunTest (test, testElement, oe);
					} catch (Exception e) {
						Log.Debug (e);
	                    docs.SystemCharacteristics.WrongPlatform.Add(testElement.ID, testElement.ID);
	                    docs.SystemCharacteristics.WriteCollectedObject(oe.ID, oe.Version, Flags.NotApplicable);
					}
				}
            }
		}

        bool AreCollectedItemsEqual(CollectedItem a, CollectedItem b)
        {
            if (a.Items.Length != b.Items.Length)
                return false;

            bool allMatch = true;
            for (int i = 0; i < a.Items.Length; i++) {
                if (!(a.Items[i].Name == b.Items[i].Name && a.Items[i].Value == b.Items[i].Value))
                    allMatch = false;
            }

            return allMatch;
        }
		
		void RunTest (Test test, TestElement testElement, ObjectElement objectElement)
		{			
			try {
				
				Log.Debug("Running test {0} ({1})", testElement.ID, testElement.Comment);
				Log.Debug("Using object {0}", objectElement.ID);

				if (this.tdItems.Contains(objectElement.ID))
					return;

				CollectedItem[] results;
				results = test.Run (testElement, objectElement);
		
				if (results.Length == 0) {
					docs.SystemCharacteristics.WriteCollectedObject(objectElement.ID, objectElement.Version, Flags.DoesNotExist);
					Log.Debug("Object does not exist");
				}
				
				if (results.Length > 100)
					Log.Warn ("Collected {0} results from object {0} as part of test {1}", results.Length, testElement.ID);
		
				foreach (CollectedItem ci in results) {			
		
					ci.ID = tdGUID++;
					Log.Debug("Test {0} finished on item {1} got status {2}", testElement.ID, ci.ID, Vocabulary.GetFlag(ci.Flag));
					
					if (!this.tdItems.Contains (objectElement.ID)) {
					
						tdItems.Add (objectElement.ID);			
	
						docs.SystemCharacteristics.WriteCollectedObject (objectElement.ID, objectElement.Version, ci);
						ci.Write (docs.SystemCharacteristics);
						
					} else {

						// object is a duplicate; don't rewrite it
						if (results.Length > 1) {
							docs.SystemCharacteristics.AddToCollectedObject (objectElement.ID, ci.ID);
							ci.Write (docs.SystemCharacteristics);
						}
					}
				}
				
				foreach (string k in objectElement.Entities.Keys) {
					EntityObject eo = objectElement.Entities[k];				
					if (eo.VariableReference != null) {
						Variable var = Global.Variables [eo.VariableReference];
						foreach (string s in var.ValueArray)
							docs.SystemCharacteristics.AddToCollectedObject (objectElement.ID, var.ID, s);
					}
				}
				
			} catch (TestException te) {
				Log.Warn ("Unable to run test [{0}]. {1}", te.ID, te.Message);
				Log.Debug (te);
			} 

			Log.Debug("");
		}		
	}
}
