// 
// DefinitionAnalyzer.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text.RegularExpressions;
using System.Xml;

namespace Sussen.Oval
{
	public class DefinitionAnalyzer : Analyzer
	{
		OvalDocuments docs;
		XmlElement ovalDefinitionElement;
		XmlElement definitionElement;
		XmlElement criteriaElement;
		
		public DefinitionAnalyzer ()
		{
		}

		public DefinitionAnalyzer (XmlElement ovalDefinition, OvalDocuments ovalDocuments)
		{
			if (ovalDefinition == null)
				throw new ArgumentNullException ("ovalDefinition");

			if (ovalDocuments == null)
				throw new ArgumentNullException ("ovalDocuments");
		
			docs = ovalDocuments;
			ovalDefinitionElement = ovalDefinition;
		
			string id = ovalDefinitionElement.GetAttribute ("id");
			Match m = Regex.Match (id, Vocabulary.DEFINITION_ID_PATTERN);
			if (!m.Success) {
				throw new ArgumentException("Invalid format for definition ID: " + id, "id");
			}			
		
			definitionElement = docs.Results.CreateElement ("definition");
			definitionElement.SetAttribute ("definition_id", id);
			definitionElement.SetAttribute ("version", ovalDefinition.GetAttribute ("version"));
			// FIXME: Doesn't handle analyzing the same definition multiple times with different values
			definitionElement.SetAttribute ("variable_instance", "1");
			
			criteriaElement = docs.Results.CreateElement ("criteria");
			definitionElement.AppendChild (criteriaElement);			
		}
		
		public TestResult Run ()
		{
			TestResult criteriaResult = TestResult.Unknown;
			
			XmlNodeList criteriaList = ovalDefinitionElement.GetElementsByTagName ("criteria");
			if (!(criteriaList.Count > 0))
				throw new AnalyzerException ("No criteria found in definition");
					
			Log.Debug ("Number of criteria found: {0}", criteriaList.Count);

			XmlElement ce = (XmlElement) criteriaList[0];			
			criteriaResult = AnalyzeCriteria (ce, criteriaElement);				
			
			Log.Debug ("Test result after AnalyzeCriteria: {0}", Vocabulary.GetTestResult (criteriaResult));
			definitionElement.SetAttribute ("result", Vocabulary.GetTestResult (criteriaResult));

			XmlNodeList nl = docs.Results.GetElementsByName ("results");
			if (!(nl.Count > 0))
				throw new AnalyzerException ("No results element found in results document");
				
			XmlElement results = (XmlElement) nl[0];
			nl = results.GetElementsByTagName ("definitions");
			if (!(nl.Count > 0))
				throw new AnalyzerException ("No tests element found in results document");
				
			XmlElement definitions = (XmlElement) nl[0];
			definitions.AppendChild (definitionElement);
			
			return criteriaResult;
		}

		public TestResult Run (XmlElement defElement)
		{
			TestResult criteriaResult = TestResult.Unknown;

			XmlNodeList criteriaList = defElement.GetElementsByTagName ("criteria");
			if (!(criteriaList.Count > 0))
				throw new AnalyzerException ("No criteria found in definition");
				
			Log.Debug ("Number of criteria found: {0}", criteriaList.Count);

			XmlElement ce = (XmlElement) criteriaList[0];			
			criteriaResult = AnalyzeCriteria (ce, criteriaElement);
			
			Log.Debug ("Test result after AnalyzeCriteria: {0}", Vocabulary.GetTestResult (criteriaResult));
				
			return criteriaResult;		
		}

		public TestResult Run (CriteriaElement criteriaElement, List<TestResult> testResults)
		{
			TestResult criteriaResult = TestResult.Unknown;
			
			if (criteriaElement.Operator == "")
				criteriaElement.Operator = "AND";
			
			criteriaResult = AnalyzeResults (testResults, criteriaElement.Operator);
			
			return criteriaResult;
		}

		XmlElement LookupDefinition (string definitionID)
		{
			XmlNodeList nl = docs.Definitions.GetElementsByName ("definition");
			foreach (XmlElement xe in nl) {
				string id = xe.GetAttribute ("id");
				if (definitionID == id)
					return xe;
			}
			
			return null;
		}
		
		TestResult AnalyzeExtendDefinition (string refID)
		{		
			TestResult res = TestResult.Unknown;
			
			if (resultsCache.ContainsKey (refID)) {
				res = resultsCache [refID];			
				Log.Debug ("Got cached result {0} for test {1}", res, refID);
				return res;
			}
			
			XmlElement extDefElement = LookupDefinition (refID);
			if (extDefElement != null) {
				res = this.Run (extDefElement);			
			} else {
				Log.Debug ("Unable to find extend_definition: {0}", refID); 
			}
			
			resultsCache.Add (refID, res);
			
			return res;
		}

		TestResult AnalyzeCriteria (XmlElement criteriaElement, XmlElement resCriteriaElement)
		{
			List <TestResult> allResults = new List <TestResult> ();			
			TestResult criteriaResult = TestResult.Unknown;
			
			string criteriaNegate = criteriaElement.GetAttribute("negate");
			string criteriaOperator = criteriaElement.GetAttribute ("operator");
			if (criteriaOperator == "")
                criteriaOperator = "AND";
			
			foreach (XmlElement e in criteriaElement.ChildNodes) {
		
				if (e.Name == "criteria") {
					XmlElement newResCritElement = docs.Results.CreateElement ("criteria");
					resCriteriaElement.AppendChild (newResCritElement);					
					TestResult result = AnalyzeCriteria (e, newResCritElement);
					allResults.Add (result);
				} else if (e.Name == "criterion") {				
					TestResult result = AnalyzeCriterion (e, resCriteriaElement);
					allResults.Add (result);
				} else if (e.Name == "extend_definition") {
					string refID = e.GetAttribute ("definition_ref");
					TestResult result = AnalyzeExtendDefinition (refID); 			
					allResults.Add (result);
				}
			}
			
			criteriaResult = AnalyzeResults (allResults, criteriaOperator);
			
			if (criteriaNegate == "true")
			{
				Log.Debug("Negating result {0}", Vocabulary.GetTestResult(criteriaResult));
				criteriaResult = NegateResult(criteriaResult);
			}
			
			resCriteriaElement.SetAttribute ("operator", criteriaOperator);
			resCriteriaElement.SetAttribute ("result", Vocabulary.GetTestResult (criteriaResult)); 			
			
			Log.Debug ("Analyzed definition criteria; operator: [{0}] result: [{1}]", criteriaOperator, Vocabulary.GetTestResult (criteriaResult)); 
			
			return criteriaResult;
		}

		TestResult AnalyzeCriterion (XmlElement criterionElement, XmlElement resCritElement)
		{
			TestResult criterionResult = TestResult.Unknown;
			string testRef = criterionElement.GetAttribute ("test_ref");
			string testNegate = criterionElement.GetAttribute ("negate");
			
			Log.Debug ("AnalyzeCriterion [{0}] comment: {1}", testRef, criterionElement.GetAttribute ("comment"));
			
			if (testRef == null)
				throw new AnalyzerException ("Unable to find test reference in criterion element");

			TestElement testElement = docs.Definitions.Tests [testRef];				
			if (testElement == null)
				throw new AnalyzerException ("Unable to find test element " + testRef);
			
			if (resultsCache.ContainsKey (testRef)) {
				criterionResult = resultsCache [testRef];			
				Log.Debug ("Got cached result {0} for test {1}", criterionResult, testRef);				
			} else {

				TestAnalyzer ta = new TestAnalyzer ();
				criterionResult = ta.Run (testElement, docs);
				
				if (testNegate == "true") {
					Log.Debug("Negating result {0}", Vocabulary.GetTestResult (criterionResult));
					criterionResult = NegateResult (criterionResult);
				}
					
				resultsCache.Add (testRef, criterionResult);
			}

			Log.Debug ("Analyzed definition criterion; testRef: [{0}] negate: [{1}] result: [{2}]", 
				testRef, testNegate, Vocabulary.GetTestResult (criterionResult));

			WriteCriterion (testRef, testNegate, criterionResult, resCritElement);
			
			return criterionResult;
		}
		
		void WriteCriterion (string testRef, string testNegate, TestResult result, XmlElement resCritElement)
		{
			XmlElement criterion = docs.Results.CreateElement ("criterion");
			criterion.SetAttribute ("test_ref", testRef);
			criterion.SetAttribute ("version", "1");
			criterion.SetAttribute ("variable_instance", "1");
			
			if (testNegate.ToLower() == "true")
				criterion.SetAttribute ("negate", "true");
			
			criterion.SetAttribute ("result", Vocabulary.GetTestResult (result));
			
			resCritElement.AppendChild (criterion);
		}
	}
}
