// 
// OvalDocuments.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Security.Cryptography;
using System.Text.RegularExpressions;
using System.Xml;
using System.Xml.Schema;
using System.Xml.XPath;
using System.Xml.Xsl;

namespace Sussen.Oval 
{
	public class OvalDocuments
	{
		DefinitionsDocument definitions;
		SystemCharacteristicsDocument systemCharacteristics;
		ResultsDocument results;
	
		public OvalDocuments ()
		{
			definitions = new DefinitionsDocument ();
			systemCharacteristics = new SystemCharacteristicsDocument ();
			results = new ResultsDocument (); 
		}
		
		public DefinitionsDocument Definitions
		{
			get { return definitions; }
			set { definitions = value; }
		}
		
		public SystemCharacteristicsDocument SystemCharacteristics
		{
			get { return systemCharacteristics; }
			set { systemCharacteristics = value; }
		}
		
		public ResultsDocument Results
		{
			get { return results; }
			set { results = value; }		
		}
	}
	
	public class Document
	{
		protected XmlDocument doc;
		protected bool isValid;
		protected string fileName;
		protected string schemaVersion;
		
		public Document ()
		{
			doc = new XmlDocument ();
			isValid = true;
		}
		
		public void Load (string fileName)
		{
			if (!File.Exists (fileName))
				throw new FileNotFoundException (fileName);
							
			try {
				
				doc.Load (fileName);
				this.fileName = fileName;
							
			} catch (XmlException xe) {
			
				Log.Error (xe);			
				throw xe;
			} 		
		}
		
		public void LoadXml (string xmlData)
		{
			if (xmlData == null)
				throw new ArgumentNullException ("xmlData");
	
			try {			
				doc.LoadXml (xmlData);
			} catch (XmlException xe) {
				Log.Error (xe);
				throw xe;
			}				
		}
		
		public void Save (string fileName)
		{
			try {
				
				doc.Save (fileName);
							
			} catch (Exception e) {
			
				Log.Error (e);			
				throw e;
			}		
		}		
		
		public bool CheckSchemaVersion ()
		{
			bool retVal = false;
			
			XmlNodeList elemList = doc.GetElementsByTagName("oval:schema_version");
			if (elemList.Count > 0) {				
				schemaVersion = elemList[0].InnerXml;
				if (schemaVersion.StartsWith ("5"))					
					retVal = true;
			} 
			
			elemList = doc.GetElementsByTagName("schema_version");
			if (elemList.Count > 0) {				
				schemaVersion = elemList[0].InnerXml;
				if (schemaVersion.StartsWith ("5"))					
					retVal = true;
			}
			
			return retVal;
		}
		
		public static bool CheckFileHash (string fileName, string hashValue)
		{
			bool retVal = false;
			string fileHashString = null;
			byte[] fileHash = null;

			FileStream fs = File.OpenRead (fileName);

			MD5 md5sum = new MD5CryptoServiceProvider();

			fileHash = md5sum.ComputeHash (fs);			
			fileHashString = BitConverter.ToString(fileHash).Replace("-", "");

			fs.Close ();

			if (hashValue.ToLower() == fileHashString.ToLower())
				retVal = true;

			return retVal;
		}

		public XmlElement CreateElement (string elementName)
		{
			XmlElement xe = doc.CreateElement (elementName);
			return xe;
		}

		public XmlElement GetElement (string elementName)
		{
			XmlNodeList tmpList = doc.GetElementsByTagName (elementName);
			if (!(tmpList.Count > 0))
				return null;

			return (XmlElement) tmpList[0];
		}
		
		public XmlElement CopyElement (XmlElement origElement)
		{
			if (origElement == null)
				return null;
				
			XmlElement newElement = doc.CreateElement (origElement.Name, origElement.NamespaceURI);
					
			if (origElement.HasChildNodes)
				if (!(origElement.FirstChild.NodeType == XmlNodeType.Element))
					newElement.InnerText = origElement.InnerText;

			foreach (XmlAttribute a in origElement.Attributes) {
				// FIXME: Need some way to deal with xsi attributes
				if (a.Name.StartsWith ("xsi")) {
					continue;
				}
			
				newElement.SetAttribute (a.Name, a.Value);
			}

			foreach (XmlNode n in origElement.ChildNodes) {
			
				if (!(n.NodeType == XmlNodeType.Element))
					continue;
				
				XmlElement b = CopyElement ((XmlElement)n);	
				newElement.AppendChild (b);
			}
					
			return newElement;
		}
		
		public XmlNodeList GetElementsByName (string elementName)
		{
			return doc.GetElementsByTagName (elementName);
		}

		public override string ToString ()
		{
			return doc.InnerXml;
		}

		protected void OnValidationEvent(object o, ValidationEventArgs args) 
		{
			isValid = false;
			Log.Error (args.Message);
		}

		public virtual bool Validate(string fileName, string schemaDirectory)
		{
			return true;
		}

		public string Filename
		{
			get { return fileName; }
		}
		
		public string SchemaVersion
		{
			get {
				if (schemaVersion == null)
					CheckSchemaVersion ();
					
				return schemaVersion;
			}
		}
	}
	
	public class SetElement
	{
		string setOperator;
		List<string> filters;
		List<string> objectRefs;
		List<SetElement> nestedSets;
		List<CollectedItem> results;
		
		public SetElement (XmlElement setElement)
		{
			if (setElement == null)
				throw new ArgumentNullException ("setElement");
		
			filters = new List<string> ();
			objectRefs = new List<string> ();
			nestedSets = new List <SetElement> ();
			results = new List<CollectedItem>();
			
			setOperator = setElement.GetAttribute ("set_operator");
			if (setOperator == null || setOperator == "")
				setOperator = "UNION";
					
			foreach (XmlElement se in setElement.ChildNodes) {
				if (se.NodeType != XmlNodeType.Element)
					continue;

				switch (se.Name) {

				case "oval-def:set":
				case "set":
					nestedSets.Add (new SetElement (se));
					break;

				case "oval-def:object_reference":
				case "object_reference":
					CheckID (se.InnerText, Vocabulary.OBJECT_ID_PATTERN);
					objectRefs.Add (se.InnerText);
					break;
				
				case "oval-def:filter":
				case "filter":
					CheckID (se.InnerText, Vocabulary.STATE_ID_PATTERN);
					filters.Add (se.InnerText);
					break;

				default:
					break;
				}
			}
		}

		static void CheckID (string id, string pattern)
		{
			Match m = Regex.Match (id, pattern);
			if (!m.Success) {
				throw new ArgumentException("Invalid format for ID", "id");
			}
		}
		
		public string Operator
		{
			get { return setOperator; }
		}

		public string[] ObjectReferences
		{
			get { return objectRefs.ToArray (); }
		}
		
		public string[] Filters
		{
			get { return filters.ToArray (); }
		}
				
		public List<SetElement> NestedSets
		{
			get { return nestedSets; }
		}
		
		public CollectedItem[] Results
        {
            get { return results.ToArray(); }
            set { results = new List<CollectedItem>(value); }
        }
	}

	public class EntityObject
	{
		string objName;
		string objValue;
		string varRef;
		
		Operations operation;
		OvalDataTypes dataType;
		Checks checkType;
		
		bool nil;

		StatusFlags status;
		bool isTestObject;
	
		public EntityObject (XmlElement entityElement)
		{
			if (entityElement == null)
				throw new ArgumentNullException ("entityElement");
				
			objName = entityElement.Name;
			objValue = entityElement.InnerText;
			
			string v = entityElement.GetAttribute ("var_ref");
			if (v != "")
				varRef = v; 
				
			operation = Vocabulary.GetOperation (entityElement.GetAttribute ("operation"));
			dataType = Vocabulary.GetDataType (entityElement.GetAttribute ("datatype"));
			checkType = Vocabulary.GetCheck (entityElement.GetAttribute ("var_check"));			
			
			string n = entityElement.GetAttribute ("xsi:nil");
			if (n == null || n == "" || n == "false")
				nil = false;
			else
				nil = true;
		}

		public EntityObject (string objectName, string objectValue, Operations op, OvalDataTypes dt, Checks check)
		{
			objName = objectName;
			objValue = objectValue;
			operation = op;
			dataType = dt;
			checkType = check;			
		}

		public EntityObject (string objectName, string objectValue, OvalDataTypes dt, StatusFlags sf, bool testObject)
		{
			objName = objectName;
			objValue = objectValue;
			dataType = dt;
			status = sf;
			isTestObject = testObject;
			
			operation = Vocabulary.GetOperation ("");
			dataType = Vocabulary.GetDataType ("");
		}

		public TestResult CompareTo (EntityObject eo)
		{
			TestResult result = TestResult.Unknown;
				
			switch (Vocabulary.GetDataType (this.DataType)) {
			case "binary":
				Log.Info ("Unsupported compare to operation on binary");
//				result = CompareBinaryItems (eo.Value, this.Value, this.Operation);
				break;
					
			case "boolean":
				result = CompareBoolItems (eo.Value, this.Value, this.Operation);
				break;

			case "evr_string":		
				result = CompareEvrStringItems (eo.Value, this.Value, this.Operation);
				break;
					
			case "float":
				result = CompareFloatItems (eo.Value, this.Value, this.Operation);			
				break;

			case "ios_version":
				Log.Info ("Unsupported compare to operation on ios_version");
//				result = CompareIosVersionItems (eo.Value, this.Value, this.Operation);
				break;
					
			case "int":
				result = CompareIntItems (eo.Value, this.Value, this.Operation);
				break;
					
			case "string":
				result = CompareStringItems (eo.Value, this.Value, this.Operation);			
				break;

			case "version":
				result = CompareVersionItems (eo.Value, this.Value, this.Operation);			
				break;
					
			default:
				break;
			}
			
			return result;
		}

		public TestResult CompareTo(string eo)
		{
			TestResult result = TestResult.Unknown;
	
			switch (Vocabulary.GetDataType(this.DataType)) {
			
			case "binary":
				Log.Info ("Unsupported compare to operation on binary");
//			    result = CompareBinaryItems (eo, this.Value, this.Operation);
				break;

			case "boolean":
				result = CompareBoolItems(eo, this.Value, this.Operation);
				break;

			case "evr_string":		
				result = CompareEvrStringItems (eo, this.Value, this.Operation);
				break;

			case "float":
				result = CompareFloatItems(eo, this.Value, this.Operation);
				break;

			case "ios_version":
				Log.Info ("Unsupported compare to operation on ios_version");
//				result = CompareIosVersionItems (eo, this.Value, this.Operation);
				break;

			case "int":
				result = CompareIntItems(eo, this.Value, this.Operation);
				break;

			case "string":
				result = CompareStringItems(eo, this.Value, this.Operation);
				break;

			case "version":
				result = CompareVersionItems(eo, this.Value, this.Operation);
				break;

			default:
				break;
			}

			return result;
		}

		TestResult CompareBoolItems (string b1, string b2, Operations itemOperator)
		{
			TestResult retVal = TestResult.Unknown;
			bool a, b;
			
			if (b1.ToLower() == "true" || b1 == "1")
				a =  true;
			else
				a = false;
				
			if (b2.ToLower() == "true" || b2 == "1")
				b = true;
			else
				b = false;
						
			retVal = TestResult.False;
			
			switch (itemOperator) {
			
			case Operations.Equals:
				if (a == b)
					retVal = TestResult.True;
				break;
				
			case Operations.NotEqual:
				if (a != b)
					retVal = TestResult.True;
				break;
				
			default:
				retVal = TestResult.NotApplicable;
				break;
			}
			
			return retVal;
		}

		TestResult CompareEvrStringItems (string e1, string e2, Operations itemOperator)
		{
			TestResult retVal = TestResult.False;

			EvrStringType evr1 = null;
			EvrStringType evr2 = null;

			try {
				evr1 = new EvrStringType (e1);
				evr2 = new EvrStringType (e2);
			} catch (Exception e) {		
				Log.Debug (e);
				return TestResult.Error;
			}
								
			switch (itemOperator) {
			
			case Operations.Equals:
				if (evr1 == evr2)
					retVal = TestResult.True;
				break;
				
			case Operations.NotEqual:
				if (evr1 != evr2)
					retVal = TestResult.True;
				break;
			
			case Operations.LessThan:
				if (evr1 < evr2)
					retVal = TestResult.True;			
				break;
				
			case Operations.LessThanOrEqual:
				if (evr1 <= evr2)
					retVal = TestResult.True;			
				break;
				
			case Operations.GreaterThan:
				if (evr1 > evr2)
					retVal = TestResult.True;			
				break;
				
			case Operations.GreaterThanOrEqual:
				if (evr1 >= evr2)
					retVal = TestResult.True;
				break;
			
			default:
				retVal = TestResult.NotApplicable;
				break;
			}
			
			return retVal;
		}

		TestResult CompareFloatItems (string f1, string f2, Operations itemOperator)
		{
			TestResult retVal = TestResult.Unknown;
			double a, b;
			
			try {
				a = double.Parse (f1);
				b = double.Parse (f2);				
			} catch (Exception e) {
				Log.Debug (e);
				retVal = TestResult.Error;
				return retVal;
			}
			
			retVal = TestResult.False;
			
			switch (itemOperator) {
			
			case Operations.Equals:
				if (a == b)
					retVal = TestResult.True;
				break;
				
			case Operations.NotEqual:
				if (a != b)
					retVal = TestResult.True;
				break;
				
			case Operations.LessThan:
				if (a < b)
					retVal = TestResult.True;			
				break;
				
			case Operations.GreaterThan:
				if (a > b)
					retVal = TestResult.True;			
				break;
				
			case Operations.LessThanOrEqual:
				if (a <= b)
					retVal = TestResult.True;			
				break;
				
			case Operations.GreaterThanOrEqual:
				if (a >= b)
					retVal = TestResult.True;
				break;
				
			default:
				retVal = TestResult.NotApplicable;
				break;
			}
			
			return retVal;
		}

		TestResult CompareIntItems (string i1, string i2, Operations itemOperator)
		{
			TestResult retVal = TestResult.Unknown;
			int a, b;
			
			try {
				a = int.Parse (i1);
				b = int.Parse (i2);				
			} catch (Exception e) {
				Log.Debug (e);
				retVal = TestResult.Error;
				return retVal;
			}
			
			retVal = TestResult.False;
			
			switch (itemOperator) {
			
			case Operations.Equals:
				if (a == b)
					retVal = TestResult.True;
				break;
				
			case Operations.NotEqual:
				if (a != b)
					retVal = TestResult.True;
				break;
				
			case Operations.LessThan:
				if (a < b)
					retVal = TestResult.True;			
				break;
				
			case Operations.GreaterThan:
				if (a > b)
					retVal = TestResult.True;			
				break;
				
			case Operations.LessThanOrEqual:
				if (a <= b)
					retVal = TestResult.True;			
				break;
				
			case Operations.GreaterThanOrEqual:
				if (a >= b)
					retVal = TestResult.True;			
				break;
			
			case Operations.BitwiseAnd:
				if ((a & b) == a)
					retVal = TestResult.True;			
				break;
				
			case Operations.BitwiseOr:
				if ((a | b) == a)
					retVal = TestResult.True;			
				break;
				
			default:
				retVal = TestResult.NotApplicable;
				break;
			}
			
			return retVal;
		}

		TestResult CompareStringItems (string s1, string s2, Operations itemOperator)
		{
			TestResult retVal = TestResult.Unknown;

			if (s1 == null || s2 == null)
				return retVal;
			
			retVal = TestResult.False;
			
			switch (itemOperator) {
			
			case Operations.Equals:
				if (s1 == s2)
					retVal = TestResult.True;
				break;
				
			case Operations.NotEqual:
				if (s1 != s2)
					retVal = TestResult.True;
				break;
				
			case Operations.LessThan:
				if ((string.Compare (s1, s2) < 0))
					retVal = TestResult.True;			
				break;
				
			case Operations.GreaterThan:
				if ((string.Compare (s1, s2) > 0))
					retVal = TestResult.True;			
				break;
				
			case Operations.LessThanOrEqual:
				if ((string.Compare (s1, s2) <= 0))
					retVal = TestResult.True;			
				break;
				
			case Operations.GreaterThanOrEqual:
				if ((string.Compare (s1, s2) >= 0))
					retVal = TestResult.True;			
				break;

			// FIXME: convert OVAL regex format (posix) to MS.NET format
			case Operations.PatternMatch:
				try {
					Match m = Regex.Match(s1, s2);
					if (m.Success)
						retVal = TestResult.True;
						
				} catch (Exception e) {
					Log.Debug (e);
					retVal = TestResult.Error;
				}
				break;				
				
			default:
				retVal = TestResult.NotApplicable;
				break;
			}
			
			return retVal;
		}

		TestResult CompareVersionItems (string v1, string v2, Operations itemOperator)
		{
			TestResult retVal = TestResult.False;

			VersionType ver1 = null;
			VersionType ver2 = null;

			try {
				ver1 = new VersionType (v1);
				ver2 = new VersionType (v2);
			} catch (Exception e) {
				Log.Debug (e);
				return TestResult.Error;
			}
								
			switch (itemOperator) {
			
			case Operations.Equals:
				if (ver1 == ver2)
					retVal = TestResult.True;
				break;
				
			case Operations.NotEqual:
				if (ver1 != ver2)
					retVal = TestResult.True;
				break;
			
			case Operations.LessThan:
				if (ver1 < ver2)
					retVal = TestResult.True;			
				break;
				
			case Operations.LessThanOrEqual:
				if (ver1 <= ver2)
					retVal = TestResult.True;			
				break;
				
			case Operations.GreaterThan:
				if (ver1 > ver2)
					retVal = TestResult.True;			
				break;
				
			case Operations.GreaterThanOrEqual:
				if (ver1 >= ver2)
					retVal = TestResult.True;
				break;
			
			default:
				retVal = TestResult.NotApplicable;
				break;
			}
			
			return retVal;
		}

		public override bool Equals (object o)
		{
			if (o == null)
				return false;
				
			if (o is EntityObject) {
			
				EntityObject eo = (EntityObject) o;
				if (this.Name == eo.Name)			
					return true;
				
				return false;
			}
			
			return false;
		}

		public bool Equals (EntityObject o)
		{
			if (o == null)
				return false;
				
			if (this.Name == o.Name)			
				return true;
				
			return false;
		}

		public override int GetHashCode ()
		{
			return objName.GetHashCode ();
		}

		public void Write (SystemCharacteristicsDocument scDoc, XmlElement parentItem)
		{
			XmlElement item = scDoc.CreateElement (objName);
			item.InnerText = objValue;
			
			string dtName = Vocabulary.GetDataType (dataType);
			if (dtName != "string")
				item.SetAttribute ("datatype", dtName);
				
			string stName = Vocabulary.GetStatusFlag (status);
			if (stName != "exists")
				item.SetAttribute ("status", stName);
				
			parentItem.AppendChild (item);
		}
		
		public string Name
		{
			get { return objName; }
		}

		public bool Nil
		{
			get { return nil; }
		}

		public Operations Operation
		{
			get { return operation; }
			set { operation = value; }
		}

		public OvalDataTypes DataType
		{
			get { return dataType; }
		}
				
		public Checks Check
		{
			get { return checkType; }
		}

		public bool IsTestObject
		{
			get { return isTestObject; }
		}
		
		public StatusFlags Status
		{
			get { return status; }
		}
				
		public string Value
		{
			get { return objValue; }
			set { objValue = value; }
		}

		public string VariableReference
		{
			get { return varRef; }
		}		
	}
		
	public class ObjectElement
	{
		string name;
		string id;
		string version;
		string xmlns;
		
		Dictionary <string,EntityObject> entities;
		Dictionary <string,string> behaviors;
		List <string> notes;

		SetElement setElement;

		public ObjectElement (string objectName, string objectID, string objectVersion, string objectNS)
		{				
			name = objectName;
			id = objectID;
			version = objectVersion;
			xmlns = objectNS;

			CheckID (id);
			
			notes = new List <string> ();
			entities = new Dictionary <string,EntityObject> ();
			behaviors = new Dictionary <string,string> ();
		}
	
		public ObjectElement (XmlElement objElement)
		{
			if (objElement == null)
				throw new ArgumentNullException ("objElement");

			name = objElement.Name;				
			id = objElement.GetAttribute ("id");
			version = objElement.GetAttribute ("version");
			xmlns = objElement.GetAttribute ("xmlns");
			
			CheckID (id);
			
			entities = new Dictionary <string,EntityObject> ();
			behaviors = new Dictionary <string,string> ();
			
			foreach (XmlElement ee in objElement.ChildNodes) {
				
				if (ee.NodeType != XmlNodeType.Element)
					continue;
					
				switch (ee.Name) {
				
				case "oval-def:behaviors":
				case "behaviors":
					GetBehaviors (ee);
					break;

				case "oval-def:notes":
				case "notes":
					GetNotes (ee);
					break;
					
				case "oval-def:set":
				case "set":
					setElement = new SetElement (ee);
					break;
					
				default:
					entities[ee.Name] = new EntityObject (ee);
					break;
				}
			}
		}
		
		static void CheckID (string id)
		{
			Match m = Regex.Match (id, Vocabulary.OBJECT_ID_PATTERN);
			if (!m.Success) {
				throw new ArgumentException("Invalid format for object ID", "id");
			}
		}
		
		void GetBehaviors (XmlElement behaviorsElement)
		{
			if (behaviorsElement == null)
				throw new ArgumentNullException ("behaviorsElement");
				
			int numAttrs = behaviorsElement.Attributes.Count;
			for (int i = 0; i < numAttrs; i++) {
				XmlAttribute xa = behaviorsElement.Attributes[i]; 
				behaviors.Add (xa.Name, xa.Value);
			}
		}
		
		void GetNotes (XmlElement notesElement)
		{
			if (notesElement == null)
				return;
				
			XmlNodeList nl = notesElement.GetElementsByTagName ("note");
			if (nl.Count > 0) {
				foreach (XmlElement e in nl)
					notes.Add (e.InnerText);
			}
		}
		
		public Dictionary <string,string> Behaviors
		{
			get { return behaviors; }
		}
		
		public Dictionary <string,EntityObject> Entities 
		{
			get { return entities; }
		}

		public string Name
		{
			get { return name; }
		}
		
		public string ID
		{
			get { return id; }
		}
		
		public string[] Notes
		{
			get { return notes.ToArray(); }
		}
		
		public SetElement Set
		{
			get { return setElement; }
		}
		
		public string Version
		{
			get { return version; }
		}
		
		public string XmlNs
		{
			get { return xmlns; }
		}
	}
		
	public class StateElement
	{
		string name;
		string id;
		string version;
		string xmlns;

		Dictionary <string,EntityObject> entities;
		List <string> notes;

		public StateElement (string stateName, string stateID, string stateVersion, string stateNS)
		{				
			name = stateName;
			id = stateID;
			version = stateVersion;
			xmlns = stateNS;

			Match m = Regex.Match (id, Vocabulary.STATE_ID_PATTERN);
			if (!m.Success)
				throw new ArgumentException("Invalid format for state ID", "id");
			
			notes = new List <string> ();
			entities = new Dictionary <string,EntityObject> ();
		}
		
		public StateElement (XmlElement steElement)
		{
			if (steElement == null)
				throw new ArgumentNullException ("steElement");

			name = steElement.Name;				
			id = steElement.GetAttribute ("id");
			version = steElement.GetAttribute ("version");
			xmlns = steElement.GetAttribute ("xmlns");

			Match m = Regex.Match (id, Vocabulary.STATE_ID_PATTERN);
			if (!m.Success) {
				throw new ArgumentException("Invalid format for state ID", "id");
			}
			
			notes = new List <string> ();
			entities = new Dictionary <string,EntityObject> ();
			
			foreach (XmlElement ee in steElement.ChildNodes) {
				if (ee.NodeType != XmlNodeType.Element)
					continue;
				
				if (ee.Name == "notes")
					GetNotes (ee);
				else
					entities[ee.Name] = new EntityObject (ee);
			}
		}

		void GetNotes (XmlElement notesElement)
		{
			if (notesElement == null)
				return;
				
			XmlNodeList nl = notesElement.GetElementsByTagName ("note");
			if (nl.Count > 0) {
				foreach (XmlElement e in nl)
					notes.Add (e.InnerText);
			}
		}

		public string Name
		{
			get { return name; }
		}
			
		public string[] Notes
		{
			get { return notes.ToArray(); }
		}

		public string ID
		{
			get { return id; }
		}
		
		public string Version
		{
			get { return version; }
		}
		
		public string XmlNs
		{
			get { return xmlns; }
		}
		
		public Dictionary <string,EntityObject> Entities 
		{
			get { return entities; }
		}
	}
	
	public class TestElement
	{
		string name;
		string id;
		string version;
		string xmlns;
		string comment;		
		Checks check;		
		
		string objRef;
		string stateRef;
	
		List <string> notes;
	
		public TestElement (XmlElement tstElement)
		{
			if (tstElement == null)
				throw new ArgumentNullException ("tstElement");

			notes = new List <string> ();

			name = tstElement.Name;				
			id = tstElement.GetAttribute ("id");
			version = tstElement.GetAttribute ("version");
			xmlns = tstElement.GetAttribute ("xmlns");
			comment = tstElement.GetAttribute ("comment"); 
			check = Vocabulary.GetCheck (tstElement.GetAttribute ("check"));

			Match m = Regex.Match (id, Vocabulary.TEST_ID_PATTERN);
			if (!m.Success) {
				throw new ArgumentException("Invalid format for test ID", "id");
			}
			
			foreach (XmlElement e in tstElement.ChildNodes) {
				if (e.NodeType != XmlNodeType.Element)
					continue;
					
				if (e.Name == "object")
					objRef = e.GetAttribute ("object_ref");
				else if (e.Name == "state")
					stateRef = e.GetAttribute ("state_ref");
				else if (e.Name == "notes")
					GetNotes (e);
			}
		}

		public TestElement (string id)
		{
			Match m = Regex.Match (id, Vocabulary.TEST_ID_PATTERN);
			if (!m.Success) {
				throw new ArgumentException("Invalid format for test ID", "id");
			}
			
			this.id = id;
		}

		public TestElement (string name, string id, string objRef, string stateRef)
		{
			Match m = Regex.Match (id, Vocabulary.TEST_ID_PATTERN);
			if (!m.Success)
				throw new ArgumentException("Invalid format for test ID", "id");
			
			m = Regex.Match (objRef, Vocabulary.OBJECT_ID_PATTERN);
			if (!m.Success)
				throw new ArgumentException("Invalid format for object ID", "id");			

			m = Regex.Match (stateRef, Vocabulary.STATE_ID_PATTERN);
			if (!m.Success)
				throw new ArgumentException("Invalid format for state ID", "id");
			
			this.name = name;
			this.id = id;
			this.ObjectRef = objRef;
			this.StateRef = stateRef;
		}

		void GetNotes (XmlElement notesElement)
		{
			if (notesElement == null)
				return;
				
			XmlNodeList nl = notesElement.GetElementsByTagName ("note");
			if (nl.Count > 0) {
				foreach (XmlElement e in nl)
					notes.Add (e.InnerText);
			}
		}

		public string Name
		{
			get { return name; }
			set { name = value; }
		}
			
		public string[] Notes
		{
			get { return notes.ToArray(); }
		}
			
		public string ID
		{
			get { return id; }
			set { id = value; }
		}
		
		public string Version
		{
			get { return version; }
			set { version = value; }
		}
		
		public string XmlNs
		{
			get { return xmlns; }
			set { xmlns = value; }
		}
		
		public string Comment
		{
			get { return comment; }
			set { comment = value; }
		}
		
		public Checks Check
		{
			get { return check; }
			set { check = value; }
		}
		
		public string ObjectRef
		{
			get { return objRef; }
			set { objRef = value; }
		}
		
		public string StateRef
		{
			get { return stateRef; }
			set { stateRef = value; }
		}
	}
	
	public class CriteriaElement
	{
		string critOperator;
		Dictionary <string,string> criterions;

		public CriteriaElement ()
		{
			critOperator = "AND";
			criterions = new Dictionary <string,string> ();
		}
			
		public CriteriaElement (XmlElement critElement)
		{
			criterions = new Dictionary <string,string> ();
			
			critOperator = critElement.GetAttribute ("operator");
			foreach (XmlElement e in critElement.ChildNodes) {
				if (e.NodeType != XmlNodeType.Element)
					continue;
					
				if (e.Name == "criterion")
					criterions.Add (e.GetAttribute ("test_ref"), e.GetAttribute ("comment")); 
			}
		}
		
		public string Operator
		{
			get { return critOperator; }
			set { critOperator = value; }
		}
		
		public Dictionary <string,string> Criterions
		{
			get { return criterions; }
		}
	}
	
	public class DefinitionElement
	{
		string id;
		string version;
		string defclass;
	
		Dictionary<string,string> metadata;
		List <CriteriaElement> criteria;
	
		public DefinitionElement (string id, string version, string defclass)
		{
			this.id = id;
			this.version = version;
			this.defclass = defclass;

			Match m = Regex.Match (id, Vocabulary.DEFINITION_ID_PATTERN);
			if (!m.Success)
				throw new ArgumentException("Invalid format for definition ID", "id");
			
			metadata = new Dictionary<string,string> ();
			criteria = new List <CriteriaElement> (); 		
		}
	
		public DefinitionElement (XmlElement defElement)
		{
			if (defElement == null)
				throw new ArgumentNullException ("defElement");
				
			id = defElement.GetAttribute ("id");
			version = defElement.GetAttribute ("version");
			defclass = defElement.GetAttribute ("class");

			Match m = Regex.Match (id, Vocabulary.DEFINITION_ID_PATTERN);
			if (!m.Success)
				throw new ArgumentException("Invalid format for test ID", "id");
			
			metadata = new Dictionary<string,string> ();
			criteria = new List <CriteriaElement> (); 

			foreach (XmlElement e in defElement.ChildNodes) {
				if (e.NodeType != XmlNodeType.Element)
					continue;
					
				if (e.Name == "metadata")
					ReadMetaData (e);
				else if (e.Name == "criteria")
					criteria.Add (new CriteriaElement (e));
			}			
		}
		
		void ReadMetaData (XmlElement metadataElement)
		{
			if (metadataElement == null)
				return;
				
			foreach (XmlElement e in metadataElement.ChildNodes) {
				if (e.NodeType != XmlNodeType.Element)
					continue;
					
				if (e.Name == "affected") {
					metadata.Add ("affected_family", e.GetAttribute ("family"));
					foreach (XmlElement c in e.ChildNodes) {
						if (c.NodeType != XmlNodeType.Element)
							continue;
							
						if (c.Name == "platform")
							metadata.Add ("affected_platform", c.InnerText);
						else if (c.Name == "product")
							metadata.Add ("affected_product", c.InnerText);
					}
					
				} else if (e.Name == "reference") {				
					metadata.Add ("reference_source", e.GetAttribute ("source")); 
					metadata.Add ("reference_id", e.GetAttribute ("ref_id"));
					metadata.Add ("reference_url", e.GetAttribute ("ref_url"));
				} else {
					metadata.Add (e.Name, e.InnerText.Trim());
				}
			}
		}
		
		public void AddCriteria (CriteriaElement ce)
		{
			if (ce == null)
				throw new ArgumentNullException ("ce == null");
				
			criteria.Add (ce);
		}
		
		public string GetMetaData (string name)
		{
			if (name == null)
				return "";
				
			if (metadata.ContainsKey (name))
				return metadata [name];
				
			return "";
		}
		
		public void SetMetaData (string attrName, string attrValue)
		{
			metadata [attrName] = attrValue;
		}
		
		public string ID
		{
			get { return id; }
			set { id = value; }
		}
		
		public string Class
		{
			get { return defclass; }
		}
		
		public CriteriaElement[] Criteria
		{
			get { return criteria.ToArray(); }
		}
		
		public string Version
		{
			get { return version; }
		}
	}
	
	public class DefinitionsDocument : Document
	{
		Dictionary <string,DefinitionElement> definitions;
		Dictionary <string,ObjectElement> objects;
		Dictionary <string,TestElement> tests;
		Dictionary <string,StateElement> states;
		Dictionary <string,Variable> variables;
	
		public DefinitionsDocument () : base ()
		{
			definitions = new Dictionary <string,DefinitionElement> ();
			tests = new Dictionary <string,TestElement> ();
			objects  = new Dictionary <string,ObjectElement> ();
			states  = new Dictionary <string,StateElement> ();
			variables = new Dictionary <string,Variable> ();
		}		

		public new void Load (string fileName)
		{
			base.Load (fileName);
			
			Collect (false);
		}

		public void LoadAll (string fileName)
		{
			base.Load (fileName);
			
			Collect (true);			
		}

		public new void LoadXml (string xmlData)
		{
			base.LoadXml (xmlData);
			
			Collect (false);
		}

		public void Update ()
		{
			doc = new XmlDocument ();
			doc.AppendChild (doc.CreateNode (XmlNodeType.XmlDeclaration,"",""));
			
			XmlElement oval_definitions = doc.CreateElement ("oval_definitions");
			oval_definitions.SetAttribute ("xmlns", "http://oval.mitre.org/XMLSchema/oval-definitions-5");
			oval_definitions.SetAttribute ("xmlns:oval", "http://oval.mitre.org/XMLSchema/oval-common-5");
			oval_definitions.SetAttribute ("xmlns:oval-def", "http://oval.mitre.org/XMLSchema/oval-definitions-5");
			oval_definitions.SetAttribute ("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
			doc.AppendChild (oval_definitions);	
			
			oval_definitions.AppendChild (CreateGenerator ());
			
			XmlElement definitionsElement = doc.CreateElement ("definitions");
			CreateDefinitions (definitionsElement);
			oval_definitions.AppendChild (definitionsElement);

			XmlElement testsElement = doc.CreateElement ("tests");
			CreateTests (testsElement);
			oval_definitions.AppendChild (testsElement);

			XmlElement objectsElement = doc.CreateElement ("objects");
			CreateObjects (objectsElement);
			oval_definitions.AppendChild (objectsElement);

			XmlElement statesElement = doc.CreateElement ("states");
			CreateStates (statesElement);
			oval_definitions.AppendChild (statesElement);

			XmlElement variablesElement = doc.CreateElement ("variables");
			CreateVariables (variablesElement);
			oval_definitions.AppendChild (variablesElement);
		}

		public override bool Validate (string fileName, string schemaDirectory)
		{
			Log.Debug ("Attempting to validate {0}", fileName);
/*		
            try {
            
                string fullSchemaPath = Path.Combine(schemaDirectory, this.SchemaVersion);

                XmlSchemaSet sc = new XmlSchemaSet();
                sc.Add(null, Path.Combine(fullSchemaPath, "oval-definitions-schema.xsd"));
                sc.Add(null, Path.Combine(fullSchemaPath, "oval-common-schema.xsd"));
                sc.Add(null, Path.Combine(fullSchemaPath, "apache-definitions-schema.xsd"));
                sc.Add(null, Path.Combine(fullSchemaPath, "windows-definitions-schema.xsd"));
                sc.Add(null, Path.Combine(fullSchemaPath, "independent-definitions-schema.xsd"));
                sc.Add(null, Path.Combine(fullSchemaPath, "solaris-definitions-schema.xsd"));
                sc.Add(null, Path.Combine(fullSchemaPath, "unix-definitions-schema.xsd"));
                sc.Add(null, Path.Combine(fullSchemaPath, "hpux-definitions-schema.xsd"));
                sc.Add(null, Path.Combine(fullSchemaPath, "linux-definitions-schema.xsd"));
                sc.Add(null, Path.Combine(fullSchemaPath, "xmldsig-core-schema.xsd"));

                if (this.SchemaVersion == "5.1" || this.SchemaVersion == "5.2") {
                    sc.Add(null, Path.Combine(fullSchemaPath, "oval-variables-schema.xsd"));
                }

                XmlReaderSettings settings = new XmlReaderSettings();
                settings.CloseInput = true;
                settings.ConformanceLevel = ConformanceLevel.Document;
                settings.IgnoreComments = true;
                settings.Schemas = sc;
                settings.ValidationEventHandler += OnValidationEvent;
                settings.ValidationFlags = XmlSchemaValidationFlags.ProcessSchemaLocation;
                settings.ValidationType = ValidationType.Schema;

                XmlReader reader = XmlReader.Create(fileName, settings);
                while (reader.Read()) ;

            } catch (Exception ex) {
            
                Log.Error(ex.Message);
                Log.Debug(ex);
                return false;
            }
*/
			return isValid;
		}

		void Collect (bool includeDefinitions)
		{
			if (includeDefinitions)
				CollectDefinitions ();
				
			CollectTests ();
			CollectObjects ();
			CollectStates ();
		}

		void CollectDefinitions ()
		{
			XmlNodeList nl = doc.GetElementsByTagName ("definitions");
			if (!(nl.Count > 0))
				return;
				
			XmlElement defElement = (XmlElement) nl[0];
			foreach (XmlElement te in defElement.ChildNodes) {
				if (te.NodeType == XmlNodeType.Element && te.Name == "definition") {
					try {
						DefinitionElement de = new DefinitionElement (te);
						definitions [de.ID] = de;
					} catch (ArgumentException ae) {
						Log.Debug (ae);
					}
				}
			}
		}

		void CollectTests ()
		{
			XmlNodeList nl = doc.GetElementsByTagName ("tests");
			if (!(nl.Count > 0))
				return;
			
			XmlElement tstElement = (XmlElement) nl[0];		
			foreach (XmlNode node in tstElement.ChildNodes) {
				if (node.NodeType != XmlNodeType.Element)
					continue;
			
				XmlElement te = (XmlElement) node;
				if (te.Name.EndsWith ("_test")) {
					try {
						TestElement testElement = new TestElement (te);
						tests [testElement.ID] = testElement;
					} catch (ArgumentException ae) {
						Log.Debug (ae);
					}
				}
			}
		}

		void CollectObjects ()
		{			
			XmlNodeList nl = doc.GetElementsByTagName ("objects");
			if (!(nl.Count > 0))
				return;
			
			XmlElement objElement = (XmlElement) nl[0];
			foreach (XmlNode node in objElement.ChildNodes) {
				if (node.NodeType != XmlNodeType.Element)
					continue;
			
				XmlElement oe = (XmlElement) node;
				if (oe.Name.EndsWith ("_object")) {
					ObjectElement objectElement = new ObjectElement (oe);
					objects [objectElement.ID] = objectElement;				
				}
			}
			
			Global.Objects = objects;
		}

		void CollectStates ()
		{			
			XmlNodeList nl = doc.GetElementsByTagName ("states");
			if (!(nl.Count > 0))
				return;
			
			XmlElement statesElement = (XmlElement) nl[0];
			foreach (XmlNode node in statesElement.ChildNodes) {
				if (node.NodeType != XmlNodeType.Element)
					continue;
			
				XmlElement se = (XmlElement) node;
				if (se.Name.EndsWith ("_state")) {
					StateElement stateElement = new StateElement (se);
					states [stateElement.ID] = stateElement;
				}
			}
				
			Global.States = states;
		}

		public void CollectVariables (PlatformCreateTestHandler createTest)
		{
			variables = new Dictionary <string,Variable> ();
		
			XmlNodeList nl = doc.GetElementsByTagName ("variables");
			if (!(nl.Count > 0))
				return;
			
			XmlElement varElement = (XmlElement) nl[0];
			foreach (XmlNode node in varElement.ChildNodes) {
				if (node.NodeType != XmlNodeType.Element)
					continue;
				
				XmlElement ve = (XmlElement) node;
				if (ve.Name.EndsWith ("_variable")) {
					Variable ovalVariable = VariableFactory.Create (ve, createTest);
					variables [ovalVariable.ID] = ovalVariable;
				}
			}
				
			Global.Variables = variables;
		}

		void CreateDefinitions (XmlElement definitionsElement)
		{
			foreach (string k in definitions.Keys) {
				DefinitionElement de = definitions [k];
				
				XmlElement defElement = doc.CreateElement ("definition");
				defElement.SetAttribute ("id", de.ID);
				defElement.SetAttribute ("version", de.Version);
				defElement.SetAttribute ("class", de.Class);
				
				XmlElement metadata = doc.CreateElement ("metadata");
				defElement.AppendChild (metadata);
				
				XmlElement title = doc.CreateElement ("title");
				title.InnerText = de.GetMetaData ("title");
				metadata.AppendChild (title);

				XmlElement affected = doc.CreateElement ("affected");
				affected.SetAttribute ("family", de.GetMetaData("affected_family"));
				metadata.AppendChild (affected);

				XmlElement platform = doc.CreateElement ("platform");
				platform.InnerText = de.GetMetaData ("affected_platform");
				affected.AppendChild (platform);

				XmlElement product = doc.CreateElement ("product");
				product.InnerText = de.GetMetaData ("affected_product");
				affected.AppendChild (product);
				
				XmlElement reference = doc.CreateElement ("reference");
				reference.SetAttribute ("source", de.GetMetaData ("reference_source"));
				reference.SetAttribute ("ref_id", de.GetMetaData ("reference_id"));
				reference.SetAttribute ("ref_url", de.GetMetaData ("reference_url"));
				metadata.AppendChild (reference);
				
				XmlElement description = doc.CreateElement ("description");
				description.InnerText = de.GetMetaData ("description");
				metadata.AppendChild (description);

				// FIXME: support multiple criteria
				CriteriaElement ce = de.Criteria[0];
				
				XmlElement criteria = doc.CreateElement ("criteria");
				criteria.SetAttribute ("operator", ce.Operator);
				defElement.AppendChild (criteria);
				
				foreach (string ck in ce.Criterions.Keys) {
					string cv = ce.Criterions[ck];

					XmlElement criterion = doc.CreateElement ("criterion");
					criterion.SetAttribute ("test_ref", ck);
					criterion.SetAttribute ("comment", cv);
					criteria.AppendChild (criterion);					
				}
					
				definitionsElement.AppendChild (defElement);
			}
		}

		void CreateTests (XmlElement testsElement)
		{
			foreach (string k in tests.Keys) {
				TestElement te = tests [k];
				
				XmlElement newTest = doc.CreateElement (te.Name);
				newTest.SetAttribute ("id", te.ID);
				newTest.SetAttribute ("version", te.Version);
				newTest.SetAttribute ("check", Vocabulary.GetCheck (te.Check));
				newTest.SetAttribute ("comment", te.Comment);
				newTest.SetAttribute ("xmlns", te.XmlNs);

				XmlElement objRef = doc.CreateElement ("object");
				objRef.SetAttribute ("object_ref", te.ObjectRef);
				newTest.AppendChild (objRef);
					
				XmlElement stateRef = doc.CreateElement ("state");
				stateRef.SetAttribute ("state_ref", te.StateRef);
				newTest.AppendChild (stateRef);
				
				testsElement.AppendChild (newTest);
			}
		}

		void CreateObjects (XmlElement objectsElement)
		{
			foreach (string k in objects.Keys) {
				ObjectElement oe = objects [k];

				XmlElement obj = doc.CreateElement (oe.Name);
				obj.SetAttribute ("id", oe.ID);
				obj.SetAttribute ("version", oe.Version);
				obj.SetAttribute ("xmlns", oe.XmlNs);
				
				foreach (string ek in oe.Entities.Keys) {
					EntityObject eo = oe.Entities[ek];				
					XmlElement e = doc.CreateElement (eo.Name);
					e.InnerText = eo.Value;					
					obj.AppendChild (e);
				}
				
				objectsElement.AppendChild (obj);
			}
		}

		void CreateStates (XmlElement statesElement)
		{
			foreach (string k in states.Keys) {
				StateElement se = states [k];

				XmlElement ste = doc.CreateElement (se.Name);
				ste.SetAttribute ("id", se.ID);
				ste.SetAttribute ("version", se.Version);
				ste.SetAttribute ("xmlns", se.XmlNs);
				
				foreach (string ek in se.Entities.Keys) {
					EntityObject eo = se.Entities[ek];				
					XmlElement e = doc.CreateElement (eo.Name);
					e.SetAttribute ("datatype", Vocabulary.GetDataType (eo.DataType));
					e.SetAttribute ("operation", Vocabulary.GetOperation (eo.Operation));					
					e.InnerText = eo.Value;					
					ste.AppendChild (e);
				}
				
				statesElement.AppendChild (ste);
			}
		}

		void CreateVariables (XmlElement variablesElement)
		{
//			foreach (string k in variables.Keys) {
//				Variable v = variables [k];
//		
//				variablesElement.AppendChild (foo);
//			}
		}

		XmlElement CreateGenerator ()
		{
			XmlElement generator = doc.CreateElement ("generator");
			
			XmlElement product_name = doc.CreateElement ("oval:product_name", "http://oval.mitre.org/XMLSchema/oval-common-5");
			product_name.InnerText = Defines.PACKAGE;
			generator.AppendChild (product_name);
			
			XmlElement product_version = doc.CreateElement ("oval:product_version", "http://oval.mitre.org/XMLSchema/oval-common-5");
			product_version.InnerText = Defines.VERSION;
			generator.AppendChild (product_version);

			XmlElement schema_version = doc.CreateElement ("oval:schema_version", "http://oval.mitre.org/XMLSchema/oval-common-5");
			schema_version.InnerText = "5.0";
			generator.AppendChild (schema_version);
			
			XmlElement timestamp = doc.CreateElement ("oval:timestamp", "http://oval.mitre.org/XMLSchema/oval-common-5");
			timestamp.InnerText = String.Format ("{0:yyyy-MM-ddTHH:mm:ss}", DateTime.Now);
			generator.AppendChild (timestamp);

			return generator;
		}

		public Dictionary <string,DefinitionElement> Definitions
		{
			get { return definitions; }
		}

		public Dictionary <string,ObjectElement> Objects
		{
			get { return objects; }
		}
		
		public Dictionary <string,TestElement> Tests
		{
			get { return tests; }
		}
		
		public Dictionary <string,StateElement> States
		{
			get { return states; }
		}
		
		public Dictionary <string,Variable> Variables
		{
			get { return variables; }
		}
	}

	public struct SystemInterfaceData 
	{
		public string Name;
		public string IpAddress;
		public string MacAddress;
	}

	public abstract class SystemInfo
	{
		public SystemInfo ()
		{
		}
		
		public abstract string OsName { get; }
		public abstract string OsVersion { get; }
		public abstract string OsArchitecture { get; }
		public abstract string PrimaryHostName { get; }
		public abstract SystemInterfaceData[] Interfaces { get; }
	}
	
	public class CollectedObjectElement
	{
		string id;
		string version;
		Flags flag;
		List<string> itemReferences;
	
		public CollectedObjectElement (XmlElement element)
		{
			if (element == null)
				throw new ArgumentNullException ("element");

			itemReferences = new List<string> ();

			id = element.GetAttribute ("id");
			version = element.GetAttribute ("version");
			flag = Vocabulary.GetFlag (element.GetAttribute ("flag"));
			
			foreach (XmlElement e in element.ChildNodes) {
				if (e.NodeType != XmlNodeType.Element)
					continue;
					
				if (e.Name == "reference")
					itemReferences.Add (e.GetAttribute ("item_ref"));
			}
		}
	
		public string ID
		{
			get { return id; }
		}
		
		public string Version
		{
			get { return version; }
		}
		
		public Flags Flag
		{
			get { return flag; }
		}
		
		public string[] References
		{
			get { return itemReferences.ToArray (); }
		}
	}

	public class SystemCharacteristicsDocument : Document
	{
		XmlElement system_characteristics;
		XmlElement system_info;
		XmlElement collected_objects;
		XmlElement system_data;
		
		Dictionary <string, CollectedObjectElement> collectedObjects;
		Dictionary <string, CollectedItem> systemData;
		Dictionary <string, string> knownObjects;
		
		public Dictionary <string, string> WrongPlatform;
		
		public SystemCharacteristicsDocument () : base ()
		{			 
			WrongPlatform = new Dictionary<string,string> ();
			knownObjects = new Dictionary<string,string> ();
		}
		
		public void Create ()
		{
			XmlNode xmlnode = doc.CreateNode (XmlNodeType.XmlDeclaration,"","");
			doc.AppendChild (xmlnode);
	
			system_characteristics = doc.CreateElement("oval_system_characteristics");
			system_characteristics.SetAttribute ("xmlns", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			system_characteristics.SetAttribute ("xmlns:oval", "http://oval.mitre.org/XMLSchema/oval-common-5");
			system_characteristics.SetAttribute ("xmlns:oval-def", "http://oval.mitre.org/XMLSchema/oval-definitions-5");
			system_characteristics.SetAttribute ("xmlns:oval-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			system_characteristics.SetAttribute ("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
//			system_characteristics.SetAttribute ("xsi:schemaLocation", "http://oval.mitre.org/XMLSchema/oval-definitions-5 oval-definitions-schema.xsd");		
			doc.AppendChild (system_characteristics);
			
			WriteGenerator ();

			system_info = doc.CreateElement ("system_info", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			system_characteristics.AppendChild (system_info);
			
			collected_objects = doc.CreateElement ("collected_objects");
			system_characteristics.AppendChild (collected_objects);
			
			system_data = doc.CreateElement ("system_data");
			system_characteristics.AppendChild (system_data);		
		}

		public void AddHostIdentification (string agentID)
		{
			if (agentID == null)
				throw new ArgumentNullException ("agentID");

			XmlElement hostid = doc.CreateElement ("host_identification", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			hostid.InnerText = agentID;
			system_info.AppendChild (hostid);
		}		
		
		public void AddNamespace (string nsName, string nsUri)
		{
			if (nsName == null)
				throw new ArgumentNullException (nsName);

			if (nsUri == null)
				throw new ArgumentNullException (nsUri);				
				
			system_characteristics.SetAttribute (nsName, nsUri);
		}
		
		public void AddSystemData (XmlElement itemElement)
		{
			if (itemElement == null)
				throw new ArgumentNullException ("itemElement");
				
			system_data.AppendChild (itemElement);
		}
		
		public void AddToCollectedObject (string objectID, int itemReference)
		{
			XmlNodeList nl = doc.GetElementsByTagName ("collected_objects");
			if (!(nl.Count > 0))
				return;
				
			XmlElement element = (XmlElement) nl[0];
			foreach (XmlElement oe in element.ChildNodes) {
				if (oe.NodeType != XmlNodeType.Element) 
					continue;
			
				string id = oe.GetAttribute ("id");
				if (id == objectID) {
					bool alreadyHere = false;

                    foreach (XmlElement roe in oe.ChildNodes) {
                        if (roe.NodeType != XmlNodeType.Element)
                            continue;

                        if (itemReference.ToString() == roe.GetAttribute("item_ref"))
                            alreadyHere = true;
                    }

                    if (!alreadyHere) {
                        XmlElement refElement = doc.CreateElement("reference");
                        refElement.SetAttribute("item_ref", itemReference.ToString());
                        oe.AppendChild(refElement);
                    }		
				}
			}
		}
		
		public void AddToCollectedObject(string objectID, string variableID, string variableValue)
        {
            XmlNodeList nl = doc.GetElementsByTagName("collected_objects");
            if (!(nl.Count > 0))
                return;

            XmlElement element = (XmlElement)nl[0];
            foreach (XmlElement oe in element.ChildNodes)
            {
                if (oe.NodeType != XmlNodeType.Element)
                    continue;

                string id = oe.GetAttribute("id");
                if (id == objectID)
                {
                	XmlNodeList vars = oe.GetElementsByTagName("variable_value");
                    foreach (XmlElement vv in vars) {
                        string vid = vv.GetAttribute("variable_id");
                        if (vid == variableID)
                            return;
                    }
                
                    XmlElement varElement = doc.CreateElement("variable_value");
                    varElement.SetAttribute("variable_id", variableID);
                    varElement.InnerText = variableValue;
                    oe.AppendChild(varElement);
                }
            }
        }
		
		public void WriteCollectedObject(string objectID, string version, Flags flag)
        {
            if (!knownObjects.ContainsKey(objectID)) {
				XmlElement objectElement = doc.CreateElement("object");
				objectElement.SetAttribute("id", objectID);
				objectElement.SetAttribute("version", version);
				objectElement.SetAttribute("flag", Vocabulary.GetFlag(flag));
				collected_objects.AppendChild(objectElement);

				knownObjects.Add(objectID, objectID);
            }
        }
		
		public void WriteCollectedObject (string objectID, string version, CollectedItem ci)
		{
			XmlElement objectElement = doc.CreateElement ("object");			
			objectElement.SetAttribute ("id", objectID);
			objectElement.SetAttribute ("version", version);
			objectElement.SetAttribute ("flag", Vocabulary.GetFlag (ci.Flag));
			collected_objects.AppendChild (objectElement);
			
			if (ci.Message != null) {
				XmlElement xe = doc.CreateElement ("oval-sc:message", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
				xe.SetAttribute ("level", Vocabulary.GetMessageLevel (ci.MessageLevel));			
				xe.InnerText = ci.Message;
					
				objectElement.AppendChild (xe);
			}

			if (ci.Flag == Flags.Complete) {
				XmlElement refElement = doc.CreateElement ("reference");
				refElement.SetAttribute ("item_ref", ci.ID.ToString ());
				objectElement.AppendChild (refElement);
			}
		}
		
		void GetCollectedObjects ()
		{
			collectedObjects  = new Dictionary <string, CollectedObjectElement> ();
			
			XmlNodeList nl = doc.GetElementsByTagName ("collected_objects");
			if (!(nl.Count > 0))
				return;
				
			XmlElement element = (XmlElement) nl[0];
			foreach (XmlElement oe in element.ChildNodes)
				if (oe.NodeType == XmlNodeType.Element) {
					CollectedObjectElement coElement = new CollectedObjectElement (oe); 
					collectedObjects [coElement.ID] = coElement;
				}		
		}
		
		void GetSystemData ()
		{
			systemData = new Dictionary <string, CollectedItem> ();

			XmlNodeList nl = doc.GetElementsByTagName ("system_data");
			if (!(nl.Count > 0))
				return;
				
			XmlElement sdElement = (XmlElement) nl[0];
			foreach (XmlElement sdc in sdElement.ChildNodes) {
			
				int id = int.Parse (sdc.GetAttribute ("id"));
				CollectedItem ci = new CollectedItem (sdc.Name, id, sdc.GetAttribute ("xmlns")); 
			
				foreach (XmlElement oe in sdc.ChildNodes)
					if (oe.NodeType == XmlNodeType.Element) {

						EntityObject e = new EntityObject ( 
							oe.Name, 
							oe.InnerText,
							Vocabulary.GetDataType (oe.GetAttribute ("datatype")),
							Vocabulary.GetStatusFlag (oe.GetAttribute ("status")),
							false);
						
						ci.Flag = Flags.Complete;
						ci.AddItem (e);					
						systemData [ci.ID.ToString()] = ci;
					}
			}
		}
		
		void WriteGenerator ()
		{
			XmlElement generator;		
			generator = doc.CreateElement ("generator", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			
			XmlElement product_name = doc.CreateElement ("oval:product_name", "http://oval.mitre.org/XMLSchema/oval-common-5");
			product_name.InnerText = Defines.PACKAGE;
			generator.AppendChild (product_name);
			
			XmlElement product_version = doc.CreateElement ("oval:product_version", "http://oval.mitre.org/XMLSchema/oval-common-5");
			product_version.InnerText = Defines.VERSION;
			generator.AppendChild (product_version);

			XmlElement schema_version = doc.CreateElement ("oval:schema_version", "http://oval.mitre.org/XMLSchema/oval-common-5");
			schema_version.InnerText = "5.0";
			generator.AppendChild (schema_version);
			
			XmlElement timestamp = doc.CreateElement ("oval:timestamp", "http://oval.mitre.org/XMLSchema/oval-common-5");
			timestamp.InnerText = String.Format ("{0:yyyy-MM-ddTHH:mm:ss}", DateTime.Now);
			generator.AppendChild (timestamp);
						
			system_characteristics.AppendChild (generator);		
		}

		public void WriteSystemInfo (SystemInfo platformSystemInfo)
		{
			XmlElement os_name = doc.CreateElement ("os_name", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			os_name.InnerText = platformSystemInfo.OsName;
			system_info.AppendChild (os_name);
			
			XmlElement os_version = doc.CreateElement ("os_version", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			os_version.InnerText = platformSystemInfo.OsVersion;
			system_info.AppendChild (os_version);

			XmlElement arch = doc.CreateElement ("architecture", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			arch.InnerText = platformSystemInfo.OsArchitecture;
			system_info.AppendChild (arch);
				
			XmlElement primary_host_name = doc.CreateElement ("primary_host_name", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			primary_host_name.InnerText = platformSystemInfo.PrimaryHostName;
			system_info.AppendChild (primary_host_name);
			
			XmlElement interfaces = doc.CreateElement ("interfaces", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");

			foreach (SystemInterfaceData ifd in platformSystemInfo.Interfaces) {
			
				XmlElement xe = doc.CreateElement ("interface", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
				
				XmlElement name = doc.CreateElement ("interface_name", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
				name.InnerText = ifd.Name;
				xe.AppendChild (name);

				XmlElement ip = doc.CreateElement ("ip_address", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
				ip.InnerText = ifd.IpAddress;
				xe.AppendChild (ip);
				
				XmlElement mac = doc.CreateElement ("mac_address", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
				mac.InnerText = ifd.MacAddress;
				xe.AppendChild (mac);
				
				interfaces.AppendChild (xe);
			}

			system_info.AppendChild (interfaces);
		}
		
		public Dictionary <string, CollectedObjectElement> CollectedObjects
		{
			get {
				if (collectedObjects == null)
					GetCollectedObjects();
					
				return collectedObjects;
			}
		}

		public Dictionary <string, CollectedItem> SystemData
		{
			get {
				if (systemData == null)
					GetSystemData();
					
				return systemData;
			}
		}		
	}
	
	public class ResultsDocument : Document
	{
		XmlElement oval_results;
		XmlElement oval_definitions;
		XmlElement results;
		XmlElement system;
		XmlElement definitions;
		XmlElement tests;
		XmlElement directives;
		XmlElement oval_system_characteristics;
		
		Dictionary<string,string> testCache;
		
		public ResultsDocument () : base ()
		{
			testCache = new Dictionary<string,string> ();
		
			doc.AppendChild (doc.CreateNode (XmlNodeType.XmlDeclaration,"",""));
			
			WriteHeader ();
			WriteGenerator ();

			SetDirectives(new ResultDirectives());
						
			oval_definitions = doc.CreateElement ("oval_definitions", "http://oval.mitre.org/XMLSchema/oval-definitions-5");
			oval_results.AppendChild (oval_definitions);
			
			results = doc.CreateElement ("results");
			oval_results.AppendChild (results);			

			system = doc.CreateElement ("system");			
			definitions = doc.CreateElement ("definitions");
			system.AppendChild (definitions);
			
			tests = doc.CreateElement ("tests");
			system.AppendChild (tests);

			oval_system_characteristics = doc.CreateElement ("oval_system_characteristics", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			system.AppendChild (oval_system_characteristics);			
			
			results.AppendChild (system);			
		}

		public void AddTest (XmlElement testElement)
		{
			if (testElement == null)
				throw new ArgumentNullException ("testElement");

			string testID = testElement.GetAttribute ("test_id");
			if (!testCache.ContainsKey (testID)) {	
				testCache.Add (testID, testID);				
				tests.AppendChild (testElement);
			}
		}

		public void CopyDefinitions (DefinitionsDocument definitionDocument)
		{
			XmlElement genElement =  definitionDocument.GetElement ("generator");
			if (genElement != null)
				oval_definitions.AppendChild (this.CopyElement (genElement));
			
			XmlElement defElement = definitionDocument.GetElement ("definitions");
			if (defElement != null)
				oval_definitions.AppendChild (this.CopyElement (defElement));

			XmlElement tstElement = definitionDocument.GetElement ("tests");
			if (tstElement != null)
				oval_definitions.AppendChild (this.CopyElement (tstElement));

			XmlElement objElement = definitionDocument.GetElement ("objects");
			if (objElement != null)
				oval_definitions.AppendChild (this.CopyElement (objElement));

			XmlElement steElement = definitionDocument.GetElement ("states");
			if (steElement != null)
				oval_definitions.AppendChild (this.CopyElement (steElement));

			XmlElement varElement = definitionDocument.GetElement ("variables");
			if (varElement != null)
				oval_definitions.AppendChild (this.CopyElement (varElement));
		}

		public void CopySystemCharacteristics (SystemCharacteristicsDocument scDocument)
		{
			XmlElement genElement = scDocument.GetElement ("generator");
			if (genElement != null)
				oval_system_characteristics.AppendChild (this.CopyElement (genElement));
			
			XmlElement siElement = scDocument.GetElement ("system_info");
			if (siElement != null)
				oval_system_characteristics.AppendChild (this.CopyElement (siElement));

			XmlElement cobElement = scDocument.GetElement ("collected_objects");
			if (cobElement != null)			
				oval_system_characteristics.AppendChild (this.CopyElement (cobElement));

			XmlElement sdElement = scDocument.GetElement ("system_data");
			if (sdElement != null)
				oval_system_characteristics.AppendChild (this.CopyElement (sdElement));		
		}

		public void Save (Config currentConfiguration)
		{
			if (currentConfiguration.ResultsFileFormat == ResultOutputTypes.XML) {

				doc.Save (currentConfiguration.ResultsFile);

			} else {

                XslCompiledTransform xslt = new XslCompiledTransform();
                string xslFile = Path.Combine (currentConfiguration.XmlDirectory, "results_to_html.xsl");

                if (!File.Exists (xslFile)) {
                	Log.Error ("Unable to load HTML stylesheet {0}", xslFile);
                	return;
                }
                
                xslt.Load (xslFile); 

                string tmp = Path.GetTempFileName();
                File.Delete(tmp);
                doc.Save(tmp);

                xslt.Transform(tmp, currentConfiguration.ResultsFile);
                
                File.Delete(tmp);
			}		
		}

		public void SetDirectives (ResultDirectives resultDirectives)
		{
			directives = doc.CreateElement ("directives");						
			
			XmlElement definition_true = doc.CreateElement ("definition_true");
			definition_true.SetAttribute ("content", Vocabulary.GetContent (resultDirectives.DefinitionTrue.Content));
			definition_true.SetAttribute ("reported", resultDirectives.DefinitionTrue.IsReported.ToString().ToLower());
			directives.AppendChild (definition_true);
			
			XmlElement definition_false = doc.CreateElement ("definition_false");
			definition_false.SetAttribute ("content", Vocabulary.GetContent (resultDirectives.DefinitionFalse.Content));
			definition_false.SetAttribute ("reported", resultDirectives.DefinitionFalse.IsReported.ToString().ToLower());
			directives.AppendChild (definition_false);
			
			XmlElement definition_unknown = doc.CreateElement ("definition_unknown");
			definition_unknown.SetAttribute ("content", Vocabulary.GetContent (resultDirectives.DefinitionUnknown.Content));
			definition_unknown.SetAttribute ("reported", resultDirectives.DefinitionUnknown.IsReported.ToString().ToLower());
			directives.AppendChild (definition_unknown);
			
			XmlElement definition_error = doc.CreateElement ("definition_error");
			definition_error.SetAttribute ("content", Vocabulary.GetContent (resultDirectives.DefinitionError.Content));
			definition_error.SetAttribute ("reported", resultDirectives.DefinitionError.IsReported.ToString().ToLower());
			directives.AppendChild (definition_error);
			
			XmlElement definition_not_evaluated = doc.CreateElement ("definition_not_evaluated");
			definition_not_evaluated.SetAttribute ("content", Vocabulary.GetContent (resultDirectives.DefinitionNotEvaluated.Content));
			definition_not_evaluated.SetAttribute ("reported", resultDirectives.DefinitionNotEvaluated.IsReported.ToString().ToLower());
			directives.AppendChild (definition_not_evaluated);
			
			XmlElement definition_not_applicable = doc.CreateElement ("definition_not_applicable");
			definition_not_applicable.SetAttribute ("content", Vocabulary.GetContent (resultDirectives.DefinitionNotApplicable.Content));
			definition_not_applicable.SetAttribute ("reported", resultDirectives.DefinitionNotApplicable.IsReported.ToString().ToLower());
			directives.AppendChild (definition_not_applicable);
			
			oval_results.AppendChild (directives);
		}

		void WriteHeader ()
		{
			oval_results = doc.CreateElement ("oval_results");
			oval_results.SetAttribute ("xmlns", "http://oval.mitre.org/XMLSchema/oval-results-5");
			oval_results.SetAttribute ("xmlns:oval", "http://oval.mitre.org/XMLSchema/oval-common-5");
			oval_results.SetAttribute ("xmlns:oval-def", "http://oval.mitre.org/XMLSchema/oval-definitions-5");
			oval_results.SetAttribute ("xmlns:oval-sc", "http://oval.mitre.org/XMLSchema/oval-system-characteristics-5");
			oval_results.SetAttribute ("xmlns:oval-res", "http://oval.mitre.org/XMLSchema/oval-results-5");
			oval_results.SetAttribute ("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
			
			doc.AppendChild (oval_results);
		}
		
		void WriteGenerator ()
		{
			XmlElement generator;		
			generator = doc.CreateElement ("generator");
			
			XmlElement product_name = doc.CreateElement ("oval:product_name", "http://oval.mitre.org/XMLSchema/oval-common-5");
			product_name.InnerText = Defines.PACKAGE;
			generator.AppendChild (product_name);
			
			XmlElement product_version = doc.CreateElement ("oval:product_version", "http://oval.mitre.org/XMLSchema/oval-common-5");
			product_version.InnerText = Defines.VERSION;
			generator.AppendChild (product_version);

			XmlElement schema_version = doc.CreateElement ("oval:schema_version", "http://oval.mitre.org/XMLSchema/oval-common-5");
			schema_version.InnerText = "5.0";
			generator.AppendChild (schema_version);
			
			XmlElement timestamp = doc.CreateElement ("oval:timestamp", "http://oval.mitre.org/XMLSchema/oval-common-5");
			timestamp.InnerText = String.Format ("{0:yyyy-MM-ddTHH:mm:ss}", DateTime.Now);
			generator.AppendChild (timestamp);
						
			oval_results.AppendChild (generator);		
		}

	}	
}
