// 
// Repository.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Security.Cryptography;
using System.Text;
using System.Xml;

using ICSharpCode.SharpZipLib.BZip2;
using ICSharpCode.SharpZipLib.GZip;
using ICSharpCode.SharpZipLib.Zip;

namespace Sussen.Oval
{
	public static class Repository
	{	
		static string workDirectory = null;
	
		public static string GetResource (string location)
		{
			if (location == null)
				throw new ArgumentNullException (location);		
		
			if (location.StartsWith ("http"))
				return GetRemoteResource (location);
			else	
				return location;
		}
	
		public static void Initialize (string workingDirectory)
		{
			workDirectory = workingDirectory;
		}
			
		static string GetRemoteResource (string url)
		{
			Log.Debug ("Downloading file {0}", url);
		
			string saveFile = Path.GetTempFileName ();
			File.Delete (saveFile);
					
			if (File.Exists (saveFile))
				File.Delete (saveFile);
			
			try {
				WebClient wc = new WebClient ();
				wc.DownloadFile (url, saveFile);
			} catch (Exception e) {
				Log.Debug (e);
				return null;
			}
			
			if (url.EndsWith (".zip"))
				return UnzipFile (saveFile);
			else if (url.EndsWith (".gz"))
				return GzipDecompressFile (saveFile);
			else if (url.EndsWith (".bz2"))
				return Bzip2DecompressFile (saveFile);
			else {
				string newPath = Path.Combine (workDirectory, saveFile);
				File.Move (saveFile, newPath);
				
				return newPath;
			}
		}
				
		static string UnzipFile (string zipName)
		{
			ZipInputStream s = new ZipInputStream(File.OpenRead(zipName));			
			
			ZipEntry entry;			
			string unzipName = null;			
			
			while ((entry = s.GetNextEntry()) != null) {
					
				unzipName = Path.Combine(workDirectory, entry.Name);
				FileStream streamWriter = File.Create(unzipName);
					
				int size = 2048;
				byte[] data = new byte[2048];
				while (true) {
					size = s.Read(data, 0, data.Length);
					if (size > 0) {
						streamWriter.Write(data, 0, size);
					} else {
						break;
					}
				}
					
				streamWriter.Close();
			}

			s.Close();
			
			return unzipName;
		}
		
		static string GzipDecompressFile (string gzName)
		{
			Stream gs = new GZipInputStream (File.OpenRead(gzName));
			
			string newFile = Path.Combine(workDirectory, Path.GetFileNameWithoutExtension(gzName));			
			FileStream fs = File.Create (newFile);
			
			int size = 2048;
			byte[] writeData = new byte[2048];
			while (true) {
				size = gs.Read(writeData, 0, size);
				if (size > 0) {
					fs.Write(writeData, 0, size);
				} else {
					break;
				}
			}
			
			fs.Close ();
			gs.Close();			
			
			return newFile;
		}
		
		static string Bzip2DecompressFile (string bz2Name)
		{
			string newFile = Path.Combine(workDirectory, Path.GetFileNameWithoutExtension(bz2Name));
			FileStream fs = File.Create(newFile);
			
			BZip2.Decompress(File.OpenRead(bz2Name), fs);
		
			fs.Close ();
			
			return newFile;
		}
	}
}