//
// Test.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

namespace Sussen.Oval
{
	public enum DefinitionResults {
		True,
		False,
		Unknown,
		Error,
		NotEvaluated,
		NotApplicable
	};

	public enum TestOperations {
		Equals,
		NotEqual,
		GreaterThan,
		LessThan,
		GreaterThanOrEqual,
		LessThanOrEqual,
		BitwiseAnd,
		BitwiseOr,
		PatternMatch
	};

	public enum TestOperators {
		And,
		Or,
		Xor
	};

	public class CollectedItem
	{
		string name;
		int id;
		string xmlns;
		string message;
		MessageLevels level;
		Flags flag;
		
		List<EntityObject> entities;

		public CollectedItem (string itemName, int itemID, string xmlNamespace)
		{
			name = itemName;
			id = itemID;
			xmlns = xmlNamespace;
			
			entities = new List<EntityObject> ();
		}
	
		public CollectedItem (string itemName, int itemID, string xmlNamespace, Flags itemFlag)
		{
			name = itemName;
			id = itemID;
			xmlns = xmlNamespace;
			flag = itemFlag;
			
			entities = new List<EntityObject> ();
		}
		
		public void AddItem (EntityObject item)
		{
			entities.Add (item);
		}
		
		public override string ToString ()
		{
			StringBuilder sb = new StringBuilder ();
			sb.AppendFormat ("Item: {0}\n", name);
			sb.AppendFormat ("ID: {0}\n", id);
			
			if (message != null)
				sb.AppendFormat ("Message: {0}\n", message);
			
			foreach (EntityObject oe in entities)
				sb.AppendFormat ("{0}:{1}\tStatus: {2}\n", oe.Name, oe.Value, Vocabulary.GetStatusFlag (oe.Status));

			return sb.ToString();			
		}
		
		public void Write (SystemCharacteristicsDocument scDoc)
		{
			XmlElement xe = scDoc.CreateElement (name);
			xe.SetAttribute ("id", id.ToString());
			xe.SetAttribute ("xmlns", xmlns);
			
			foreach (EntityObject eo in entities) {
				eo.Write (scDoc, xe);
			}
			
			scDoc.AddSystemData (xe);
		}
				
		public Flags Flag
		{
			get { return flag; }
			set { flag = value; }
		}
		
		public int ID
		{
			get { return id; }
			set { id = value; }
		}
		
		public EntityObject[] Items
		{
			get { return entities.ToArray (); }
		}
		
		public string Message
		{
			get { return message; }
			set { message = value; }
		}
		
		public MessageLevels MessageLevel
		{
			get { return level; }
			set { level = value; }
		}
	}

	public abstract class Test
	{
		protected int itemCount;		
		protected Dictionary <string,Operations[]> objectData;
		protected Dictionary <string,Operations[]> stateData;
		
		public Test ()
		{
			itemCount = 0;
			objectData = new Dictionary <string,Operations[]> ();
			stateData = new Dictionary <string,Operations[]> ();
		}

		// FIXME: should remove this
		protected static bool CompareItems (string lhs, string rhs, Operations operation)
		{
			bool retVal = false;
			
			switch (operation) {

			case Operations.Equals:
				if (lhs == rhs)
					retVal = true;
				break;
					
			case Operations.NotEqual:
				if (lhs != rhs)
					retVal = true;
				break;

			// FIXME: convert OVAL regex format (posix) to MS.NET format (pcre)				
			case Operations.PatternMatch:
				try {
				
					Regex r = new Regex (lhs, RegexOptions.None);
					Match m = r.Match (rhs);
					if (m.Success)
						retVal = true;			
					
				} catch (Exception e) {
					Log.Debug (e);
				}
				break;
				
			default:
				throw new ArgumentOutOfRangeException (Vocabulary.GetOperation (operation));
			}
			
			return retVal;
		}

		public Operations[] GetObjectOperations (string objectName)
		{
			if (!objectData.ContainsKey (objectName))
				return null;
				
			return objectData [objectName];
		}

		public Operations[] GetStateOperations (string stateName)
		{
			if (!stateData.ContainsKey (stateName))
				return null;
				
			return stateData [stateName];
		}

		public string[] ObjectNames 
		{
			get {
				List<string> names = new List<string> ();
				foreach (string s in objectData.Keys)
					names.Add (s);
				
				names.Sort ();
				return names.ToArray ();
			}
		}
		
		public string[] StateNames 
		{
			get {
				List<string> names = new List<string> ();
				foreach (string s in stateData.Keys)
					names.Add (s);
						
				names.Sort ();
				return names.ToArray ();
			}
		}
			
		public abstract CollectedItem[] Run (TestElement testElement, ObjectElement objectElement);
	}

	public class TestException : Exception
	{
		string id;
		
		public TestException () : base ()
		{
		}

		public TestException (string message) : base (message)
		{
		}

		public TestException (string testID, string message) : base (message)
		{
			id = testID;
		}
		
		public string ID
		{
			get { return id; }
		}
	}
}
