// 
// TestAnalyzer.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text.RegularExpressions;
using System.Xml;

namespace Sussen.Oval
{
	public class TestAnalyzer : Analyzer
	{
		OvalDocuments docs;
	
		public TestAnalyzer ()
		{
		}

		public TestResult Run (TestElement testElement, CollectedItem[] items, DefinitionsDocument doc)
		{
			if (testElement == null)
				throw new ArgumentNullException ("testElement");

			TestResult testResult = TestResult.Unknown;			
			List <TestResult> objectResults = new List <TestResult> ();
			StateElement stateElement = null;
			
			if (testElement.ObjectRef == null) {
				Log.Debug ("Test {0} has null object reference", testElement.ID); 
				return testResult;
			}
			
			if (testElement.StateRef != null)
				stateElement = doc.States [testElement.StateRef];  

			foreach (CollectedItem ci in items) {
				if (ci.Flag == Flags.DoesNotExist) {
					objectResults.Add (TestResult.False);					
				} else if (stateElement != null) {
					TestResult res = CompareCollectedDataToState (stateElement, ci);
					objectResults.Add (res);				
				} else {
			
					TestResult res = TestResult.Unknown;
					foreach (EntityObject ie in ci.Items)
						if (ie.Status == StatusFlags.Exists)
							res = TestResult.True;
						else
							res = TestResult.False;

					objectResults.Add (res);			
				}			
			}

			testResult = AnalyzeResultsByCheck (objectResults, Vocabulary.GetCheck (testElement.Check));
			
			return testResult;
		}

		public TestResult Run (TestElement testElement, OvalDocuments ovalDocuments)
		{
			if (testElement == null)
				throw new ArgumentNullException ("testElement");

			if (ovalDocuments == null)
				throw new ArgumentNullException ("ovalDocuments");
				
			docs = ovalDocuments;
			TestResult testResult = TestResult.Unknown;
			
			List <TestResult> objectResults = new List <TestResult> ();
			StateElement stateElement = null;
			
			if (testElement.ObjectRef == null) {
				Log.Debug ("Test {0} has null object reference", testElement.ID); 
				return testResult;
			}
				
			if (testElement.StateRef != null)
				stateElement = docs.Definitions.States [testElement.StateRef];

			CollectedObjectElement coElement = null;
			ObjectElement oe = docs.Definitions.Objects [testElement.ObjectRef];

			XmlElement test = docs.Results.CreateElement ("test"); 
			test.SetAttribute ("test_id", testElement.ID);
			test.SetAttribute ("version", testElement.Version);
			test.SetAttribute ("variable_instance", "1");
			test.SetAttribute ("check", Vocabulary.GetCheck (testElement.Check));
			
			if (oe.Set != null) {
				foreach (string s in oe.Set.ObjectReferences) {
				
					coElement = docs.SystemCharacteristics.CollectedObjects [s];
					
					if (coElement == null || coElement.References.Length == 0)
						continue;

					foreach (string r in coElement.References) {
					
						CollectedItem ci = docs.SystemCharacteristics.SystemData [r];
						if (CheckSetFilter (oe.Set, ci))
							continue;
						
						if (stateElement != null) {
							TestResult res = CompareCollectedDataToState (stateElement, ci);
							
							XmlElement tested_item = docs.Results.CreateElement ("tested_item");
							tested_item.SetAttribute ("item_id", ci.ID.ToString());
							tested_item.SetAttribute ("result", Vocabulary.GetTestResult(res));			
							test.AppendChild (tested_item);	
							
							objectResults.Add (res);				
						} else {
					
							TestResult res = TestResult.Unknown;

							foreach (EntityObject eo in ci.Items)
								if (eo.Status == StatusFlags.Exists)
									res = TestResult.True;
								else
									res = TestResult.False;

							XmlElement tested_item = docs.Results.CreateElement ("tested_item");
							tested_item.SetAttribute ("item_id", ci.ID.ToString());
							tested_item.SetAttribute ("result", Vocabulary.GetTestResult(res));			
							test.AppendChild (tested_item);	
							
							objectResults.Add (res);			
						}
					}
				}
				
			} else {
				
				if (!docs.SystemCharacteristics.CollectedObjects.ContainsKey (testElement.ObjectRef)) {				
					if (docs.SystemCharacteristics.WrongPlatform.ContainsKey (testElement.ID))
						testResult = TestResult.NotApplicable;				
					else if (testElement.Check == Checks.NoneExist)
						testResult = TestResult.True;
					else
						testResult = TestResult.False;
						
					return testResult;				
				}

				coElement = docs.SystemCharacteristics.CollectedObjects [testElement.ObjectRef];					
				if (coElement.References.Length == 0) {
					Log.Debug ("No references found for collected object [{0}] (test: {1})", testElement.ObjectRef, testElement.ID);
					testResult = AnalyzeObjectFlag (coElement.Flag, testElement.Check); 
					return testResult;
				}

				foreach (string r in coElement.References) {										
					CollectedItem ci = docs.SystemCharacteristics.SystemData [r];
					
					if (ci.Flag == Flags.DoesNotExist) {
					
						XmlElement tested_item = docs.Results.CreateElement ("tested_item");
						tested_item.SetAttribute ("item_id", ci.ID.ToString());
						tested_item.SetAttribute ("result", Vocabulary.GetTestResult(TestResult.False));			
						test.AppendChild (tested_item);
							
						objectResults.Add (TestResult.False);					
					
					} else if (stateElement != null) {
						TestResult res = CompareCollectedDataToState (stateElement, ci);
						
						XmlElement tested_item = docs.Results.CreateElement ("tested_item");
						tested_item.SetAttribute ("item_id", ci.ID.ToString());
						tested_item.SetAttribute ("result", Vocabulary.GetTestResult(res));			
						test.AppendChild (tested_item);
							
						objectResults.Add (res);				
					} else {
				
						TestResult res = TestResult.Unknown;

						foreach (EntityObject ie in ci.Items)
							if (ie.Status == StatusFlags.Exists)
								res = TestResult.True;
							else
								res = TestResult.False;

						XmlElement tested_item = docs.Results.CreateElement ("tested_item");
						tested_item.SetAttribute ("item_id", ci.ID.ToString());
						tested_item.SetAttribute ("result", Vocabulary.GetTestResult(res));			
						test.AppendChild (tested_item);	

						objectResults.Add (res);			
					}
				}
				
			}

			testResult = AnalyzeResultsByCheck (objectResults, Vocabulary.GetCheck (testElement.Check));

			Log.Debug ("Analyzed test: [{0}] version: [{1}] check: [{2}] result: [{3}]",
				testElement.ID, testElement.Version, Vocabulary.GetCheck (testElement.Check), 
				Vocabulary.GetTestResult (testResult));
			
			test.SetAttribute ("result", Vocabulary.GetTestResult (testResult));
			docs.Results.AddTest (test);
			
			return testResult;
		}

		TestResult AnalyzeObjectFlag (Flags flag, Checks check)
		{
			switch (flag) {
			
			case Flags.Error:
				return TestResult.Error;

            case Flags.Incomplete:
                return TestResult.Unknown;					
			
			case Flags.NotCollected:
				return TestResult.NotEvaluated;
				
			case Flags.NotApplicable:
				return TestResult.NotApplicable;
							
			default:
                break;
			}

            switch (check) {

			case Checks.All:
				return TestResult.False;

			case Checks.AtLeastOne:
				return TestResult.False;

			case Checks.NoneExist:
				return TestResult.True;

			case Checks.OnlyOne:
				return TestResult.False;
            }

            return TestResult.Unknown;
		}

		bool CheckSetFilter (SetElement setElement, CollectedItem systemData)
		{
			TestResult res = TestResult.Unknown;
		
			foreach (string f in setElement.Filters) {
				StateElement se = docs.Definitions.States [f];
				if (se == null)
					continue;
					
				int trueCount = 0;
				int totalCount = se.Entities.Count;
					
				foreach (KeyValuePair<string, EntityObject> kvp in se.Entities) {
					EntityObject lhs = kvp.Value;			
					foreach (EntityObject rhs in systemData.Items) {
						if (lhs.Equals (rhs)) {
							res = lhs.CompareTo (rhs);
							
							if (res == TestResult.True)
								trueCount++;
						}
					}
				}
				
				if (trueCount == totalCount)
					return true;
			}
						
			return false;
		}
				
		TestResult CompareCollectedDataToState (StateElement stateElement, CollectedItem systemData)
		{
			TestResult res = TestResult.Unknown;
			bool allGood = true;

            foreach (EntityObject rhs in systemData.Items) {
                if (stateElement.Entities.ContainsKey(rhs.Name)) {
                    EntityObject lhs = stateElement.Entities[rhs.Name];
                    if (lhs.Equals(rhs))
                        if (lhs.CompareTo(rhs) != TestResult.True)
                            allGood = false;
                }
            }

            if (allGood)
                res = TestResult.True;
            else
                res = TestResult.False;

            return res;

		}
	}
}
