// 
// Variable.cs
// Author: Loren Bandiera
// Copyright 2004-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

namespace Sussen.Oval
{
	public class VariableFactory
	{
		public VariableFactory ()
		{
		}
		
		public static Variable Create (XmlElement variableElement, PlatformCreateTestHandler createTest)
		{
			VariableTypes varType = Vocabulary.GetVariableType (variableElement.Name); 			
			switch (varType) {
			
			case VariableTypes.Constant:
				return new ConstantVariable (variableElement);
				
			case VariableTypes.External:
				return new ExternalVariable (variableElement);
				
			case VariableTypes.Local:
				return new LocalVariable (variableElement, createTest);
				
			default:
				throw new ArgumentOutOfRangeException ("varType");
			}
		}
	}

	public abstract class Variable
	{
		string id;
		int version;
		VariableTypes variableType;
		OvalDataTypes dataType;
		string comment;
		bool isDeprecated;
		
		public Variable (XmlElement variableElement)
		{
			if (variableElement == null)
				throw new ArgumentNullException ("variableElement");
							
			id = variableElement.GetAttribute ("id");
			CheckID (id);

			string strVersion = variableElement.GetAttribute ("version");
			try { 
				version = int.Parse (strVersion); 
			} catch (Exception e) {
				Log.Debug (e);
				throw e;
			}
			
			variableType = Vocabulary.GetVariableType (variableElement.Name);
			dataType = Vocabulary.GetDataType (variableElement.GetAttribute ("datatype"));
			comment = variableElement.GetAttribute ("comment");
			
			string strIsDeprecated = variableElement.GetAttribute ("deprecated");
			if (strIsDeprecated == null) {			
				isDeprecated = false;
			} else {
				try { isDeprecated = bool.Parse (strIsDeprecated); }
				catch { isDeprecated = false; }
			}
		}

		static void CheckID (string id)
		{
			Match m = Regex.Match (id, Vocabulary.VARIABLE_ID_PATTERN);
			if (!m.Success) {
				throw new ArgumentException("Invalid format for variable ID", "id");
			}
		}
		
		public string ID 
		{ 
			get { return id; } 
		}
		
		public int Version
		{
			get { return version; }
		}
		
		public VariableTypes Type
		{
			get { return variableType; }			
		}
		
		public OvalDataTypes DataType
		{
			get { return dataType; }
		}
		
		public string Comment
		{
			get { return comment; }
		}
		
		public bool IsDeprecated
		{
			get { return isDeprecated; }
		}

		public abstract string Value { get ; }
		public abstract string[] ValueArray { get ; }
	}
	
	public class ConstantVariable : Variable
	{
		List<string> values;
		
		public ConstantVariable (XmlElement variableElement) : base (variableElement)
		{
			values = new List<string> ();
		
			foreach (XmlElement ee in variableElement.ChildNodes) {
				if (ee.NodeType == XmlNodeType.Element)
					if (ee.Name == "value")
						values.Add (ee.InnerText);
			}
		}

		public override string Value
		{
			get { 
				if (values.Count == 0)
					return null;
				else
					return (string) values[0];
			}
		}
		
		public override string[] ValueArray
		{
			get { return values.ToArray (); }
		}
	}
	
	public class ExternalVariable : Variable
	{		
		public ExternalVariable (XmlElement variableElement) : base (variableElement)
		{
		}

		public override string Value
		{
			get { return null; }
		}
		
		public override string[] ValueArray
		{
			get { return null; }
		}
	}
	
	public class LocalVariable : Variable
	{
		List<string> values;
		PlatformCreateTestHandler CreateTest;
		
		public LocalVariable (XmlElement variableElement, PlatformCreateTestHandler createTest) : base (variableElement)
		{
			CreateTest = createTest;
			values = new List<string> ();			
			ParseVariable (variableElement, "");
		}

		string ParseVariable (XmlElement variableElement, string operation)
		{
			StringBuilder sb = new StringBuilder ();
			
			List<string> obj = new List<string>();
			List<string> lit = new List<string>();
			List<string> var = new List<string>();
	
			foreach (XmlElement ee in variableElement.ChildNodes) {
				if (ee.NodeType == XmlNodeType.Element) {
					switch (ee.Name) {
					
					case "begin":
						ParseVariable (ee, "begin");
						break;					

					case "concat":
						values.Add (ParseVariable (ee, "concat"));
						break;
						
					case "end":
						ParseVariable (ee, "end");
						break;
						
					case "escape_regex":
						ParseVariable (ee, "escape_regex");
						break;
						
					case "split":
						ParseVariable (ee, "split");
						break;
						
					case "substring":
						ParseVariable (ee, "substring");
						break;
						
					case "object_component":
						obj.Add (GetObjectValue (ee.GetAttribute ("item_field"), ee.GetAttribute ("object_ref")));
						break;
						
					case "variable_component":
						var.Add (GetVariableValue (ee.GetAttribute ("var_ref")));
						break;
						
					case "literal_component":
						lit.Add (ee.InnerText);
						break;
						
					default:
						break;
					}					
				}
			}
	
			switch (operation) {

			case "begin":
				string begstr = variableElement.GetAttribute ("character");
				foreach (string o in obj)
					values.Add (AddBegin (o, begstr));
				foreach (string l in lit)
					values.Add (AddBegin (l, begstr));
				foreach (string v in var)
					values.Add (AddBegin (v, begstr));			
				break;
					
			case "concat":		
				foreach (string o in obj)
					sb.Append (o);
				foreach (string l in lit)
					sb.Append (l);
				foreach (string v in var)
					sb.Append (v);
				break;

			case "end":
				string endstr = variableElement.GetAttribute ("character");
				foreach (string o in obj)
					values.Add (AddEnd (o, endstr));
				foreach (string l in lit)
					values.Add (AddEnd (l, endstr));
				foreach (string v in var)
					values.Add (AddEnd (v, endstr));			
				break;

			case "escape_regex":
				foreach (string o in obj)
					values.Add (Regex.Escape (o));
				foreach (string l in lit)
					values.Add (Regex.Escape (l));
				foreach (string v in var)
					values.Add (Regex.Escape (v));
				break;
			
			case "split":
				string c = variableElement.GetAttribute ("character");
				foreach (string o in obj)
					SplitValues (o, c);
				foreach (string l in lit)
					SplitValues (l, c);
				foreach (string v in var)
					SplitValues (v, c);
				break;

			case "substring":
				string a = variableElement.GetAttribute ("substring_start");
				string b = variableElement.GetAttribute ("substring_length");
				foreach (string o in obj)
					GetSubstring (o, a, b);
				foreach (string l in lit)
					GetSubstring (l, a, b);
				foreach (string v in var)
					GetSubstring(v, a, b);
				break;
					
			default:
				foreach (string o in obj)
					values.Add (o);
				foreach (string l in lit)
					values.Add (l);
				foreach (string v in var)
					values.Add (v);
				break;
			}
				
			return sb.ToString();
		}

		string AddBegin (string item, string begin)
		{
			if (item.StartsWith (begin))
				return item;
				
			return begin + item;
		}
		
		string AddEnd (string item, string end)
		{
			if (item.EndsWith (end))
				return item;
				
			return item + end;
		}

		void GetSubstring (string item, string start, string length)
		{
			try {
			
				// substring_start isn't zero based
				int st = int.Parse (start);
				st -= 1;
				
				int ln = int.Parse (length);
				
				string sub = item.Substring (st, ln);				
				values.Add (sub);
			
			} catch {}
		}

		void SplitValues (string item, string delim)
		{
			string[] split = item.Split (delim.ToCharArray());		
			foreach (string s in split) {
				values.Add (s);
			}
		}

		CollectedItem FilterSet(CollectedItem[] results, StateElement filter)
        {
            foreach (CollectedItem ci in results)
            {
                foreach (EntityObject eo in ci.Items)
                {                    
                    if (filter.Entities.ContainsKey(eo.Name))
                    {
                        EntityObject feo = filter.Entities[eo.Name];
                        if (feo.CompareTo(eo) == TestResult.False)
                            return ci;
                    }
                }
            }

            return null;
        }

		string GetObjectValue (string itemField, string objectRef)
		{
			ObjectElement oe = Global.Objects [objectRef];
            string testName = oe.Name.Replace("_object", "_test");

            if (oe.Entities.Count == 0) {
                ObjectElement oe2 = Global.Objects[oe.Set.ObjectReferences[0]];
                StateElement se2 = Global.States[oe.Set.Filters[0]];

				try {
	                Test test = CreateTest(testName, Global.CurrentSchemaVersion);
    	            CollectedItem[] results = test.Run(null, oe2);

    	            CollectedItem ci = FilterSet(results, se2);
    	            foreach (EntityObject ceo in ci.Items)
    	                if (ceo.Name == itemField)
    	                    return ceo.Value;
    	                    
				} catch (Exception e) {
					Log.Debug (e);
				}
				
                return null;
            }
								
			try {
				
				Test test = CreateTest (testName, Global.CurrentSchemaVersion);
					
				CollectedItem[] results;
				results = test.Run (null, oe);
					
				CollectedItem ci = results[0];
				foreach (EntityObject eo in ci.Items) {
                    if (eo.Name == itemField)
                        return eo.Value;
				}

			} catch (Exception e) {
				Log.Debug (e);
			}
			
			return null;
		}

		string GetVariableValue (string varRef)
		{
			Variable v = Global.Variables [varRef];
			if (v == null)
				return "";
				
			return v.Value;
		}

		public override string Value
		{
			get { 
				if (values.Count == 0)
					return null;
				else
					return (string) values[0];
			}
		}
		
		public override string[] ValueArray
		{
			get { return values.ToArray (); }
		}
	}
}
