#include <stdio.h>
#include <string.h>
#include <rpm/rpmlib.h>
#include <rpm/rpmio.h>
#include <rpm/rpmdb.h>
#include <rpm/rpmts.h>

struct __RpmInfo {
	char	*rpm_name;
	char	*rpm_epoch;
	char	*rpm_version;
	char	*rpm_release;
	char	*rpm_arch;
	char	*rpm_keyid;
};

int rpm_version_compare (const char *a, const char *b)
{
	rpmds ads = rpmdsSingle (RPMTAG_PROVIDENAME, "", a, RPMSENSE_EQUAL);
	rpmds bds = rpmdsSingle (RPMTAG_PROVIDENAME, "", b, RPMSENSE_EQUAL);

	int rcEqual = rpmdsCompare (ads, bds);
	if (rcEqual > 0) {
		/* a == b */
		return 0;
	}
	
	rpmds ads2 = rpmdsSingle (RPMTAG_REQUIRENAME, "", a, RPMSENSE_LESS);
	rpmds bds2 = rpmdsSingle (RPMTAG_PROVIDENAME, "", b, RPMSENSE_GREATER);

	int rcLess = rpmdsCompare (ads2, bds2);
	if (rcLess > 0) {
		/* a < b  */
		return -1;
	} else {
		/* b > a */
		return 1;
	}

	/* should never get here  */
	return -1;
}

/* From RedHat RPM Guide */
int_32 readHeaderInt32 (Header header, int_32 tagId) {
	int_32 type;
	void *pointer;
	int_32 data_size;

	int header_status = headerGetEntry (header, tagId, &type, (void **)&pointer, &data_size);
	if (header_status) {
		if (type == RPM_INT32_TYPE) {
			int_32 *p = (int_32 *) pointer;
			return *p;
		}
	}

	return -1;
}

/* From RedHat RPM Guide */
char* readHeaderString (Header header, int_32 tagId) {

	int_32 type;
	void *pointer;
	int_32 data_size;

	int header_status = headerGetEntry (header, tagId, &type, &pointer, &data_size);
	if (header_status) {
		if (type == RPM_STRING_TYPE) {
			return (char *) pointer;
		}
	}

	return (char*) NULL;
}

/* from librpm */
const char * queryHeader(Header h, const char * qfmt)
{
    const char * errstr = "(unkown error)";
    const char * str;

    str = headerSprintf(h, qfmt, rpmTagTable, rpmHeaderFormats, &errstr);
    if (str == NULL)
	rpmError(RPMERR_QFMT, "incorrect format: %s\n", errstr);

    return str;
}

int rpm_check_installed_package (const char *package_name, 
	const char *check_version)
{
	if (rpmReadConfigFiles ((const char*) NULL, (const char *) NULL)) {
		printf ("cannot read rpm config files\n");
		return -2;
	}

	rpmts ts = rpmtsCreate ();
	rpmdbMatchIterator iter = rpmtsInitIterator (ts, RPMTAG_NAME, package_name, 0);
	Header header;

	char *installed_epoch, *installed_version, *installed_release;
	int ret = -3;

	while ((header = rpmdbNextIterator (iter)) != NULL) {

		int_32 int_installed_epoch = readHeaderInt32 (header, RPMTAG_EPOCH);
		installed_version = readHeaderString (header, RPMTAG_VERSION);
		installed_release = readHeaderString (header, RPMTAG_RELEASE);

		char *pkg_ver;

		if (int_installed_epoch != -1) {
			sprintf (pkg_ver, "%d:%s-%s", 
				int_installed_epoch, 
				installed_version,
				installed_release);
		} else {
			sprintf (pkg_ver, "%s-%s", 
				installed_version,
				installed_release);
		}

		ret = rpm_version_compare (check_version, pkg_ver);
	}

//	rpmdbFreeIterator (iter);
//	rpmtsFree (ts);

	return ret;
}

int rpm_get_info (const char *name, struct __RpmInfo *ribuf)
{
	if (rpmReadConfigFiles ((const char*) NULL, (const char *) NULL)) {
		printf ("cannot read rpm config files\n");
		return -1;
	}

	/* Debian prints out lots of crap on stderr */
	fclose (stderr);

	rpmts ts = rpmtsCreate ();
	rpmdbMatchIterator iter = rpmtsInitIterator (ts, RPMTAG_NAME, name, 0);
	Header header;
	int ret = -1;

	while ((header = rpmdbNextIterator (iter)) != NULL) {

		int e = readHeaderInt32 (header, RPMTAG_EPOCH);
		char *e2;
		sprintf (e2, "%d", e);

		ribuf->rpm_epoch = strdup (e2);
		ribuf->rpm_version = strdup(readHeaderString (header, RPMTAG_VERSION));
		ribuf->rpm_release = strdup(readHeaderString (header, RPMTAG_RELEASE));
		ribuf->rpm_arch = strdup(readHeaderString (header, RPMTAG_ARCH));
		ribuf->rpm_name = strdup (name);
		ribuf->rpm_keyid = strdup (queryHeader (header, "%{SIGGPG:pgpsig}"));

		ret = 0;
	}

	rpmdbFreeIterator (iter);
	rpmtsFree (ts);

	return ret;
}

/* Could probably do IPC through unix sockets or something */
int rpm_dump_package_names (const char *filename)
{
	if (rpmReadConfigFiles ((const char*) NULL, (const char *) NULL)) {
		printf ("cannot read rpm config files\n");
		return -1;
	}

	/* Debian prints out lots of crap on stderr */
	fclose (stderr);		

	FILE *file;
	file = fopen (filename, "w");

	rpmts ts = rpmtsCreate ();
	
	rpmdbMatchIterator iter = rpmtsInitIterator (ts, RPMTAG_NAME, NULL, 0);
	Header header;

	while ((header = rpmdbNextIterator (iter)) != NULL) {
		char *pname = readHeaderString (header, RPMTAG_NAME);
		fprintf (file, "%s\n", pname);
	}

	rpmdbFreeIterator (iter);
	rpmtsFree (ts);

	fclose (file);

	return 0;
}
