//
// XmlFileContentTest.cs - NUnit Test Cases for Sussen.Oval.Unix
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Xml;
using NUnit.Framework;
using Sussen.Oval;
using Sussen.Oval.Independent;
using Sussen.Oval.Unix;

namespace SussenTests.Oval.Independent
{
	[TestFixture]
	public class TextFileContentTestTest
	{
		TextFileContentTest xt;
		ObjectElement oe;

		[SetUp]
		public void SetUp()
		{
			xt = new TextFileContentTest ();

			oe = new ObjectElement ("textfilecontent_object", "oval:org.mmgsecurity:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", ".", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			EntityObject eo2 = new EntityObject ("filename", "sample.lsb.release", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;
		}

		[TearDown]
		public void TearDown ()
		{
			xt = null;
			oe = null;
		}

		[Test]
		public void TestIfUbuntuInstalled ()
		{
			EntityObject eo3 = new EntityObject ("line", "DISTRIB_ID=*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;
			
			CollectedItem[] testResults;
			testResults = xt.Run (null, oe);

			Assert.AreEqual (1, testResults.Length,  "No results found for textfilecontent test");
			CollectedItem c = testResults[0];
			
			bool goodResults = false;

			foreach (EntityObject ie in c.Items) {
				if (ie.Name == "line") {
					if (ie.Value == "DISTRIB_ID=Ubuntu")				
						goodResults = true;
				}
			}

			Assert.IsTrue (goodResults, "Failed to find line in textfilecontent test");
		}
		
		[Test]
		public void TestReleaseVersion ()
		{
			EntityObject eo3 = new EntityObject ("line", "DISTRIB_RELEASE=*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;
			
			CollectedItem[] testResults;
			testResults = xt.Run (null, oe);

			Assert.AreEqual (1, testResults.Length,  "No results found for textfilecontent test");
			CollectedItem c = testResults[0];
			
			bool goodResults = false;

			foreach (EntityObject ie in c.Items) {
				if (ie.Name == "line") {
					if (ie.Value == "DISTRIB_RELEASE=6.06")				
						goodResults = true;
				}
			}

			Assert.IsTrue (goodResults, "Failed to find line in textfilecontent test");
		}
		
		[Test]
		public void TestXmlVersion ()
		{
			Config config;
			DataCollector dc;
			OvalDocuments docs;
			
			config = new Config ("unix");
			config.DefinitionsFile = "textfilecontent_test.xml";

			docs = new OvalDocuments ();
			docs.Definitions.Load (config.DefinitionsFile);
			
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			docs.SystemCharacteristics.Create ();
			docs.SystemCharacteristics.WriteSystemInfo (new UnixSystemInfo());

			dc = new DataCollector (docs);
			dc.CreateTest = new PlatformCreateTestHandler (Interpreter.CreateTest);
			dc.Run ();
			
			DataAnalyzer da = new DataAnalyzer (docs);
			da.Run ();

			XmlNodeList nl = docs.Results.GetElementsByName ("results");
			if (!(nl.Count > 0))
				Assert.Fail ("No results element found in results document");
				
			XmlElement results = (XmlElement) nl[0];
			nl = results.GetElementsByTagName ("definitions");
			if (!(nl.Count > 0))
				Assert.Fail ("No definitions element found in results document");

			bool allGood = false;

			XmlElement definitions = (XmlElement) nl[0];		
			foreach (XmlElement def in definitions.ChildNodes) {
				string id = def.GetAttribute ("definition_id");
				string res = def.GetAttribute ("result");

				if (id == "oval:com.mmgsecurity.oval.testing:def:1") {
					Assert.AreEqual ("true", res, "Invalid test result");					
					allGood = true;
				}
			}

			Assert.AreEqual (true, allGood, "No test results found");			
		}
	}
}
