//
// Sets.cs - NUnit Test Cases for Sussen.Oval
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Xml;
using NUnit.Framework;
using Sussen.Oval;

#if MONO_BUILD
using Sussen.Oval.Unix;
#else
using Sussen.Oval.Windows;
#endif

namespace SussenTests.Oval
{
	[TestFixture]
	public class SetTests
	{
		Config config;
		DataCollector dc;
		OvalDocuments docs;

		[SetUp]
		public void SetUp()
		{
#if MONO_BUILD
			config = new Config ("unix");
            config.DefinitionsFile = "sets.xml";
#else
            config = new Config("windows");
            config.DefinitionsFile = "windows.sets.xml";
#endif
			

			docs = new OvalDocuments ();
			docs.Definitions.Load (config.DefinitionsFile);
		}

		[TearDown]
		public void TearDown ()
		{
			docs = null;
		}

		[Test]
		public void SimpleSetParseTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			
			ObjectElement oe = docs.Definitions.Objects ["oval:com.mmgsecurity:obj:1"];
			Assert.IsNotNull (oe, "oe");
			Assert.IsNotNull (oe.Set, "oe.Set");

			string[] objectReferences = oe.Set.ObjectReferences;
			Assert.IsNotNull (objectReferences, "objectReferences");
			Assert.AreEqual ("oval:com.mmgsecurity:obj:2", objectReferences[0]);
			Assert.AreEqual ("oval:com.mmgsecurity:obj:3", objectReferences[1]);
		}

		[Test]
		public void NestedSetParseTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");
			
			ObjectElement oe = docs.Definitions.Objects ["oval:com.mmgsecurity:obj:4"];
			Assert.IsNotNull (oe, "oe");
			Assert.IsNotNull (oe.Set, "os.Set");

			string[] objectReferences = oe.Set.ObjectReferences;
			Assert.IsNotNull (objectReferences, "objectReferences");
			Assert.AreEqual ("oval:com.mmgsecurity:obj:2", objectReferences[0]);

			Assert.AreEqual (1, oe.Set.NestedSets.Count, "oe.Set.NestedSets.Length");
			objectReferences = oe.Set.NestedSets[0].ObjectReferences;
			Assert.IsNotNull (objectReferences, "objectReferences");
			Assert.AreEqual ("oval:com.mmgsecurity:obj:3", objectReferences[0]);
		}

		[Test]
		public void SimpleSetResultsTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			docs.SystemCharacteristics.Create ();

#if MONO_BUILD
            docs.SystemCharacteristics.WriteSystemInfo (new UnixSystemInfo());
#else
            docs.SystemCharacteristics.WriteSystemInfo(new WindowsSystemInfo());
#endif

			dc = new DataCollector (docs);
			dc.CreateTest = new PlatformCreateTestHandler (Interpreter.CreateTest);
			dc.Run ();
			
			DataAnalyzer da = new DataAnalyzer (docs);
			da.Run ();

			XmlNodeList nl = docs.Results.GetElementsByName ("results");
			if (!(nl.Count > 0))
				Assert.Fail ("No results element found in results document");
				
			XmlElement results = (XmlElement) nl[0];
			nl = results.GetElementsByTagName ("definitions");
			if (!(nl.Count > 0))
				Assert.Fail ("No definitions element found in results document");

			bool allGood = false;

			XmlElement definitions = (XmlElement) nl[0];		
			foreach (XmlElement def in definitions.ChildNodes) {
				string id = def.GetAttribute ("definition_id");
				string res = def.GetAttribute ("result");

				if (id == "oval:com.mmgsecurity:def:2") {
					Assert.AreEqual ("true", res, "Invalid test result");
					allGood = true;
				}
			}

			Assert.AreEqual (true, allGood, "No test results found");
		}

		[Test]
		public void NestedSetResultsTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			docs.SystemCharacteristics.Create ();
#if MONO_BUILD
            docs.SystemCharacteristics.WriteSystemInfo (new UnixSystemInfo());
#else
            docs.SystemCharacteristics.WriteSystemInfo(new WindowsSystemInfo());
#endif

			dc = new DataCollector (docs);
			dc.CreateTest = new PlatformCreateTestHandler (Interpreter.CreateTest);
			dc.Run ();
			
			DataAnalyzer da = new DataAnalyzer (docs);
			da.Run ();

			XmlNodeList nl = docs.Results.GetElementsByName ("results");
			if (!(nl.Count > 0))
				Assert.Fail ("No results element found in results document");
				
			XmlElement results = (XmlElement) nl[0];
			nl = results.GetElementsByTagName ("definitions");
			if (!(nl.Count > 0))
				Assert.Fail ("No definitions element found in results document");

			bool allGood = false;

			XmlElement definitions = (XmlElement) nl[0];		
			foreach (XmlElement def in definitions.ChildNodes) {
				string id = def.GetAttribute ("definition_id");
				string res = def.GetAttribute ("result");

				if (id == "oval:com.mmgsecurity:def:3") {
					Assert.AreEqual ("true", res, "Invalid test result");
					allGood = true;
				}
			}

			Assert.AreEqual (true, allGood, "No test results found");
		}

		[Test]
		public void SetFilterTest ()
		{
			if (!docs.Definitions.CheckSchemaVersion())
				Assert.Fail ("Invalid OVAL definition file");

			docs.SystemCharacteristics.Create ();
#if MONO_BUILD
            docs.SystemCharacteristics.WriteSystemInfo (new UnixSystemInfo());
#else
            docs.SystemCharacteristics.WriteSystemInfo(new WindowsSystemInfo());
#endif

			dc = new DataCollector (docs);
			dc.CreateTest = new PlatformCreateTestHandler (Interpreter.CreateTest);
			dc.Run ();
			
			DataAnalyzer da = new DataAnalyzer (docs);
			da.Run ();

			XmlNodeList nl = docs.Results.GetElementsByName ("results");
			if (!(nl.Count > 0))
				Assert.Fail ("No results element found in results document");
				
			XmlElement results = (XmlElement) nl[0];
			nl = results.GetElementsByTagName ("definitions");
			if (!(nl.Count > 0))
				Assert.Fail ("No definitions element found in results document");

			bool allGood = false;

			XmlElement definitions = (XmlElement) nl[0];		
			foreach (XmlElement def in definitions.ChildNodes) {
				string id = def.GetAttribute ("definition_id");
				string res = def.GetAttribute ("result");

				if (id == "oval:com.mmgsecurity:def:4") {
					Assert.AreEqual ("false", res, "Invalid test result");

					XmlNodeList testList = results.GetElementsByTagName ("tests");
					XmlElement tests = (XmlElement) testList[0];
					foreach (XmlElement t in tests.ChildNodes) {
						string tid = t.GetAttribute ("test_id");
						if (tid == "oval:com.mmgsecurity:tst:3") {
							XmlNodeList il = t.GetElementsByTagName ("tested_item");
#if MONO_BUILD
							Assert.AreEqual (3, il.Count, "Missing expected items");
#else
                            Assert.AreEqual(1, il.Count, "Missing expected items");
#endif			
							allGood = true;
						}
					}
				}
			}

			Assert.AreEqual (true, allGood, "Test results not found");
		}
	}
}
