//
// TortureTests.cs - NUnit Test Cases for Sussen.Oval
//
// Loren Bandiera (lorenb@mmgsecurity.com)
//
// Copyright (C) MMG Security, Inc.
// 

using System;
using System.Xml;
using NUnit.Framework;
using Sussen.Oval;
using Sussen.Oval.Apache;
using Sussen.Oval.Independent;

#if MONO_BUILD
using Sussen.Oval.Unix;
#else
using Sussen.Oval.Windows;
#endif

namespace SussenTests.Oval
{
	[TestFixture]
	public class TortureTests
	{
		Config config;
		TestElement te;

		[SetUp]
		public void SetUp()
		{
#if MONO_BUILD		
			config = new Config ("unix");
#else
			config = new Config ("windows");
#endif
			
			te = new TestElement ("oval:com.mmgsecurity.oval:tst:1");
		}

		[TearDown]
		public void TearDown ()
		{
			config = null;
		}

		[Test]
		[ExpectedException(typeof(XmlException))]
		public void TestInvalidXml ()
		{
			config.DefinitionsFile = "TortureTests.cs";
			new Interpreter (config);
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestNullConfig ()
		{
			new Interpreter (null);
		}

		[Test]
		[ExpectedException(typeof(ArgumentOutOfRangeException))]
		public void TestConfigInvalidFamily ()
		{
			config = new Config ("shackleford");
			new Interpreter (config);
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestConfigNoOvalDocFilename ()
		{
			new Interpreter (config);
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestConfigNoAccessSystemCharacteristicsFile ()
		{
			config.SystemCharacteristicsFile = "/root/bad-file-name.xml";
			config.DefinitionsFile = "oval.v5.test.definitions.xml";
			Interpreter i = new Interpreter (config);
			i.CollectData ();
			i.AnalyzeData ();
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestConfigDoesNotExistSystemCharacteristicsFile ()
		{
			config.SystemCharacteristicsFile = "/does/not/exist/bad-file-name.xml";
			config.DefinitionsFile = "oval.v5.test.definitions.xml";
			Interpreter i = new Interpreter (config);
			i.CollectData ();
			i.AnalyzeData ();
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestConfigBadMD5DefinitionsFile ()
		{
			config.DefinitionsFile = "oval.v5.test.definitions.xml";
			config.DefinitionsFileHash = "notlikely";
			config.VerifyMD5 = true;

			Interpreter i = new Interpreter (config);
			i.CollectData ();
			i.AnalyzeData ();
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestConfigNoAccessResultsFile ()
		{
			config.ResultsFile = "/root/bad-file-name.xml";
			config.DefinitionsFile = "oval.v5.test.definitions.xml";
			Interpreter i = new Interpreter (config);
			i.CollectData ();
			i.AnalyzeData ();
		}

		[Test]
		[ExpectedException(typeof(InterpreterException))]
		public void TestConfigDoesNotExistResultsFile ()
		{
			config.ResultsFile = "/does/not/exist/bad-file-name.xml";
			config.DefinitionsFile = "oval.v5.test.definitions.xml";
			Interpreter i = new Interpreter (config);
			i.CollectData ();
			i.AnalyzeData ();
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void TestDataCollectorNullConfig ()
		{
			new DataCollector (null);
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void TestDataCollectorEmptyConfig ()
		{			 
			OvalDocuments docs = null;
			DataCollector dc = new DataCollector (docs);
			dc.Run ();
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void TestDataAnalyzerNullConfig ()
		{
			new DataAnalyzer (null);
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void TestDataAnalyzerEmptyConfig ()
		{
			OvalDocuments docs = null;
			DataAnalyzer da = new DataAnalyzer (docs);
			da.Run ();
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void TestDefinitionAnalyzerNullArguments ()
		{
			new DefinitionAnalyzer (null, null);
		}

		[Test]
		[ExpectedException(typeof(ArgumentNullException))]
		public void TestTestAnalyzerNullArguments ()
		{
			TestAnalyzer ta = new TestAnalyzer ();
			ta.Run (null, null);
		}
		
		// environment variable
		[Test]
		[ExpectedException(typeof(TestException))]
		public void EnvironmentVariableNoNameTest ()
		{
			EnvironmentVariableTest et = new EnvironmentVariableTest ();
			ObjectElement oe = new ObjectElement ("environmentvariable_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			et.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void EnvironmentVariableNameIsNotStringTest ()
		{
			EnvironmentVariableTest et = new EnvironmentVariableTest ();
			ObjectElement oe = new ObjectElement ("environmentvariable_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("name", "1", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			et.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void EnvironmentVariableNameInvalidOperationTest ()
		{
			EnvironmentVariableTest et = new EnvironmentVariableTest ();
			ObjectElement oe = new ObjectElement ("environmentvariable_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("name", "spoon", Operations.BitwiseAnd, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			et.Run (te, oe);
		}
		
		// filemd5
		[Test]
		[ExpectedException(typeof(TestException))]
		public void FileMD5NoPathTest ()
		{
			FileMD5Test ft = new FileMD5Test ();
			ObjectElement oe = new ObjectElement ("filemd5_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void FileMD5NoFileTest ()
		{
			FileMD5Test ft = new FileMD5Test ();
			ObjectElement oe = new ObjectElement ("filemd5_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void FileMD5PathIsNotStringTest ()
		{
			FileMD5Test ft = new FileMD5Test ();
			ObjectElement oe = new ObjectElement ("filemd5_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");		

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			
	
			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void FileMD5FilenameIsNotStringTest ()
		{
			FileMD5Test ft = new FileMD5Test ();
			ObjectElement oe = new ObjectElement ("filemd5_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "spoon", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			EntityObject eo2 = new EntityObject ("filename", "1", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo2.Name] = eo2;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void FileMD5PathInvalidOperationTest ()
		{
			FileMD5Test ft = new FileMD5Test ();
			ObjectElement oe = new ObjectElement ("filemd5_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.BitwiseAnd, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(TestException))]
		public void FileMD5FilenameInvalidOperationTest ()
		{
			FileMD5Test ft = new FileMD5Test ();
			ObjectElement oe = new ObjectElement ("filemd5_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.GreaterThanOrEqual, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;

			ft.Run (te, oe);
		}
		
		// text file content
		[Test]
		[ExpectedException(typeof(TestException))]
		public void TextFileContentNoPathTest ()
		{
			TextFileContentTest ft = new TextFileContentTest ();
			ObjectElement oe = new ObjectElement ("textfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void TextFileContentNoFilenameTest ()
		{
			TextFileContentTest ft = new TextFileContentTest ();
			ObjectElement oe = new ObjectElement ("textfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(TestException))]
		public void TextFileContentNoLineTest ()
		{
			TextFileContentTest ft = new TextFileContentTest ();
			ObjectElement oe = new ObjectElement ("textfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void TextFileContentPathIsNotStringTest ()
		{
			TextFileContentTest ft = new TextFileContentTest ();
			ObjectElement oe = new ObjectElement ("textfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("line", "^.foo*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void TextFileContentFilenameIsNotStringTest ()
		{
			TextFileContentTest ft = new TextFileContentTest ();
			ObjectElement oe = new ObjectElement ("textfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("line", "^.foo*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void TextFileContentLineIsNotStringTest ()
		{
			TextFileContentTest ft = new TextFileContentTest ();
			ObjectElement oe = new ObjectElement ("textfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("line", "^.foo*", Operations.PatternMatch, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void TextFileContentPathInvalidOperationTest ()
		{
			TextFileContentTest ft = new TextFileContentTest ();
			ObjectElement oe = new ObjectElement ("textfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("line", "^.foo*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(TestException))]
		public void TextFileContentFilenameInvalidOperationTest ()
		{
			TextFileContentTest ft = new TextFileContentTest ();
			ObjectElement oe = new ObjectElement ("textfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.BitwiseOr, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("line", "^.foo*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void TextFileContentLineInvalidOperationTest ()
		{
			TextFileContentTest ft = new TextFileContentTest ();
			ObjectElement oe = new ObjectElement ("textfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("line", "^.foo*", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}
		
		// variable 
		[Test]
		[ExpectedException(typeof(TestException))]
		public void VariableNoVarRefTest ()
		{
			VariableTest et = new VariableTest ();
			TestElement te = new TestElement ("oval:com.mmgsecurity.oval:tst:1");
			ObjectElement oe = new ObjectElement ("variable_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			et.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void VariableVarRefIsNotStringTest ()
		{
			VariableTest et = new VariableTest ();
			TestElement te = new TestElement ("oval:com.mmgsecurity.oval:tst:1");
			ObjectElement oe = new ObjectElement ("variable_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("var_ref", "1", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			et.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void VariableVarRefInvalidOperationTest ()
		{
			VariableTest et = new VariableTest ();
			TestElement te = new TestElement ("oval:com.mmgsecurity.oval:tst:1");
			ObjectElement oe = new ObjectElement ("variable_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("var_ref", "spoon", Operations.BitwiseAnd, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			et.Run (te, oe);
		}
		
		// xml file content
		[Test]
		[ExpectedException(typeof(TestException))]
		public void XmlFileContentNoPathTest ()
		{
			XmlFileContentTest ft = new XmlFileContentTest ();
			ObjectElement oe = new ObjectElement ("xmlfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void XmlFileContentNoFilenameTest ()
		{
			XmlFileContentTest ft = new XmlFileContentTest ();
			ObjectElement oe = new ObjectElement ("xmlfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(TestException))]
		public void XmlFileContentNoXpathTest ()
		{
			XmlFileContentTest ft = new XmlFileContentTest ();
			ObjectElement oe = new ObjectElement ("xmlfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void XmlFileContentPathIsNotStringTest ()
		{
			XmlFileContentTest ft = new XmlFileContentTest ();
			ObjectElement oe = new ObjectElement ("xmlfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("xpath", "//foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void XmlFileContentFilenameIsNotStringTest ()
		{
			XmlFileContentTest ft = new XmlFileContentTest ();
			ObjectElement oe = new ObjectElement ("xmlfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("xpath", "//foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void XmlFileContentXpathIsNotStringTest ()
		{
			XmlFileContentTest ft = new XmlFileContentTest ();
			ObjectElement oe = new ObjectElement ("xmlfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("xpath", "//foo", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void XmlFileContentPathInvalidOperationTest ()
		{
			XmlFileContentTest ft = new XmlFileContentTest ();
			ObjectElement oe = new ObjectElement ("xmlfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("xpath", "//foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(TestException))]
		public void XmlFileContentFilenameInvalidOperationTest ()
		{
			XmlFileContentTest ft = new XmlFileContentTest ();
			ObjectElement oe = new ObjectElement ("xmlfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.BitwiseOr, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("xpath", "//foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void XmlFileContentXpathInvalidOperationTest ()
		{
			XmlFileContentTest ft = new XmlFileContentTest ();
			ObjectElement oe = new ObjectElement ("xmlfilecontent_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("xpath", "//foo", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}

#if MONO_BUILD
		
		// dpkginfo
		[Test]
		[ExpectedException(typeof(TestException))]
		public void DpkgInfoNoNameTest ()
		{
			DpkgInfoTest et = new DpkgInfoTest ();
			TestElement te = new TestElement ("oval:com.mmgsecurity.oval:tst:1");
			ObjectElement oe = new ObjectElement ("dpkginfo_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			et.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void DpkgInfoNameIsNotStringTest ()
		{
			DpkgInfoTest et = new DpkgInfoTest ();
			TestElement te = new TestElement ("oval:com.mmgsecurity.oval:tst:1");
			ObjectElement oe = new ObjectElement ("dpkginfo_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("name", "1", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			et.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void DpkgInfoNameInvalidOperationTest ()
		{
			DpkgInfoTest et = new DpkgInfoTest ();
			TestElement te = new TestElement ("oval:com.mmgsecurity.oval:tst:1");
			ObjectElement oe = new ObjectElement ("dpkginfo_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("name", "spoon", Operations.BitwiseAnd, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			et.Run (te, oe);
		}

		// inetlisteningservers
		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetListeningServersNoProtocolTest ()
		{
			InetListeningServersTest ft = new InetListeningServersTest ();
			ObjectElement oe = new ObjectElement ("inetlisteningservers_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetListeningServersNoLocalAddressTest ()
		{
			InetListeningServersTest ft = new InetListeningServersTest ();
			ObjectElement oe = new ObjectElement ("inetlisteningservers_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("procotol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetListeningServersNoLocalPortTest ()
		{
			InetListeningServersTest ft = new InetListeningServersTest ();
			ObjectElement oe = new ObjectElement ("inetlisteningservers_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("local_address", "^.*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetListeningServersProtocolIsNotStringTest ()
		{
			InetListeningServersTest ft = new InetListeningServersTest ();
			ObjectElement oe = new ObjectElement ("inetlisteningservers_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", "tcp", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("local_address", "^.*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("local_port", "22", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetListeningServersLocalAddressIsNotStringTest ()
		{
			InetListeningServersTest ft = new InetListeningServersTest ();
			ObjectElement oe = new ObjectElement ("inetlisteningservers_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("local_address", "^.*", Operations.PatternMatch, OvalDataTypes.BooleanType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("local_port", "22", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetListeningServersLocalPortIsNotStringTest ()
		{
			InetListeningServersTest ft = new InetListeningServersTest ();
			ObjectElement oe = new ObjectElement ("inetlisteningservers_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("local_address", "^.*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("local_port", "22", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetListeningServersProtocolInvalidOperationTest ()
		{
			InetListeningServersTest ft = new InetListeningServersTest ();
			ObjectElement oe = new ObjectElement ("inetlisteningservers_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", "tcp", Operations.BitwiseAnd, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("local_address", "^.*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("local_port", "22", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetListeningServersLocalAddressInvalidOperationTest ()
		{
			InetListeningServersTest ft = new InetListeningServersTest ();
			ObjectElement oe = new ObjectElement ("inetlisteningservers_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("local_address", "^.*", Operations.BitwiseAnd, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("local_port", "22", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetListeningServersLocalPortInvalidOperationTest ()
		{
			InetListeningServersTest ft = new InetListeningServersTest ();
			ObjectElement oe = new ObjectElement ("inetlisteningservers_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("local_address", "^.*", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			EntityObject eo3 = new EntityObject ("local_port", "22", Operations.BitwiseAnd, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo3.Name] = eo3;

			ft.Run (te, oe);
		}
		
		// file
		[Test]
		[ExpectedException(typeof(NullReferenceException))]
		public void FileNoPathTest ()
		{
			FileTest ft = new FileTest ();
			ObjectElement oe = new ObjectElement ("file_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(NullReferenceException))]
		public void FileNoFilenameTest ()
		{
			FileTest ft = new FileTest ();
			ObjectElement oe = new ObjectElement ("file_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(NullReferenceException))]
		public void FilePathIsNotStringTest ()
		{
			FileTest ft = new FileTest ();
			ObjectElement oe = new ObjectElement ("file_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(NullReferenceException))]
		public void FileFilenameIsNotStringTest ()
		{
			FileTest ft = new FileTest ();
			ObjectElement oe = new ObjectElement ("file_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(NullReferenceException))]
		public void FilePathInvalidOperationTest ()
		{
			FileTest ft = new FileTest ();
			ObjectElement oe = new ObjectElement ("file_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(NullReferenceException))]
		public void FileFilenameInvalidOperationTest ()
		{
			FileTest ft = new FileTest ();
			ObjectElement oe = new ObjectElement ("file_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("filename", "foo", Operations.BitwiseOr, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			ft.Run (te, oe);
		}

		// inetd
		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetdNoProtocolTest ()
		{
			InetdTest ft = new InetdTest ();
			ObjectElement oe = new ObjectElement ("inetd_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetdNoServiceNameTest ()
		{
			InetdTest ft = new InetdTest ();
			ObjectElement oe = new ObjectElement ("inetd_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetdProtocolIsNotStringTest ()
		{
			InetdTest ft = new InetdTest ();
			ObjectElement oe = new ObjectElement ("inetd_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", "tcp", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("service_name", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetdServiceNameIsNotStringTest ()
		{
			InetdTest ft = new InetdTest ();
			ObjectElement oe = new ObjectElement ("inetd_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("service_name", "foo", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetdProtocolInvalidOperationTest ()
		{
			InetdTest ft = new InetdTest ();
			ObjectElement oe = new ObjectElement ("inetd_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", "tcp", Operations.BitwiseOr, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("service_name", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;		

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(TestException))]
		public void InetdServiceNameInvalidOperationTest ()
		{
			InetdTest ft = new InetdTest ();
			ObjectElement oe = new ObjectElement ("inetd_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("protocol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("service_name", "foo", Operations.BitwiseOr, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;	

			ft.Run (te, oe);
		}

		// interface
		[Test]
		[ExpectedException(typeof(TestException))]
		public void InterfaceNoNameTest ()
		{
			InterfaceTest ft = new InterfaceTest ();
			ObjectElement oe = new ObjectElement ("interface_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void InterfaceNameIsNotStringTest ()
		{
			InterfaceTest ft = new InterfaceTest ();
			ObjectElement oe = new ObjectElement ("interface_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("name", "eth0", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void InterfaceNameInvalidOperationTest ()
		{
			InterfaceTest ft = new InterfaceTest ();
			ObjectElement oe = new ObjectElement ("interface_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("name", "eth0", Operations.LessThan, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;			

			ft.Run (te, oe);
		}
		
		// password
		[Test]
		[ExpectedException(typeof(TestException))]
		public void PasswordNoNameTest ()
		{
			PasswordTest ft = new PasswordTest ();
			ObjectElement oe = new ObjectElement ("password_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void PasswordNameIsNotStringTest ()
		{
			PasswordTest ft = new PasswordTest ();
			ObjectElement oe = new ObjectElement ("password_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("username", "rusty", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void PasswordNameInvalidOperationTest ()
		{
			PasswordTest ft = new PasswordTest ();
			ObjectElement oe = new ObjectElement ("password_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("username", "rusty", Operations.LessThan, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;			

			ft.Run (te, oe);
		}

		// process
		[Test]
		[ExpectedException(typeof(TestException))]
		public void ProcessNoCommandTest ()
		{
			ProcessTest ft = new ProcessTest ();
			ObjectElement oe = new ObjectElement ("process_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void ProcessCommandIsNotStringTest ()
		{
			ProcessTest ft = new ProcessTest ();
			ObjectElement oe = new ObjectElement ("process_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("command", "foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void ProcessCommandInvalidOperationTest ()
		{
			ProcessTest ft = new ProcessTest ();
			ObjectElement oe = new ObjectElement ("process_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("command", "foo", Operations.LessThan, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;			

			ft.Run (te, oe);
		}

		// runlevel
		[Test]
		[ExpectedException(typeof(TestException))]
		public void RunLevelNoServiceNameTest ()
		{
			RunLevelTest ft = new RunLevelTest ();
			ObjectElement oe = new ObjectElement ("runlevel_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void RunLevelNoRunLevelTest ()
		{
			RunLevelTest ft = new RunLevelTest ();
			ObjectElement oe = new ObjectElement ("runlevel_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("service_name", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(TestException))]
		public void RunLevelServiceNameIsNotStringTest ()
		{
			RunLevelTest ft = new RunLevelTest ();
			ObjectElement oe = new ObjectElement ("runlevel_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("service_name", "tcp", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("runlevel", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void RunLevelRunLevelIsNotStringTest ()
		{
			RunLevelTest ft = new RunLevelTest ();
			ObjectElement oe = new ObjectElement ("runlevel_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("service_name", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("runlevel", "foo", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void RunLevelServiceNameInvalidOperationTest ()
		{
			RunLevelTest ft = new RunLevelTest ();
			ObjectElement oe = new ObjectElement ("runlevel_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("service_name", "tcp", Operations.BitwiseOr, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("runlevel", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;		

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(TestException))]
		public void RunLevelRunLevelInvalidOperationTest ()
		{
			RunLevelTest ft = new RunLevelTest ();
			ObjectElement oe = new ObjectElement ("runlevel_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("service_name", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("runlevel", "foo", Operations.BitwiseOr, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;	

			ft.Run (te, oe);
		}

		// sccs
		[Test]
		[ExpectedException(typeof(TestException))]
		public void SccsNoPathTest ()
		{
			SccsTest ft = new SccsTest ();
			ObjectElement oe = new ObjectElement ("sccs_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void SccsNoSccsnameTest ()
		{
			SccsTest ft = new SccsTest ();
			ObjectElement oe = new ObjectElement ("sccs_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(TestException))]
		public void SccsPathIsNotStringTest ()
		{
			SccsTest ft = new SccsTest ();
			ObjectElement oe = new ObjectElement ("sccs_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("sccsname", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void SccsSccsnameIsNotStringTest ()
		{
			SccsTest ft = new SccsTest ();
			ObjectElement oe = new ObjectElement ("sccs_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("sccsname", "foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void SccsPathInvalidOperationTest ()
		{
			SccsTest ft = new SccsTest ();
			ObjectElement oe = new ObjectElement ("sccs_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("sccsname", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			ft.Run (te, oe);
		}
		
		[Test]
		[ExpectedException(typeof(TestException))]
		public void SccsSccsnameInvalidOperationTest ()
		{
			SccsTest ft = new SccsTest ();
			ObjectElement oe = new ObjectElement ("sccs_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("path", "/tmp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			EntityObject eo2 = new EntityObject ("sccsname", "foo", Operations.BitwiseOr, OvalDataTypes.StringType, Checks.All);
			oe.Entities [eo2.Name] = eo2;			

			ft.Run (te, oe);
		}		

		// shadow
		[Test]
		[ExpectedException(typeof(TestException))]
		public void ShadowNoNameTest ()
		{
			ShadowTest ft = new ShadowTest ();
			ObjectElement oe = new ObjectElement ("shadow_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void ShadowNameIsNotStringTest ()
		{
			ShadowTest ft = new ShadowTest ();
			ObjectElement oe = new ObjectElement ("shadow_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("username", "rusty", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;
			
			ft.Run (te, oe);
		}

		[Test]
		[ExpectedException(typeof(TestException))]
		public void ShadowNameInvalidOperationTest ()
		{
			ShadowTest ft = new ShadowTest ();
			ObjectElement oe = new ObjectElement ("shadow_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

			EntityObject eo1 = new EntityObject ("username", "rusty", Operations.LessThan, OvalDataTypes.IntegerType, Checks.All);
			oe.Entities [eo1.Name] = eo1;			

			ft.Run (te, oe);
		}
#else		

        // access token
        [Test]
        [ExpectedException(typeof(TestException))]
        public void AccessTokenNoPrincipleTest()
        {
            AccessTokenTest ft = new AccessTokenTest();
            ObjectElement oe = new ObjectElement("accesstoken_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void AccessTokenNoPrincipleIsNotStringTest()
        {
            AccessTokenTest ft = new AccessTokenTest();
            ObjectElement oe = new ObjectElement("accesstoken_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("security_principle", "rusty", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        // activedirectory
        [Test]
        [ExpectedException(typeof(TestException))]
        public void ActiveDirectoryNoNamingContextTest()
        {
            ActiveDirectoryTest ft = new ActiveDirectoryTest();
            ObjectElement oe = new ObjectElement("activedirectory_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void ActiveDirectoryNoRelativeDNTest()
        {
            ActiveDirectoryTest ft = new ActiveDirectoryTest();
            ObjectElement oe = new ObjectElement("activedirectory_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("naming_context", "domain", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void ActiveDirectoryNoAttributeTest()
        {
            ActiveDirectoryTest ft = new ActiveDirectoryTest();
            ObjectElement oe = new ObjectElement("activedirectory_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("naming_context", "domain", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("relative_dn", "domain", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void ActiveDirectoryNamingContextIsNotStringTest()
        {
            ActiveDirectoryTest ft = new ActiveDirectoryTest();
            ObjectElement oe = new ObjectElement("activedirectory_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("naming_context", "domain", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("relative_dn", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("attribute", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void ActiveDirectoryRelativeDNIsNotStringTest()
        {
            ActiveDirectoryTest ft = new ActiveDirectoryTest();
            ObjectElement oe = new ObjectElement("activedirectory_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("naming_context", "domain", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("relative_dn", "foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("attribute", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void ActiveDirectoryAttributeIsNotStringTest()
        {
            ActiveDirectoryTest ft = new ActiveDirectoryTest();
            ObjectElement oe = new ObjectElement("activedirectory_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("naming_context", "domain", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("relative_dn", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("attribute", "bar", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void ActiveDirectoryNamingContextInvalidOperationTest()
        {
            ActiveDirectoryTest ft = new ActiveDirectoryTest();
            ObjectElement oe = new ObjectElement("activedirectory_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("naming_context", "domain", Operations.BitwiseAnd, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("relative_dn", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("attribute", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        // file audited permission
        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileAuditedPermissionsNoPathTest()
        {
            FileAuditedPermissionsTest ft = new FileAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("fileauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileAuditedPermissionsNoFilenameTest()
        {
            FileAuditedPermissionsTest ft = new FileAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("fileauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\temp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileAuditedPermissionsNoTrusteeNameTest()
        {
            FileAuditedPermissionsTest ft = new FileAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("fileauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\temp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileAuditedPermissionsPathIsNotStringTest()
        {
            FileAuditedPermissionsTest ft = new FileAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("fileauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\temp", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "rusty", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileAuditedPermissionsFilenameIsNotStringTest()
        {
            FileAuditedPermissionsTest ft = new FileAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("fileauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\temp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("filename", "foo", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "rusty", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileAuditedPermissionsTrusteeNameIsNotStringTest()
        {
            FileAuditedPermissionsTest ft = new FileAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("fileauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\temp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "rusty", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        // file effective rights
        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileEffectiveRightsNoPathTest()
        {
            FileEffectiveRightsTest ft = new FileEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("fileeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileEffectiveRightsNoFilenameTest()
        {
            FileEffectiveRightsTest ft = new FileEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("fileeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\temp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileEffectiveRightsNoTrusteeNameTest()
        {
            FileEffectiveRightsTest ft = new FileEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("fileeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\temp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileEffectiveRightsPathIsNotStringTest()
        {
            FileEffectiveRightsTest ft = new FileEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("fileeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\temp", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "rusty", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileEffectiveRightsFilenameIsNotStringTest()
        {
            FileEffectiveRightsTest ft = new FileEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("fileeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\temp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("filename", "foo", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "rusty", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileEffectiveRightsTrusteeNameIsNotStringTest()
        {
            FileEffectiveRightsTest ft = new FileEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("fileeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\temp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "rusty", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        // file
        [Test]
        [ExpectedException(typeof(NullReferenceException))]
        public void FileNoPathTest()
        {
            FileTest ft = new FileTest();
            ObjectElement oe = new ObjectElement("file_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(NullReferenceException))]
        public void FileNoFilenameTest()
        {
            FileTest ft = new FileTest();
            ObjectElement oe = new ObjectElement("file_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(NullReferenceException))]
        public void FilePathIsNotStringTest()
        {
            FileTest ft = new FileTest();
            ObjectElement oe = new ObjectElement("file_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileFilenameIsNotStringTest()
        {
            FileTest ft = new FileTest();
            ObjectElement oe = new ObjectElement("file_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("filename", "foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(NullReferenceException))]
        public void FilePathInvalidOperationTest()
        {
            FileTest ft = new FileTest();
            ObjectElement oe = new ObjectElement("file_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\foo", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("filename", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void FileFilenameInvalidOperationTest()
        {
            FileTest ft = new FileTest();
            ObjectElement oe = new ObjectElement("file_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("path", "c:\\foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("filename", "foo", Operations.BitwiseOr, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        // group
        [Test]
        [ExpectedException(typeof(TestException))]
        public void GroupNoGroupTest()
        {
            GroupTest ft = new GroupTest();
            ObjectElement oe = new ObjectElement("group_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void GroupGroupIsNotStringTest()
        {
            GroupTest ft = new GroupTest();
            ObjectElement oe = new ObjectElement("group_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("group", "foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void GroupGroupInvalidOperationTest()
        {
            GroupTest ft = new GroupTest();
            ObjectElement oe = new ObjectElement("group_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("group", "foo", Operations.BitwiseAnd, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        // interface
        [Test]
        [ExpectedException(typeof(TestException))]
        public void InterfaceNoNameTest()
        {
            InterfaceTest ft = new InterfaceTest();
            ObjectElement oe = new ObjectElement("interface_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void InterfaceNameIsNotStringTest()
        {
            InterfaceTest ft = new InterfaceTest();
            ObjectElement oe = new ObjectElement("interface_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("name", "foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void InterfaceNameInvalidOperationTest()
        {
            InterfaceTest ft = new InterfaceTest();
            ObjectElement oe = new ObjectElement("interface_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("name", "foo", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        // metabase
        [Test]
        [ExpectedException(typeof(TestException))]
        public void MetabaseNoKeyTest()
        {
            MetabaseTest ft = new MetabaseTest();
            ObjectElement oe = new ObjectElement("metabase_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void MetabaseNoIDTest()
        {
            MetabaseTest ft = new MetabaseTest();
            ObjectElement oe = new ObjectElement("metabase_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("key", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void MetabaseKeyIsNotStringTest()
        {
            MetabaseTest ft = new MetabaseTest();
            ObjectElement oe = new ObjectElement("metabase_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("key", "foo", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("id", "1234", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void MetabaseIDIsNotIntegerTest()
        {
            MetabaseTest ft = new MetabaseTest();
            ObjectElement oe = new ObjectElement("metabase_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("key", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("id", "1234", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void MetabaseKeyInvalidOperationTest()
        {
            MetabaseTest ft = new MetabaseTest();
            ObjectElement oe = new ObjectElement("metabase_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("key", "foo", Operations.BitwiseOr, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("id", "1234", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void MetabaseIDInvalidOperationTest()
        {
            MetabaseTest ft = new MetabaseTest();
            ObjectElement oe = new ObjectElement("metabase_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("key", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("id", "1234", Operations.PatternMatch, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        // port
        [Test]
        [ExpectedException(typeof(TestException))]
        public void PortNoLocalAddressTest()
        {
            PortTest ft = new PortTest();
            ObjectElement oe = new ObjectElement("port_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void PortNoLocalPortTest()
        {
            PortTest ft = new PortTest();
            ObjectElement oe = new ObjectElement("port_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("local_address", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void PortNoLocalProtocolTest()
        {
            PortTest ft = new PortTest();
            ObjectElement oe = new ObjectElement("port_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("local_address", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("local_port", "22", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void PortLocalAddressIsNotStringTest()
        {
            PortTest ft = new PortTest();
            ObjectElement oe = new ObjectElement("port_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("local_address", "foo", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("local_port", "22", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("local_protocol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void PortLocalPortIsNotIntegerTest()
        {
            PortTest ft = new PortTest();
            ObjectElement oe = new ObjectElement("port_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("local_address", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("local_port", "22", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("local_protocol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void PortLocalProtocolIsNotStringTest()
        {
            PortTest ft = new PortTest();
            ObjectElement oe = new ObjectElement("port_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("local_address", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("local_port", "22", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("local_protocol", "tcp", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void PortLocalAddressInvalidOperationTest()
        {
            PortTest ft = new PortTest();
            ObjectElement oe = new ObjectElement("port_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("local_address", "foo", Operations.BitwiseAnd, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("local_port", "22", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("local_protocol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void PortLocalPortInvalidOperationTest()
        {
            PortTest ft = new PortTest();
            ObjectElement oe = new ObjectElement("port_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("local_address", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("local_port", "22", Operations.PatternMatch, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("local_protocol", "tcp", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void PortLocalProtocolInvalidOperationTest()
        {
            PortTest ft = new PortTest();
            ObjectElement oe = new ObjectElement("port_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("local_address", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("local_port", "22", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("local_protocol", "tcp", Operations.BitwiseOr, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void PortLocalProtocolInvalidValueTest()
        {
            PortTest ft = new PortTest();
            ObjectElement oe = new ObjectElement("port_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("local_address", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("local_port", "22", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("local_protocol", "invalid", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        // process
        [Test]
        [ExpectedException(typeof(TestException))]
        public void ProcessNoCmdTest()
        {
            ProcessTest ft = new ProcessTest();
            ObjectElement oe = new ObjectElement("process_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void ProcessCmdIsNotStringTest()
        {
            ProcessTest ft = new ProcessTest();
            ObjectElement oe = new ObjectElement("process_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("cmd", "foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void ProcessCmdInvalidOperationTest()
        {
            ProcessTest ft = new ProcessTest();
            ObjectElement oe = new ObjectElement("process_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("cmd", "foo", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        // registry 
        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegistryNoHiveTest()
        {
            RegistryTest ft = new RegistryTest();
            ObjectElement oe = new ObjectElement("registry_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegistryNoKeyTest()
        {
            RegistryTest ft = new RegistryTest();
            ObjectElement oe = new ObjectElement("registry_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegistryNoNameTest()
        {
            RegistryTest ft = new RegistryTest();
            ObjectElement oe = new ObjectElement("registry_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegistryHiveIsNotStringTest()
        {
            RegistryTest ft = new RegistryTest();
            ObjectElement oe = new ObjectElement("registry_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("name", "whatever", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegistryKeyIsNotStringTest()
        {
            RegistryTest ft = new RegistryTest();
            ObjectElement oe = new ObjectElement("registry_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("name", "whatever", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegistryNameIsNotStringTest()
        {
            RegistryTest ft = new RegistryTest();
            ObjectElement oe = new ObjectElement("registry_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("name", "whatever", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegistryHiveInvalidOperationTest()
        {
            RegistryTest ft = new RegistryTest();
            ObjectElement oe = new ObjectElement("registry_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("name", "whatever", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;
            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegistryKeyInvalidOperationTest()
        {
            RegistryTest ft = new RegistryTest();
            ObjectElement oe = new ObjectElement("registry_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("name", "whatever", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegistryNameInvalidOperationTest()
        {
            RegistryTest ft = new RegistryTest();
            ObjectElement oe = new ObjectElement("registry_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("name", "whatever", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        // regkeyauditedpermissions 
        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyAuditedPermissionsNoHiveTest()
        {
            RegKeyAuditedPermissionsTest ft = new RegKeyAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("regkeyauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyAuditedPermissionsNoKeyTest()
        {
            RegKeyAuditedPermissionsTest ft = new RegKeyAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("regkeyauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyAuditedPermissionsNoNameTest()
        {
            RegKeyAuditedPermissionsTest ft = new RegKeyAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("regkeyauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyAuditedPermissionsHiveIsNotStringTest()
        {
            RegKeyAuditedPermissionsTest ft = new RegKeyAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("regkeyauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "whatever", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyAuditedPermissionsKeyIsNotStringTest()
        {
            RegKeyAuditedPermissionsTest ft = new RegKeyAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("regkeyauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "whatever", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyAuditedPermissionsNameIsNotStringTest()
        {
            RegKeyAuditedPermissionsTest ft = new RegKeyAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("regkeyauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "whatever", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyAuditedPermissionsHiveInvalidOperationTest()
        {
            RegKeyAuditedPermissionsTest ft = new RegKeyAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("regkeyauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "whatever", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;
            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyAuditedPermissionsKeyInvalidOperationTest()
        {
            RegKeyAuditedPermissionsTest ft = new RegKeyAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("regkeyauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "whatever", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyAuditedPermissionsNameInvalidOperationTest()
        {
            RegKeyAuditedPermissionsTest ft = new RegKeyAuditedPermissionsTest();
            ObjectElement oe = new ObjectElement("regkeyauditedpermissions_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "whatever", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        // registry effective rights
        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyEffectiveRightsNoHiveTest()
        {
            RegKeyEffectiveRightsTest ft = new RegKeyEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("regkeyeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyEffectiveRightsNoKeyTest()
        {
            RegKeyEffectiveRightsTest ft = new RegKeyEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("regkeyeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyEffectiveRightsNoNameTest()
        {
            RegKeyEffectiveRightsTest ft = new RegKeyEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("regkeyeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyEffectiveRightsHiveIsNotStringTest()
        {
            RegKeyEffectiveRightsTest ft = new RegKeyEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("regkeyeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "whatever", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyEffectiveRightsKeyIsNotStringTest()
        {
            RegKeyEffectiveRightsTest ft = new RegKeyEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("regkeyeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "whatever", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyEffectiveRightsNameIsNotStringTest()
        {
            RegKeyEffectiveRightsTest ft = new RegKeyEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("regkeyeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "whatever", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyEffectiveRightsHiveInvalidOperationTest()
        {
            RegKeyEffectiveRightsTest ft = new RegKeyEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("regkeyeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.PatternMatch, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "whatever", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;
            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyEffectiveRightsKeyInvalidOperationTest()
        {
            RegKeyEffectiveRightsTest ft = new RegKeyEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("regkeyeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "whatever", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void RegKeyEffectiveRightsNameInvalidOperationTest()
        {
            RegKeyEffectiveRightsTest ft = new RegKeyEffectiveRightsTest();
            ObjectElement oe = new ObjectElement("regkeyeffectiverights_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("hive", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("key", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            EntityObject eo3 = new EntityObject("trustee_name", "whatever", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo3.Name] = eo3;

            ft.Run(te, oe);
        }

        // sid
        [Test]
        [ExpectedException(typeof(TestException))]
        public void SidNoTrusteeNameTest()
        {
            SidTest ft = new SidTest();
            ObjectElement oe = new ObjectElement("sid_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void SidTrusteeNameIsNotStringTest()
        {
            SidTest ft = new SidTest();
            ObjectElement oe = new ObjectElement("sid_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("trustee_name", "foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void SidTrusteeNameInvalidOperationTest()
        {
            SidTest ft = new SidTest();
            ObjectElement oe = new ObjectElement("sid_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("trustee_name", "foo", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }


        // user
        [Test]
        [ExpectedException(typeof(TestException))]
        public void UserNoUserTest()
        {
            UserTest ft = new UserTest();
            ObjectElement oe = new ObjectElement("user_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void UserUserIsNotStringTest()
        {
            UserTest ft = new UserTest();
            ObjectElement oe = new ObjectElement("user_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("user", "foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void UserUserInvalidOperationTest()
        {
            UserTest ft = new UserTest();
            ObjectElement oe = new ObjectElement("user_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("user", "foo", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        // volume
        [Test]
        [ExpectedException(typeof(TestException))]
        public void VolumeNoRootPathTest()
        {
            VolumeTest ft = new VolumeTest();
            ObjectElement oe = new ObjectElement("volume_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void VolumeRootPathIsNotStringTest()
        {
            VolumeTest ft = new VolumeTest();
            ObjectElement oe = new ObjectElement("volume_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("rootpath", "foo", Operations.Equals, OvalDataTypes.IntegerType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void VolumeRootPathInvalidOperationTest()
        {
            VolumeTest ft = new VolumeTest();
            ObjectElement oe = new ObjectElement("volume_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("rootpath", "foo", Operations.LessThan, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        // wmi
        [Test]
        [ExpectedException(typeof(TestException))]
        public void WMINoNamespaceTest()
        {
            WMITest ft = new WMITest();
            ObjectElement oe = new ObjectElement("wmi_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void WMINoWqlTest()
        {
            WMITest ft = new WMITest();
            ObjectElement oe = new ObjectElement("wmi_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("namespace", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void WMINamespaceIsNotStringTest()
        {
            WMITest ft = new WMITest();
            ObjectElement oe = new ObjectElement("wmi_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("namespace", "foo", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("wql", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void WMIWqlIsNotStringTest()
        {
            WMITest ft = new WMITest();
            ObjectElement oe = new ObjectElement("wmi_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("namespace", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("wql", "bar", Operations.Equals, OvalDataTypes.BooleanType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void WMINamespaceInvalidOperationTest()
        {
            WMITest ft = new WMITest();
            ObjectElement oe = new ObjectElement("wmi_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("namespace", "foo", Operations.BitwiseOr, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("wql", "bar", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

        [Test]
        [ExpectedException(typeof(TestException))]
        public void WMIWqlInvalidOperationTest()
        {
            WMITest ft = new WMITest();
            ObjectElement oe = new ObjectElement("wmi_object", "oval:com.mmgsecurity.oval:obj:1", "1", "");

            EntityObject eo1 = new EntityObject("namespace", "foo", Operations.Equals, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo1.Name] = eo1;

            EntityObject eo2 = new EntityObject("wql", "bar", Operations.BitwiseAnd, OvalDataTypes.StringType, Checks.All);
            oe.Entities[eo2.Name] = eo2;

            ft.Run(te, oe);
        }

#endif
		
	}
}
